@@ -1,8 +1,8 @@
 /*****************************************************************************
- * beos_specific.h: BeOS specific features 
+ * beos_specific.h: BeOS specific features
  *****************************************************************************
  * Copyright (C) 1999, 2000 VideoLAN
- * $Id: beos_specific.h,v 1.5 2001/04/12 01:52:45 sam Exp $
+ * $Id: beos_specific.h 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Jean-Marc Dressler <polux@via.ecp.fr>
  *
@@ -10,7 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -29,10 +29,6 @@
 extern "C" {
 #endif
 
-void    system_Create ( int *pi_argc, char *ppsz_argv[], char *ppsz_env[] );
-void    system_Destroy( void );
-char  * system_GetProgramPath( void );
-
 #ifdef __cplusplus
 }
 #endif
