@@ -2,7 +2,7 @@
  * modules.h : Module management functions.
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: modules.h,v 1.63 2003/01/19 03:16:24 sam Exp $
+ * $Id: modules.h 7857 2004-06-01 15:45:07Z gbazin $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -27,13 +27,22 @@
 
 /* Number of tries before we unload an unused module */
 #define MODULE_HIDE_DELAY 50
-#define MODULE_SHORTCUT_MAX 10
+#define MODULE_SHORTCUT_MAX 50
 
 /* The module handle type. */
-#ifdef SYS_BEOS
-typedef int     module_handle_t;
-#else
-typedef void *  module_handle_t;
+#if defined(HAVE_DL_DYLD)
+#   if defined (HAVE_MACH_O_DYLD_H)
+#       include <mach-o/dyld.h>
+#   endif
+typedef NSModule module_handle_t;
+#elif defined(HAVE_IMAGE_H)
+typedef int module_handle_t;
+#elif defined(WIN32) || defined(UNDER_CE)
+typedef void * module_handle_t;
+#elif defined(HAVE_DL_DLOPEN)
+typedef void * module_handle_t;
+#elif defined(HAVE_DL_SHL_LOAD)
+typedef shl_t module_handle_t;
 #endif
 
 /*****************************************************************************
@@ -45,7 +54,19 @@ struct module_bank_t
 {
     VLC_COMMON_MEMBERS
 
+    int              i_usage;
     module_symbols_t symbols;
+
+    /* Plugins cache */
+    vlc_bool_t     b_cache;
+    vlc_bool_t     b_cache_dirty;
+    vlc_bool_t     b_cache_delete;
+
+    int            i_cache;
+    module_cache_t **pp_cache;
+
+    int            i_loaded_cache;
+    module_cache_t **pp_loaded_cache;
 };
 
 /*****************************************************************************
@@ -58,6 +79,7 @@ struct module_t
     /*
      * Variables set by the module to identify itself
      */
+    char *psz_shortname;                                      /* Module name */
     char *psz_longname;                           /* Module descriptive name */
 
     /*
@@ -94,6 +116,7 @@ struct module_t
     char *              psz_filename;                     /* Module filename */
 
     vlc_bool_t          b_builtin;  /* Set to true if the module is built in */
+    vlc_bool_t          b_loaded;        /* Set to true if the dll is loaded */
 
     /*
      * Symbol table we send to the module so that it can access vlc symbols
@@ -101,6 +124,21 @@ struct module_t
     module_symbols_t *p_symbols;
 };
 
+/*****************************************************************************
+ * Module cache description structure
+ *****************************************************************************/
+struct module_cache_t
+{
+    /* Mandatory cache entry header */
+    char       *psz_file;
+    int64_t    i_time;
+    int64_t    i_size;
+    vlc_bool_t b_junk;
+
+    /* Optional extra data */
+    module_t *p_module;
+};
+
 /*****************************************************************************
  * Exported functions.
  *****************************************************************************/
@@ -117,8 +155,8 @@ void  __module_EndBank         ( vlc_object_t * );
 #define module_ResetBank(a)    __module_ResetBank(VLC_OBJECT(a))
 void  __module_ResetBank       ( vlc_object_t * );
 
-#define module_Need(a,b,c) __module_Need(VLC_OBJECT(a),b,c)
-VLC_EXPORT( module_t *, __module_Need, ( vlc_object_t *, const char *, const char * ) );
+#define module_Need(a,b,c,d) __module_Need(VLC_OBJECT(a),b,c,d)
+VLC_EXPORT( module_t *, __module_Need, ( vlc_object_t *, const char *, const char *, vlc_bool_t ) );
 #define module_Unneed(a,b) __module_Unneed(VLC_OBJECT(a),b)
 VLC_EXPORT( void, __module_Unneed, ( vlc_object_t *, module_t * ) );
 
