@@ -2,7 +2,7 @@
  * osd.h : Constants for use with osd modules
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: osd.h,v 1.7 2004/02/15 18:22:26 sigmunau Exp $
+ * $Id: osd.h 8354 2004-08-03 00:11:32Z gbazin $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -31,6 +31,14 @@
 #define OSD_ALIGN_TOP 0x4
 #define OSD_ALIGN_BOTTOM 0x8
 
+#define OSD_HOR_SLIDER 1
+#define OSD_VERT_SLIDER 2
+
+#define OSD_PLAY_ICON 1
+#define OSD_PAUSE_ICON 2
+#define OSD_SPEAKER_ICON 3
+#define OSD_MUTE_ICON 4
+
 /**
  * Text style information.
  * This struct is currently ignored
@@ -45,14 +53,16 @@ struct text_style_t
 };
 static const text_style_t default_text_style = { 22, 0xffffff, VLC_FALSE, VLC_FALSE, VLC_FALSE };
 
-VLC_EXPORT( subpicture_t *, vout_ShowTextRelative, ( vout_thread_t *, char *, text_style_t *, int, int, int, mtime_t ) );
-VLC_EXPORT( void,  vout_ShowTextAbsolute, ( vout_thread_t *, char *, text_style_t *, int, int, int, mtime_t, mtime_t ) );
-VLC_EXPORT( void,  __vout_OSDMessage, ( vlc_object_t *, char *, ... ) );
+VLC_EXPORT( int, vout_ShowTextRelative, ( vout_thread_t *, int, char *, text_style_t *, int, int, int, mtime_t ) );
+VLC_EXPORT( int, vout_ShowTextAbsolute, ( vout_thread_t *, int, char *, text_style_t *, int, int, int, mtime_t, mtime_t ) );
+VLC_EXPORT( void,  __vout_OSDMessage, ( vlc_object_t *, int, char *, ... ) );
 /**
  * Same as __vlc_OSDMessage() but with automatic casting
  */
 #if defined(HAVE_VARIADIC_MACROS)
-#    define vout_OSDMessage( obj, fmt, args...) __vout_OSDMessage( VLC_OBJECT(obj), fmt, ## args )
+#    define vout_OSDMessage( obj, chan, fmt, args...) __vout_OSDMessage( VLC_OBJECT(obj), chan, fmt, ## args )
 #else
 #    define vout_OSDMessage __vout_OSDMessage
 #endif
+VLC_EXPORT( void, vout_OSDSlider, ( vlc_object_t *, int, int , short ) );
+VLC_EXPORT( void, vout_OSDIcon, ( vlc_object_t *, int, short ) );
