@@ -2,7 +2,7 @@
  * variables.h: variables handling
  *****************************************************************************
  * Copyright (C) 2002-2004 VideoLAN
- * $Id: variables.h,v 1.22 2004/01/25 18:17:08 zorglub Exp $
+ * $Id: variables.h 7997 2004-06-18 11:35:45Z sigmunau $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -250,6 +250,20 @@ static inline int __var_SetInteger( vlc_object_t *p_obj, const char *psz_name, i
     return __var_Set( p_obj, psz_name, val );
 }
 
+/**
+ * Set the value of an boolean variable
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ * \param b The new boolean value of this variable
+ */
+static inline int __var_SetBool( vlc_object_t *p_obj, const char *psz_name, vlc_bool_t b )
+{
+    vlc_value_t val;
+    val.b_bool = b;
+    return __var_Set( p_obj, psz_name, val );
+}
+
 /**
  * Set the value of a time variable
  *
@@ -278,6 +292,20 @@ static inline int __var_SetFloat( vlc_object_t *p_obj, const char *psz_name, flo
     return __var_Set( p_obj, psz_name, val );
 }
 
+/**
+ * Set the value of a string variable
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ * \param psz_string The new string value of this variable
+ */
+static inline int __var_SetString( vlc_object_t *p_obj, const char *psz_name, char *psz_string )
+{
+    vlc_value_t val;
+    val.psz_string = psz_string;
+    return __var_Set( p_obj, psz_name, val );
+}
+
 /**
  * Trigger the callbacks on a void variable
  *
@@ -295,6 +323,11 @@ static inline int __var_SetVoid( vlc_object_t *p_obj, const char *psz_name )
  * __var_SetInteger() with automatic casting
  */
 #define var_SetInteger(a,b,c)   __var_SetInteger( VLC_OBJECT(a),b,c)
+/**
+ * __var_SetBool() with automatic casting
+ */
+#define var_SetBool(a,b,c)   __var_SetBool( VLC_OBJECT(a),b,c)
+
 /**
  * __var_SetTime() with automatic casting
  */
@@ -303,11 +336,218 @@ static inline int __var_SetVoid( vlc_object_t *p_obj, const char *psz_name )
  * __var_SetFloat() with automatic casting
  */
 #define var_SetFloat(a,b,c)     __var_SetFloat( VLC_OBJECT(a),b,c)
+/**
+ * __var_SetString() with automatic casting
+ */
+#define var_SetString(a,b,c)     __var_SetString( VLC_OBJECT(a),b,c)
 /**
  * __var_SetVoid() with automatic casting
  */
 #define var_SetVoid(a,b)        __var_SetVoid( VLC_OBJECT(a),b)
 
+/**
+ * Get a integer value
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline int __var_GetInteger( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.i_int;
+    else
+        return 0;
+}
+
+/**
+ * Get a boolean value
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline int __var_GetBool( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.b_bool;
+    else
+        return VLC_FALSE;
+}
+
+/**
+ * Get a time value
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline int64_t __var_GetTime( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.i_time;
+    else
+        return 0;
+}
+
+/**
+ * Get a float value
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline float __var_GetFloat( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.f_float;
+    else
+        return 0.0;
+}
+
+/**
+ * Get a string value
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline char *__var_GetString( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.psz_string;
+    else
+        return strdup( "" );
+}
+
+/**
+ * __var_GetInteger() with automatic casting
+ */
+#define var_GetInteger(a,b)   __var_GetInteger( VLC_OBJECT(a),b)
+/**
+ * __var_GetBool() with automatic casting
+ */
+#define var_GetBool(a,b)   __var_GetBool( VLC_OBJECT(a),b)
+/**
+ * __var_GetTime() with automatic casting
+ */
+#define var_GetTime(a,b)   __var_GetTime( VLC_OBJECT(a),b)
+/**
+ * __var_GetFloat() with automatic casting
+ */
+#define var_GetFloat(a,b)   __var_GetFloat( VLC_OBJECT(a),b)
+/**
+ * __var_GetString() with automatic casting
+ */
+#define var_GetString(a,b)   __var_GetString( VLC_OBJECT(a),b)
+
+
+/**
+ * Create a integer variable with inherit and get its value.
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline int __var_CreateGetInteger( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+
+    __var_Create( p_obj, psz_name, VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.i_int;
+    else
+        return 0;
+}
+
+/**
+ * Create a boolean variable with inherit and get its value.
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline int __var_CreateGetBool( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+
+    __var_Create( p_obj, psz_name, VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.b_bool;
+    else
+        return VLC_FALSE;
+}
+
+/**
+ * Create a time variable with inherit and get its value.
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline int64_t __var_CreateGetTime( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+
+    __var_Create( p_obj, psz_name, VLC_VAR_TIME | VLC_VAR_DOINHERIT );
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.i_time;
+    else
+        return 0;
+}
+
+/**
+ * Create a float variable with inherit and get its value.
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline float __var_CreateGetFloat( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+
+    __var_Create( p_obj, psz_name, VLC_VAR_FLOAT | VLC_VAR_DOINHERIT );
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.f_float;
+    else
+        return 0.0;
+}
+
+/**
+ * Create a string variable with inherit and get its value.
+ *
+ * \param p_obj The object that holds the variable
+ * \param psz_name The name of the variable
+ */
+static inline char *__var_CreateGetString( vlc_object_t *p_obj, const char *psz_name )
+{
+    vlc_value_t val;
+
+    __var_Create( p_obj, psz_name, VLC_VAR_STRING | VLC_VAR_DOINHERIT );
+    if( !__var_Get( p_obj, psz_name, &val ) )
+        return val.psz_string;
+    else
+        return strdup( "" );
+}
+
+/**
+ * __var_CreateGetInteger() with automatic casting
+ */
+#define var_CreateGetInteger(a,b)   __var_CreateGetInteger( VLC_OBJECT(a),b)
+/**
+ * __var_CreateGetBool() with automatic casting
+ */
+#define var_CreateGetBool(a,b)   __var_CreateGetBool( VLC_OBJECT(a),b)
+/**
+ * __var_CreateGetTime() with automatic casting
+ */
+#define var_CreateGetTime(a,b)   __var_CreateGetTime( VLC_OBJECT(a),b)
+/**
+ * __var_CreateGetFloat() with automatic casting
+ */
+#define var_CreateGetFloat(a,b)   __var_CreateGetFloat( VLC_OBJECT(a),b)
+/**
+ * __var_CreateGetString() with automatic casting
+ */
+#define var_CreateGetString(a,b)   __var_CreateGetString( VLC_OBJECT(a),b)
+
 /**
  * @}
  */
