@@ -2,7 +2,7 @@
  * hotkeys.h: keycode defines
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: vlc_keys.h,v 1.13 2004/01/25 18:17:08 zorglub Exp $
+ * $Id: vlc_keys.h 8966 2004-10-10 10:08:44Z ipkiss $
  *
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
  *
@@ -55,6 +55,8 @@
 #define KEY_PAGEDOWN         0x00180000
 #define KEY_TAB              0x00190000
 #define KEY_BACKSPACE        0x001A0000
+#define KEY_MOUSEWHEELUP     0x001B0000
+#define KEY_MOUSEWHEELDOWN   0x001C0000
 
 #define KEY_ASCII            0x0000007F
 #define KEY_UNSET            0
@@ -105,6 +107,8 @@ static const struct key_descriptor_s vlc_keys[] =
     { "Page Down", KEY_PAGEDOWN },
     { "Tab", KEY_TAB },
     { "Backspace", KEY_BACKSPACE },
+    { "Mouse Wheel Up", KEY_MOUSEWHEELUP },
+    { "Mouse Wheel Down", KEY_MOUSEWHEELDOWN },
     { "a", 'a' },
     { "b", 'b' },
     { "c", 'c' },
@@ -222,3 +226,11 @@ static inline int StringToKey( char *psz_key )
 #define ACTIONID_PLAY_BOOKMARK9        44
 #define ACTIONID_PLAY_BOOKMARK10       45
 /* end of contiguous zone */
+#define ACTIONID_SUBDELAY_UP           46
+#define ACTIONID_SUBDELAY_DOWN         47
+#define ACTIONID_HISTORY_BACK          48
+#define ACTIONID_HISTORY_FORWARD       49
+#define ACTIONID_AUDIO_TRACK           50
+#define ACTIONID_SUBTITLE_TRACK        51
+#define ACTIONID_INTF_SHOW             52
+
