@@ -2,7 +2,7 @@
  * dshow.cpp : DirectShow access module for vlc
  *****************************************************************************
  * Copyright (C) 2002, 2003 VideoLAN
- * $Id: dshow.cpp 9097 2004-11-01 13:10:14Z gbazin $
+ * $Id: dshow.cpp 9244 2004-11-08 12:24:32Z gbazin $
  *
  * Author: Gildas Bazin <gbazin@videolan.org>
  *
@@ -306,7 +306,7 @@ static int CommonOpen( vlc_object_t *p_this, access_sys_t *p_sys,
 
     /* Initialize some data */
     p_sys->i_streams = 0;
-    p_sys->pp_streams = (dshow_stream_t **)malloc( 1 );
+    p_sys->pp_streams = 0;
     p_sys->i_width = i_width;
     p_sys->i_height = i_height;
     p_sys->i_chroma = i_chroma;
@@ -315,6 +315,9 @@ static int CommonOpen( vlc_object_t *p_this, access_sys_t *p_sys,
     p_sys->p_capture_graph_builder2 = NULL;
     p_sys->p_control = NULL;
 
+    vlc_mutex_init( p_this, &p_sys->lock );
+    vlc_cond_init( p_this, &p_sys->wait );
+
     /* Build directshow graph */
     CreateDirectShowGraph( p_sys );
 
@@ -338,10 +341,25 @@ static int CommonOpen( vlc_object_t *p_this, access_sys_t *p_sys,
                 p_stream->i_fourcc == VLC_FOURCC('m','p','2','v') )
             {
                 b_audio = VLC_FALSE;
+
+                if( b_access_demux )
+                {
+                    /* Let the access (only) take care of that */
+                  return VLC_EGENERIC;
+                }
             }
         }
 
-        if( p_stream->mt.majortype == MEDIATYPE_Stream ) b_audio = VLC_FALSE;
+        if( p_stream->mt.majortype == MEDIATYPE_Stream )
+        {
+            b_audio = VLC_FALSE;
+
+            if( b_access_demux )
+            {
+                /* Let the access (only) take care of that */
+                return VLC_EGENERIC;
+            }
+        }
     }
 
     if( b_audio && OpenDevice( p_this, p_sys, adevname, 1 ) != VLC_SUCCESS )
@@ -398,8 +416,6 @@ static int CommonOpen( vlc_object_t *p_this, access_sys_t *p_sys,
 
     /* Initialize some data */
     p_sys->i_current_stream = 0;
-    vlc_mutex_init( p_this, &p_sys->lock );
-    vlc_cond_init( p_this, &p_sys->wait );
 
     if( !p_sys->i_streams ) return VLC_EGENERIC;
 
@@ -493,7 +509,7 @@ static int AccessOpen( vlc_object_t *p_this )
     p_access->p_sys = p_sys = (access_sys_t *)malloc( sizeof( access_sys_t ) );
     memset( p_sys, 0, sizeof( access_sys_t ) );
 
-    if( CommonOpen( p_this, p_sys, VLC_TRUE ) != VLC_SUCCESS )
+    if( CommonOpen( p_this, p_sys, VLC_FALSE ) != VLC_SUCCESS )
     {
         CommonClose( p_this, p_sys );
         return VLC_EGENERIC;
@@ -549,11 +565,8 @@ static void CommonClose( vlc_object_t *p_this, access_sys_t *p_sys )
     CoUninitialize();
 
     /* Remove filters from graph */
-    for( int i = 0; i < p_sys->i_streams; i++ )
-    {
-        delete p_sys->pp_streams[i];
-    }
-    free( p_sys->pp_streams );
+    for( int i = 0; i < p_sys->i_streams; i++ ) delete p_sys->pp_streams[i];
+    if( p_sys->i_streams ) free( p_sys->pp_streams );
 
     vlc_mutex_destroy( &p_sys->lock );
     vlc_cond_destroy( &p_sys->wait );
@@ -612,7 +625,8 @@ static bool ConnectFilters( vlc_object_t *p_this, access_sys_t *p_sys,
         // Sort out all the possible video inputs
         // The class needs to be given the capture filters ANALOGVIDEO input pin
         IEnumPins *pins = 0;
-        if( mediaType.majortype == MEDIATYPE_Video &&
+        if( ( mediaType.majortype == MEDIATYPE_Video ||
+              mediaType.majortype == MEDIATYPE_Stream ) &&
             SUCCEEDED(p_filter->EnumPins(&pins)) )
         {
             IPin        *pP = 0;
@@ -764,7 +778,7 @@ static int OpenDevice( vlc_object_t *p_this, access_sys_t *p_sys,
      * won't add a prefered media type as this doesn't seem to work well
      * -- to investigate. */
     vlc_bool_t b_stream_type = VLC_FALSE;
-    for( int i = 0; i < media_count; i++ )
+    for( size_t i = 0; i < media_count; i++ )
     {
         if( media_types[i].majortype == MEDIATYPE_Stream )
         {
