@@ -2,7 +2,7 @@
  * cdrom.c: cdrom tools
  *****************************************************************************
  * Copyright (C) 1998-2001 VideoLAN
- * $Id: cdrom.c,v 1.8 2003/03/10 00:12:53 gbazin Exp $
+ * $Id: cdrom.c 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Johan Bilien <jobi@via.ecp.fr>
  *          Gildas Bazin <gbazin@netcourrier.com>
@@ -67,8 +67,8 @@
 #   include <linux/cdrom.h>
 #endif
 
+#include "cdrom_internals.h"
 #include "cdrom.h"
-#include "vcd.h"
 
 /*****************************************************************************
  * ioctl_Open: Opens a VCD device or file and returns an opaque handle
@@ -101,8 +101,7 @@ vcddev_t *ioctl_Open( vlc_object_t *p_this, const char *psz_dev )
      *  Check if we are dealing with a device or a file (vcd image)
      */
 #ifdef WIN32
-    if( strlen( psz_dev ) == 1 ||
-        (strlen( psz_dev ) == 2 && psz_dev[1] == ':') )
+    if( (strlen( psz_dev ) == 2 && psz_dev[1] == ':') )
     {
         b_is_file = 0;
     }
@@ -542,12 +541,16 @@ int ioctl_GetTracksMap( vlc_object_t *p_this, const vcddev_t *p_vcddev,
 }
 
 /****************************************************************************
- * ioctl_ReadSector: Read a sector (2324 bytes)
+ * ioctl_ReadSector: Read VCD or CDDA sectors
  ****************************************************************************/
-int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
-                      int i_sector, byte_t * p_buffer )
+int ioctl_ReadSectors( vlc_object_t *p_this, const vcddev_t *p_vcddev,
+                       int i_sector, byte_t * p_buffer, int i_nb, int i_type )
 {
-    byte_t p_block[ VCD_SECTOR_SIZE ];
+    byte_t *p_block;
+    int i;
+
+    if( i_type == VCD_TYPE ) p_block = malloc( VCD_SECTOR_SIZE * i_nb );
+    else p_block = p_buffer;
 
     if( p_vcddev->i_vcdimage_handle != -1 )
     {
@@ -558,20 +561,18 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
                    SEEK_SET ) == -1 )
         {
             msg_Err( p_this, "Could not lseek to sector %d", i_sector );
+            if( i_type == VCD_TYPE ) free( p_block );
             return -1;
         }
 
-        if( read( p_vcddev->i_vcdimage_handle, p_block, VCD_SECTOR_SIZE )
+        if( read( p_vcddev->i_vcdimage_handle, p_block, VCD_SECTOR_SIZE * i_nb)
             == -1 )
         {
-            // msg_Err( p_this, "Could not read sector %d", i_sector );
+            msg_Err( p_this, "Could not read sector %d", i_sector );
+            if( i_type == VCD_TYPE ) free( p_block );
             return -1;
         }
 
-        /* We don't want to keep the header of the read sector */
-        memcpy( p_buffer, p_block + VCD_DATA_START, VCD_DATA_SIZE );
-
-        return 0;
     }
     else
     {
@@ -592,11 +593,12 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
         cd_read.sectorType = kCDSectorTypeUnknown;
 
         cd_read.buffer = p_block;
-        cd_read.bufferLength = sizeof(p_block);
+        cd_read.bufferLength = VCD_SECTOR_SIZE * i_nb;
 
         if( ioctl( p_vcddev->i_device_handle, DKIOCCDREAD, &cd_read ) == -1 )
         {
             msg_Err( p_this, "could not read block %d", i_sector );
+            if( i_type == VCD_TYPE ) free( p_block );
             return -1;
         }
 
@@ -610,6 +612,7 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
             hEvent = CreateEvent( NULL, TRUE, FALSE, NULL );
             if( hEvent == NULL )
             {
+                if( i_type == VCD_TYPE ) free( p_block );
                 return -1;
             }
 
@@ -627,6 +630,10 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
             /* Operation code */
             ssc.CDBByte[ 0 ] = READ_CD;
 
+            /* Sector type */
+            ssc.CDBByte[ 1 ] = i_type == VCD_TYPE ? SECTOR_TYPE_MODE2_FORM2 :
+                                                    SECTOR_TYPE_CDDA;
+
             /* Start of LBA */
             ssc.CDBByte[ 2 ] = ( i_sector >> 24 ) & 0xff;
             ssc.CDBByte[ 3 ] = ( i_sector >> 16 ) & 0xff;
@@ -634,16 +641,17 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
             ssc.CDBByte[ 5 ] = ( i_sector       ) & 0xff;
 
             /* Transfer length */
-            ssc.CDBByte[ 6 ] = 0;
-            ssc.CDBByte[ 7 ] = 0;
-            ssc.CDBByte[ 8 ] = 1;
+            ssc.CDBByte[ 6 ] = ( i_nb >> 16 ) & 0xff;
+            ssc.CDBByte[ 7 ] = ( i_nb >> 8  ) & 0xff;
+            ssc.CDBByte[ 8 ] = ( i_nb       ) & 0xff;
 
             /* Data selection */
-            ssc.CDBByte[ 9 ] = READ_CD_USERDATA_MODE2;
+            ssc.CDBByte[ 9 ] = i_type == VCD_TYPE ? READ_CD_RAW_MODE2 :
+                                                    READ_CD_USERDATA;
 
             /* Result buffer */
             ssc.SRB_BufPointer  = p_block;
-            ssc.SRB_BufLen = VCD_SECTOR_SIZE;
+            ssc.SRB_BufLen = VCD_SECTOR_SIZE * i_nb;
 
             /* Initiate transfer */
             ResetEvent( hEvent );
@@ -660,12 +668,9 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
             /* check that the transfer went as planned */
             if( ssc.SRB_Status != SS_COMP )
             {
+                if( i_type == VCD_TYPE ) free( p_block );
                 return -1;
             }
-
-            /* We don't want to keep the footer of the read sector */
-            memcpy( p_buffer, p_block, VCD_DATA_SIZE );
-            return 0;
         }
         else
         {
@@ -674,54 +679,53 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
 
             /* Initialize CDROM_RAW_READ structure */
             cdrom_raw.DiskOffset.QuadPart = CD_SECTOR_SIZE * i_sector;
-            cdrom_raw.SectorCount = 1;
-            cdrom_raw.TrackMode = XAForm2;
+            cdrom_raw.SectorCount = i_nb;
+            cdrom_raw.TrackMode =  i_type == VCD_TYPE ? XAForm2 : CDDA;
 
             if( DeviceIoControl( p_vcddev->h_device_handle,
                                  IOCTL_CDROM_RAW_READ, &cdrom_raw,
                                  sizeof(RAW_READ_INFO), p_block,
-                                 sizeof(p_block), &dwBytesReturned, NULL )
-                == 0 )
+                                 VCD_SECTOR_SIZE * i_nb, &dwBytesReturned,
+                                 NULL ) == 0 )
             {
-                return -1;
+                if( i_type == VCD_TYPE )
+                {
+                    /* Retry in YellowMode2 */
+                    cdrom_raw.TrackMode = YellowMode2;
+                    if( DeviceIoControl( p_vcddev->h_device_handle,
+                                 IOCTL_CDROM_RAW_READ, &cdrom_raw,
+                                 sizeof(RAW_READ_INFO), p_block,
+                                 VCD_SECTOR_SIZE * i_nb, &dwBytesReturned,
+                                 NULL ) == 0 )
+                    {
+                        free( p_block );
+                        return -1;
+                    }
+                }
+                else return -1;
             }
-
-            /* We don't want to keep the header of the read sector */
-            memcpy( p_buffer, p_block + VCD_DATA_START, VCD_DATA_SIZE );
-
-            return 0;
         }
 
 #elif defined( HAVE_SCSIREQ_IN_SYS_SCSIIO_H )
         struct scsireq  sc;
-
-        int i_blocks = 1;
         int i_ret;
 
-        int sector_type = 5; /* mode2/form2 */
-        int sync = 0,
-            header_code = 0,
-            user_data = 1,
-            edc_ecc = 0,
-            error_field = 0;
-        int sub_channel = 0;
-
         memset( &sc, 0, sizeof(sc) );
         sc.cmd[0] = 0xBE;
-        sc.cmd[1] = (sector_type) << 2;
+        sc.cmd[1] = i_type == VCD_TYPE ? SECTOR_TYPE_MODE2_FORM2:
+                                         SECTOR_TYPE_CDDA;
         sc.cmd[2] = (i_sector >> 24) & 0xff;
         sc.cmd[3] = (i_sector >> 16) & 0xff;
         sc.cmd[4] = (i_sector >>  8) & 0xff;
         sc.cmd[5] = (i_sector >>  0) & 0xff;
-        sc.cmd[6] = (i_blocks >> 16) & 0xff;
-        sc.cmd[7] = (i_blocks >>  8) & 0xff;
-        sc.cmd[8] = (i_blocks >>  0) & 0xff;
-        sc.cmd[9] = (sync << 7) | (header_code << 5) | (user_data << 4) |
-                    (edc_ecc << 3) | (error_field << 1);
-        sc.cmd[10] = sub_channel;
+        sc.cmd[6] = (i_nb >> 16) & 0xff;
+        sc.cmd[7] = (i_nb >>  8) & 0xff;
+        sc.cmd[8] = (i_nb      ) & 0xff;
+        sc.cmd[9] = i_type == VCD_TYPE ? READ_CD_RAW_MODE2 : READ_CD_USERDATA;
+        sc.cmd[10] = 0; /* sub channel */
         sc.cmdlen = 12;
         sc.databuf = (caddr_t)p_block;
-        sc.datalen = VCD_SECTOR_SIZE;      // was 2328 == VCD_DATA_SIZE + 4;
+        sc.datalen = VCD_SECTOR_SIZE * i_nb;
         sc.senselen = sizeof( sc.sense );
         sc.flags = SCCMD_READ;
         sc.timeout = 10000;
@@ -730,23 +734,25 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
         if( i_ret == -1 )
         {
             msg_Err( p_this, "SCIOCCOMMAND failed" );
+            if( i_type == VCD_TYPE ) free( p_block );
             return -1;
         }
         if( sc.retsts || sc.error )
         {
             msg_Err( p_this, "SCSI command failed: status %d error %d\n",
                              sc.retsts, sc.error );
+            if( i_type == VCD_TYPE ) free( p_block );
            return -1;
         }
 
 #elif defined( HAVE_IOC_TOC_HEADER_IN_SYS_CDIO_H )
-
         int i_size = VCD_SECTOR_SIZE;
 
         if( ioctl( p_vcddev->i_device_handle, CDRIOCSETBLOCKSIZE, &i_size )
             == -1 )
         {
             msg_Err( p_this, "Could not set block size" );
+            if( i_type == VCD_TYPE ) free( p_block );
             return( -1 );
         }
 
@@ -754,41 +760,60 @@ int ioctl_ReadSector( vlc_object_t *p_this, const vcddev_t *p_vcddev,
                    i_sector * VCD_SECTOR_SIZE, SEEK_SET ) == -1 )
         {
             msg_Err( p_this, "Could not lseek to sector %d", i_sector );
+            if( i_type == VCD_TYPE ) free( p_block );
             return( -1 );
         }
 
-        if( read( p_vcddev->i_device_handle, p_block, VCD_SECTOR_SIZE ) == -1 )
+        if( read( p_vcddev->i_device_handle,
+                  p_block, VCD_SECTOR_SIZE * i_nb ) == -1 )
         {
             msg_Err( p_this, "Could not read sector %d", i_sector );
+            if( i_type == VCD_TYPE ) free( p_block );
             return( -1 );
         }
 
 #else
-        int i_dummy = i_sector + 2 * CD_FRAMES;
+        for( i = 0; i < i_nb; i++ )
+        {
+            int i_dummy = i_sector + i + 2 * CD_FRAMES;
 
-#define p_msf ((struct cdrom_msf0 *)p_block)
-        p_msf->minute =   i_dummy / (CD_FRAMES * CD_SECS);
-        p_msf->second = ( i_dummy % (CD_FRAMES * CD_SECS) ) / CD_FRAMES;
-        p_msf->frame =  ( i_dummy % (CD_FRAMES * CD_SECS) ) % CD_FRAMES;
+#define p_msf ((struct cdrom_msf0 *)(p_block + i * VCD_SECTOR_SIZE))
+            p_msf->minute =   i_dummy / (CD_FRAMES * CD_SECS);
+            p_msf->second = ( i_dummy % (CD_FRAMES * CD_SECS) ) / CD_FRAMES;
+            p_msf->frame =  ( i_dummy % (CD_FRAMES * CD_SECS) ) % CD_FRAMES;
 #undef p_msf
 
-        if( ioctl(p_vcddev->i_device_handle, CDROMREADRAW, p_block) == -1 )
-        {
-            msg_Err( p_this, "could not read block %i from disc", i_sector );
-            return( -1 );
+            if( ioctl( p_vcddev->i_device_handle, CDROMREADRAW,
+                       p_block + i * VCD_SECTOR_SIZE ) == -1 )
+            {
+                msg_Err( p_this, "could not read block %i from disc",
+                         i_sector );
+
+                if( i == 0 )
+                {
+                    if( i_type == VCD_TYPE ) free( p_block );
+                    return( -1 );
+                }
+                else break;
+            }
         }
 #endif
-
-#if defined( HAVE_SCSIREQ_IN_SYS_SCSIIO_H )
-        /* FIXME: is this ok? */
-        memcpy( p_buffer, p_block, VCD_DATA_SIZE );
-#else
-        /* We don't want to keep the header of the read sector */
-        memcpy( p_buffer, p_block + VCD_DATA_START, VCD_DATA_SIZE );
-#endif
-
-        return( 0 );
     }
+
+    /* For VCDs, we don't want to keep the header and footer of the
+     * sectors read */
+    if( i_type == VCD_TYPE )
+    {
+        for( i = 0; i < i_nb; i++ )
+        {
+            memcpy( p_buffer + i * VCD_DATA_SIZE,
+                    p_block + i * VCD_SECTOR_SIZE + VCD_DATA_START,
+                    VCD_DATA_SIZE );
+        }
+        free( p_block );
+    }
+
+    return( 0 );
 }
 
 /****************************************************************************
@@ -832,7 +857,7 @@ static int OpenVCDImage( vlc_object_t * p_this, const char *psz_dev,
     }
 
     /* Open the cue file and try to parse it */
-    msg_Dbg( p_this,"using .cue file: %s", psz_cuefile );
+    msg_Dbg( p_this,"trying .cue file: %s", psz_cuefile );
     cuefile = fopen( psz_cuefile, "rt" );
     if( cuefile && fscanf( cuefile, "FILE %c", line ) &&
         fgets( line, 1024, cuefile ) )
@@ -843,7 +868,8 @@ static int OpenVCDImage( vlc_object_t * p_this, const char *psz_dev,
             *p_pos = 0;
 
             /* Take care of path standardization */
-            if( *line != '/' && (p_pos = strrchr( psz_cuefile, '/' )) )
+            if( *line != '/' && ((p_pos = strrchr( psz_cuefile, '/' ))
+                || (p_pos = strrchr( psz_cuefile, '\\' ) )) )
             {
                 psz_vcdfile = malloc( strlen(line) +
                                       (p_pos - psz_cuefile + 1) + 1 );
