@@ -2,7 +2,7 @@
  * trivial.c : trivial mixer plug-in (1 input, no downmixing)
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: trivial.c,v 1.11 2002/11/20 16:43:33 sam Exp $
+ * $Id: trivial.c 6961 2004-03-05 17:34:23Z sam $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -10,7 +10,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -42,7 +42,7 @@ static void DoWork    ( aout_instance_t *, aout_buffer_t * );
  * Module descriptor
  *****************************************************************************/
 vlc_module_begin();
-    set_description( _("trivial audio mixer module") );
+    set_description( _("Trivial audio mixer") );
     set_capability( "audio mixer", 1 );
     set_callbacks( Create, NULL );
 vlc_module_end();
@@ -73,7 +73,7 @@ static void DoWork( aout_instance_t * p_aout, aout_buffer_t * p_buffer )
     int i = 0;
     aout_input_t * p_input = p_aout->pp_inputs[i];
     int i_nb_channels = aout_FormatNbChannels( &p_aout->mixer.mixer );
-    int i_nb_bytes = p_buffer->i_nb_samples * sizeof(s32)
+    int i_nb_bytes = p_buffer->i_nb_samples * sizeof(int32_t)
                       * i_nb_channels;
     byte_t * p_in;
     byte_t * p_out;
@@ -92,7 +92,7 @@ static void DoWork( aout_instance_t * p_aout, aout_buffer_t * p_buffer )
         ptrdiff_t i_available_bytes = (p_input->fifo.p_first->p_buffer
                                         - p_in)
                                         + p_input->fifo.p_first->i_nb_samples
-                                           * sizeof(s32)
+                                           * sizeof(int32_t)
                                            * i_nb_channels;
 
         if ( i_available_bytes < i_nb_bytes )
