@@ -2,7 +2,7 @@
  * waveout.c : Windows waveOut plugin for vlc
  *****************************************************************************
  * Copyright (C) 2001 VideoLAN
- * $Id: waveout.c 8927 2004-10-06 09:19:18Z gbazin $
+ * $Id: waveout.c 9153 2004-11-05 12:56:35Z gbazin $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *      
@@ -494,10 +494,10 @@ static int OpenWaveOut( aout_instance_t *p_aout, int i_format,
 #define waveformat p_aout->output.p_sys->waveformat
 
     waveformat.dwChannelMask = 0;
-    for( i = 0; i < sizeof(pi_channels_in)/sizeof(uint32_t); i++ )
+    for( i = 0; i < sizeof(pi_channels_src)/sizeof(uint32_t); i++ )
     {
-        if( i_channels & pi_channels_in[i] )
-            waveformat.dwChannelMask |= pi_channels_out[i];
+        if( i_channels & pi_channels_src[i] )
+            waveformat.dwChannelMask |= pi_channels_in[i];
     }
 
     switch( i_format )
