@@ -2,10 +2,10 @@
  * video.c: video decoder using the ffmpeg library
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: video.c,v 1.57 2003/12/02 10:55:21 gbazin Exp $
+ * $Id: video.c 9156 2004-11-05 14:57:53Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
- *          Gildas Bazin <gbazin@netcourrier.com>
+ *          Gildas Bazin <gbazin@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +71,9 @@ struct decoder_sys_t
 
     vlc_bool_t b_has_b_frames;
 
+    /* Hack to force display of still pictures */
+    vlc_bool_t b_first_frame;
+
     int i_buffer_orig, i_buffer;
     char *p_buffer_orig, *p_buffer;
 
@@ -81,6 +84,9 @@ struct decoder_sys_t
     vlc_bool_t b_pp_init;
 };
 
+/* FIXME (dummy palette for now) */
+static AVPaletteControl palette_control;
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
@@ -88,6 +94,12 @@ static void ffmpeg_CopyPicture    ( decoder_t *, picture_t *, AVFrame * );
 static int  ffmpeg_GetFrameBuf    ( struct AVCodecContext *, AVFrame * );
 static void ffmpeg_ReleaseFrameBuf( struct AVCodecContext *, AVFrame * );
 
+static uint32_t ffmpeg_CodecTag( vlc_fourcc_t fcc )
+{
+    uint8_t *p = (uint8_t*)&fcc;
+    return p[0] | (p[1] << 8) | (p[2] << 16) | (p[3] << 24);
+}
+
 /*****************************************************************************
  * Local Functions
  *****************************************************************************/
@@ -168,6 +180,12 @@ static inline picture_t *ffmpeg_NewPictBuf( decoder_t *p_dec,
         }
     }
 
+    if( p_context->frame_rate > 0 && p_context->frame_rate_base > 0 )
+    {
+        p_dec->fmt_out.video.i_frame_rate = p_context->frame_rate;
+        p_dec->fmt_out.video.i_frame_rate_base = p_context->frame_rate_base;
+    }
+
     p_pic = p_dec->pf_vout_buffer_new( p_dec );
 
 #ifdef LIBAVCODEC_PP
@@ -194,8 +212,6 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
     decoder_sys_t *p_sys;
     vlc_value_t lockval;
     vlc_value_t val;
-    int i_tmp;
-
 
     var_Get( p_dec->p_libvlc, "avcodec", &lockval );
 
@@ -214,24 +230,35 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
     p_sys->p_ff_pic = avcodec_alloc_frame();
 
     /* ***** Fill p_context with init values ***** */
+    /* FIXME: remove when ffmpeg deals properly with avc1 */
+    if( p_dec->fmt_in.i_codec != VLC_FOURCC('a','v','c','1') )
+    /* End FIXME */
+    p_sys->p_context->codec_tag = ffmpeg_CodecTag( p_dec->fmt_in.i_codec );
     p_sys->p_context->width  = p_dec->fmt_in.video.i_width;
     p_sys->p_context->height = p_dec->fmt_in.video.i_height;
+    p_sys->p_context->bits_per_sample = p_dec->fmt_in.video.i_bits_per_pixel;
 
     /*  ***** Get configuration of ffmpeg plugin ***** */
-    i_tmp = config_GetInt( p_dec, "ffmpeg-workaround-bugs" );
-    p_sys->p_context->workaround_bugs  = __MAX( __MIN( i_tmp, 99 ), 0 );
-
-    i_tmp = config_GetInt( p_dec, "ffmpeg-error-resilience" );
-    p_sys->p_context->error_resilience = __MAX( __MIN( i_tmp, 99 ), -1 );
+    p_sys->p_context->workaround_bugs =
+        config_GetInt( p_dec, "ffmpeg-workaround-bugs" );
+    p_sys->p_context->error_resilience =
+        config_GetInt( p_dec, "ffmpeg-error-resilience" );
 
     var_Create( p_dec, "grayscale", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
     var_Get( p_dec, "grayscale", &val );
     if( val.b_bool ) p_sys->p_context->flags |= CODEC_FLAG_GRAY;
 
-    /* Decide if we set CODEC_FLAG_TRUNCATED */
-    var_Create( p_dec, "ffmpeg-truncated", VLC_VAR_INTEGER|VLC_VAR_DOINHERIT );
-    var_Get( p_dec, "ffmpeg-truncated", &val );
-    if( val.i_int > 0 ) p_sys->p_context->flags |= CODEC_FLAG_TRUNCATED;
+    var_Create( p_dec, "ffmpeg-vismv", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_dec, "ffmpeg-vismv", &val );
+#if LIBAVCODEC_BUILD >= 4698
+    if( val.i_int ) p_sys->p_context->debug_mv = val.i_int;
+#endif
+
+    var_Create( p_dec, "ffmpeg-lowres", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_dec, "ffmpeg-lowres", &val );
+#if LIBAVCODEC_BUILD >= 4723
+    if( val.i_int > 0 && val.i_int <= 2 ) p_sys->p_context->lowres = val.i_int;
+#endif
 
     /* ***** ffmpeg frame skipping ***** */
     var_Create( p_dec, "ffmpeg-hurry-up", VLC_VAR_BOOL | VLC_VAR_DOINHERIT );
@@ -246,7 +273,14 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
         ffmpeg_PixFmtToChroma( p_sys->p_context->pix_fmt ) &&
         /* Apparently direct rendering doesn't work with YUV422P */
         p_sys->p_context->pix_fmt != PIX_FMT_YUV422P &&
-        !(p_sys->p_context->width % 16) && !(p_sys->p_context->height % 16) )
+        /* H264 uses too many reference frames */
+        p_sys->i_codec_id != CODEC_ID_H264 &&
+        !(p_sys->p_context->width % 16) && !(p_sys->p_context->height % 16) &&
+#if LIBAVCODEC_BUILD >= 4698
+        !p_sys->p_context->debug_mv )
+#else
+        1 )
+#endif
     {
         /* Some codecs set pix_fmt only after the 1st frame has been decoded,
          * so we need to do another check in ffmpeg_GetFrameBuf() */
@@ -289,6 +323,31 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
             memcpy( &p[0],  "SVQ3", 4 );
             memset( &p[4], 0, 8 );
             memcpy( &p[12], p_dec->fmt_in.p_extra, i_size );
+
+            /* Now remove all atoms before the SMI one */
+            if( p_sys->p_context->extradata_size > 0x5a &&
+                strncmp( &p[0x56], "SMI ", 4 ) )
+            {
+                uint8_t *psz = &p[0x52];
+
+                while( psz < &p[p_sys->p_context->extradata_size - 8] )
+                {
+                    int i_size = GetDWBE( psz );
+                    if( i_size <= 1 )
+                    {
+                        /* FIXME handle 1 as long size */
+                        break;
+                    }
+                    if( !strncmp( &psz[4], "SMI ", 4 ) )
+                    {
+                        memmove( &p[0x52], psz,
+                                 &p[p_sys->p_context->extradata_size] - psz );
+                        break;
+                    }
+
+                    psz += i_size;
+                }
+            }
         }
         else if( p_dec->fmt_in.i_codec == VLC_FOURCC( 'R', 'V', '1', '0' ) ||
                  p_dec->fmt_in.i_codec == VLC_FOURCC( 'R', 'V', '1', '3' ) ||
@@ -304,9 +363,16 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
                         p_dec->fmt_in.i_extra );
                 p_sys->p_context->sub_id= ((uint32_t*)p_dec->fmt_in.p_extra)[1];
 
-                msg_Warn( p_dec, "using extra data for RV codec sub_id=%08x", p_sys->p_context->sub_id );
+                msg_Warn( p_dec, "using extra data for RV codec sub_id=%08x",
+                          p_sys->p_context->sub_id );
             }
         }
+        /* FIXME: remove when ffmpeg deals properly with avc1 */
+        else if( p_dec->fmt_in.i_codec == VLC_FOURCC('a','v','c','1') )
+        {
+            ;
+        }
+        /* End FIXME */
         else
         {
             p_sys->p_context->extradata_size = i_size;
@@ -323,21 +389,32 @@ int E_(InitVideoDec)( decoder_t *p_dec, AVCodecContext *p_context,
     p_sys->input_pts = p_sys->input_dts = 0;
     p_sys->i_pts = 0;
     p_sys->b_has_b_frames = VLC_FALSE;
+    p_sys->b_first_frame = VLC_TRUE;
     p_sys->i_late_frames = 0;
     p_sys->i_buffer = 0;
     p_sys->i_buffer_orig = 1;
-    p_sys->p_buffer_orig = p_sys->p_buffer = malloc( p_sys->i_buffer );
+    p_sys->p_buffer_orig = p_sys->p_buffer = malloc( p_sys->i_buffer_orig );
 
     /* Set output properties */
     p_dec->fmt_out.i_cat = VIDEO_ES;
     p_dec->fmt_out.i_codec = ffmpeg_PixFmtToChroma( p_context->pix_fmt );
 
+    /* Setup palette */
+#if LIBAVCODEC_BUILD >= 4688
+    if( p_dec->fmt_in.video.p_palette )
+        p_sys->p_context->palctrl =
+            (AVPaletteControl *)p_dec->fmt_in.video.p_palette;
+    else
+        p_sys->p_context->palctrl = &palette_control;
+#endif
+
     /* ***** Open the codec ***** */
     vlc_mutex_lock( lockval.p_address );
     if( avcodec_open( p_sys->p_context, p_sys->p_codec ) < 0 )
     {
         vlc_mutex_unlock( lockval.p_address );
         msg_Err( p_dec, "cannot open codec (%s)", p_sys->psz_namecodec );
+        free( p_sys );
         return VLC_EGENERIC;
     }
     vlc_mutex_unlock( lockval.p_address );
@@ -354,22 +431,26 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
     int b_drawpicture;
+    int b_null_size = VLC_FALSE;
     block_t *p_block;
 
     if( !pp_block || !*pp_block ) return NULL;
 
     p_block = *pp_block;
 
-    if( p_block->b_discontinuity )
+    if( p_block->i_flags & BLOCK_FLAG_DISCONTINUITY )
     {
         p_sys->i_buffer = 0;
         p_sys->i_pts = 0; /* To make sure we recover properly */
 
+        p_sys->input_pts = p_sys->input_dts = 0;
+        p_sys->i_late_frames = 0;
+
         block_Release( p_block );
         return NULL;
     }
 
-    if( p_sys->i_late_frames > 0 &&
+    if( !p_dec->b_pace_control && p_sys->i_late_frames > 0 &&
         mdate() - p_sys->i_late_frames_start > I64C(5000000) )
     {
         if( p_sys->i_pts )
@@ -394,7 +475,8 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
 
     /* TODO implement it in a better way */
     /* A good idea could be to decode all I pictures and see for the other */
-    if( p_sys->b_hurry_up && p_sys->i_late_frames > 4 )
+    if( !p_dec->b_pace_control &&
+        p_sys->b_hurry_up && p_sys->i_late_frames > 4 )
     {
         b_drawpicture = 0;
         if( p_sys->i_late_frames < 8 )
@@ -422,6 +504,7 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
     if( p_sys->p_context->width <= 0 || p_sys->p_context->height <= 0 )
     {
         p_sys->p_context->hurry_up = 5;
+        b_null_size = VLC_TRUE;
     }
 
     /*
@@ -462,6 +545,17 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
         i_used = avcodec_decode_video( p_sys->p_context, p_sys->p_ff_pic,
                                        &b_gotpicture,
                                        p_sys->p_buffer, p_sys->i_buffer );
+        if( b_null_size && p_sys->p_context->width > 0 &&
+            p_sys->p_context->height > 0 )
+        {
+            /* Reparse it to not drop the I frame */
+            b_null_size = VLC_FALSE;
+            p_sys->p_context->hurry_up = 0;
+            i_used = avcodec_decode_video( p_sys->p_context, p_sys->p_ff_pic,
+                                           &b_gotpicture,
+                                           p_sys->p_buffer, p_sys->i_buffer );
+        }
+
         if( i_used < 0 )
         {
             msg_Warn( p_dec, "cannot decode one frame (%d bytes)",
@@ -479,7 +573,11 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
         p_sys->p_buffer += i_used;
 
         /* Nothing to display */
-        if( !b_gotpicture ) continue;
+        if( !b_gotpicture )
+        {
+            if( i_used == 0 ) break;
+            continue;
+        }
 
         /* Update frame late count*/
         if( p_sys->i_pts && p_sys->i_pts <= mdate() )
@@ -493,13 +591,13 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
             p_sys->i_late_frames = 0;
         }
 
-        if( !b_drawpicture || p_sys->p_ff_pic->linesize[0] == 0 )
+        if( !b_drawpicture || !p_sys->p_ff_pic->linesize[0] )
         {
             /* Do not display the picture */
             continue;
         }
 
-        if( !p_sys->b_direct_rendering || p_sys->b_pp )
+        if( !p_sys->p_ff_pic->opaque )
         {
             /* Get a new picture */
             p_pic = ffmpeg_NewPictBuf( p_dec, p_sys->p_context );
@@ -540,6 +638,20 @@ picture_t *E_(DecodeVideo)( decoder_t *p_dec, block_t **pp_block )
                     p_sys->p_context->frame_rate_base /
                     (2 * p_sys->p_context->frame_rate);
             }
+
+            if( p_sys->b_first_frame )
+            {
+                /* Hack to force display of still pictures */
+                p_sys->b_first_frame = VLC_FALSE;
+                p_pic->b_force = VLC_TRUE;
+            }
+
+            p_pic->i_nb_fields = 2 + p_sys->p_ff_pic->repeat_pict;
+#if LIBAVCODEC_BUILD >= 4685
+            p_pic->b_progressive = !p_sys->p_ff_pic->interlaced_frame;
+            p_pic->b_top_field_first = p_sys->p_ff_pic->top_field_first;
+#endif
+
             return p_pic;
         }
         else
@@ -562,7 +674,7 @@ void E_(EndVideoDec)( decoder_t *p_dec )
 {
     decoder_sys_t *p_sys = p_dec->p_sys;
 
-    if( p_sys->p_ff_pic ) free( p_sys->p_ff_pic );
+    if( p_sys->p_ff_pic ) av_free( p_sys->p_ff_pic );
 
 #ifdef LIBAVCODEC_PP
     E_(ClosePostproc)( p_dec, p_sys->p_pp );
@@ -599,7 +711,8 @@ static void ffmpeg_CopyPicture( decoder_t *p_dec,
             i_dst_stride = p_pic->p[i_plane].i_pitch;
 
             i_size = __MIN( i_src_stride, i_dst_stride );
-            for( i_line = 0; i_line < p_pic->p[i_plane].i_lines; i_line++ )
+            for( i_line = 0; i_line < p_pic->p[i_plane].i_visible_lines;
+                 i_line++ )
             {
                 p_dec->p_vlc->pf_memcpy( p_dst, p_src, i_size );
                 p_src += i_src_stride;
@@ -615,8 +728,9 @@ static void ffmpeg_CopyPicture( decoder_t *p_dec,
         /* we need to convert to I420 */
         switch( p_sys->p_context->pix_fmt )
         {
-        case( PIX_FMT_YUV410P ):
-        case( PIX_FMT_YUV411P ):
+        case PIX_FMT_YUV410P:
+        case PIX_FMT_YUV411P:
+        case PIX_FMT_PAL8:
             for( i = 0; i < p_pic->i_planes; i++ )
             {
                 dest_pic.data[i] = p_pic->p[i].p_pixels;
@@ -660,7 +774,7 @@ static int ffmpeg_GetFrameBuf( struct AVCodecContext *p_context,
         /* Some demuxers only set the dts so let's try to find a useful
          * timestamp from this */
         if( !p_context->has_b_frames || !p_sys->b_has_b_frames ||
-            !p_ff_pic->reference )
+            !p_ff_pic->reference || !p_sys->i_pts )
         {
             p_ff_pic->pts = p_sys->input_dts;
         }
@@ -668,7 +782,12 @@ static int ffmpeg_GetFrameBuf( struct AVCodecContext *p_context,
     }
     else p_ff_pic->pts = 0;
 
-    p_sys->input_pts = p_sys->input_dts = 0;
+    if( p_sys->i_pts ) /* make sure 1st frame has a pts > 0 */
+    {
+        p_sys->input_pts = p_sys->input_dts = 0;
+    }
+
+    p_ff_pic->opaque = 0;
 
     /* Not much to do in indirect rendering mode */
     if( !p_sys->b_direct_rendering || p_sys->b_pp )
@@ -725,7 +844,7 @@ static void ffmpeg_ReleaseFrameBuf( struct AVCodecContext *p_context,
     decoder_t *p_dec = (decoder_t *)p_context->opaque;
     picture_t *p_pic;
 
-    if( p_ff_pic->type != FF_BUFFER_TYPE_USER )
+    if( !p_ff_pic->opaque )
     {
         avcodec_default_release_buffer( p_context, p_ff_pic );
         return;
