@@ -2,7 +2,7 @@
  * flac.c: flac decoder/packetizer/encoder module making use of libflac
  *****************************************************************************
  * Copyright (C) 1999-2001 VideoLAN
- * $Id: flac.c 7487 2004-04-25 18:08:13Z gbazin $
+ * $Id: flac.c 8546 2004-08-28 11:02:51Z gbazin $
  *
  * Authors: Gildas Bazin <gbazin@videolan.org>
  *          Sigmund Augdal <sigmunau@idi.ntnu.no>
@@ -215,6 +215,36 @@ static int OpenDecoder( vlc_object_t *p_this )
     p_dec->pf_decode_audio = DecodeBlock;
     p_dec->pf_packetize    = PacketizeBlock;
 
+    return VLC_SUCCESS;
+}
+
+static int OpenPacketizer( vlc_object_t *p_this )
+{
+    decoder_t *p_dec = (decoder_t*)p_this;
+    int i_ret;
+
+    /* Hmmm, mem leak ?*/
+    es_format_Copy( &p_dec->fmt_out, &p_dec->fmt_in );
+
+    i_ret = OpenDecoder( p_this );
+
+    /* Set output properties */
+    p_dec->fmt_out.i_codec = VLC_FOURCC('f','l','a','c');
+
+    if( i_ret != VLC_SUCCESS ) return i_ret;
+
+    return i_ret;
+}
+
+/*****************************************************************************
+ * ProcessHeader: processe Flac header.
+ *****************************************************************************/
+static void ProcessHeader( decoder_t *p_dec )
+{
+    decoder_sys_t *p_sys = p_dec->p_sys;
+
+    if( !p_dec->fmt_in.i_extra ) return;
+
     /* Decode STREAMINFO */
     msg_Dbg( p_dec, "decode STREAMINFO" );
     p_sys->p_block = block_New( p_dec, p_dec->fmt_in.i_extra );
@@ -223,20 +253,16 @@ static int OpenDecoder( vlc_object_t *p_this )
     FLAC__stream_decoder_process_until_end_of_metadata( p_sys->p_flac );
     msg_Dbg( p_dec, "STREAMINFO decoded" );
 
-    return VLC_SUCCESS;
-}
+    if( !p_sys->b_stream_info ) return;
 
-static int OpenPacketizer( vlc_object_t *p_this )
-{
-    decoder_t *p_dec = (decoder_t*)p_this;
-
-    int i_ret = OpenDecoder( p_this );
-
-    if( i_ret != VLC_SUCCESS ) return i_ret;
-
-    es_format_Copy( &p_dec->fmt_out, &p_dec->fmt_in );
-
-    return i_ret;
+    if( p_dec->fmt_out.i_codec == VLC_FOURCC('f','l','a','c') )
+    {
+        p_dec->fmt_out.i_extra = p_dec->fmt_in.i_extra;
+        p_dec->fmt_out.p_extra =
+            realloc( p_dec->fmt_out.p_extra, p_dec->fmt_out.i_extra );
+        memcpy( p_dec->fmt_out.p_extra,
+                p_dec->fmt_in.p_extra, p_dec->fmt_out.i_extra );
+    }
 }
 
 /****************************************************************************
@@ -252,12 +278,19 @@ static block_t *PacketizeBlock( decoder_t *p_dec, block_t **pp_block )
 
     if( !pp_block || !*pp_block ) return NULL;
 
+    if( !p_sys->b_stream_info ) ProcessHeader( p_dec );
+
     if( !aout_DateGet( &p_sys->end_date ) && !(*pp_block)->i_pts )
     {
         /* We've just started the stream, wait for the first PTS. */
         block_Release( *pp_block );
         return NULL;
     }
+    else if( !aout_DateGet( &p_sys->end_date ) )
+    {
+        /* The first PTS is as good as anything else. */
+        aout_DateSet( &p_sys->end_date, (*pp_block)->i_pts );
+    }
 
     if( (*pp_block)->i_flags&BLOCK_FLAG_DISCONTINUITY )
     {
@@ -325,7 +358,6 @@ static block_t *PacketizeBlock( decoder_t *p_dec, block_t **pp_block )
             {
                 p_dec->fmt_out.audio.i_rate = p_sys->i_rate;
                 aout_DateInit( &p_sys->end_date, p_sys->i_rate );
-                p_dec->fmt_out.audio.i_rate = p_sys->i_rate;
             }
             p_sys->i_state = STATE_NEXT_SYNC;
             p_sys->i_frame_size = 1;
@@ -364,7 +396,6 @@ static block_t *PacketizeBlock( decoder_t *p_dec, block_t **pp_block )
                 /* Need more data */
                 return NULL;
             }
-            break;
 
         case STATE_SEND_DATA:
             p_sout_block = block_New( p_dec, p_sys->i_frame_size );
