@@ -2,7 +2,7 @@
  * asf.c : ASF demux module
  *****************************************************************************
  * Copyright (C) 2002-2003 VideoLAN
- * $Id: asf.c,v 1.51 2004/01/31 05:27:02 fenrir Exp $
+ * $Id: asf.c 9063 2004-10-27 10:42:48Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -42,7 +42,7 @@ static void Close ( vlc_object_t * );
 
 vlc_module_begin();
     set_description( _("ASF v1.0 demuxer") );
-    set_capability( "demux", 200 );
+    set_capability( "demux2", 200 );
     set_callbacks( Open, Close );
     add_shortcut( "asf" );
 vlc_module_end();
@@ -51,10 +51,10 @@ vlc_module_end();
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static int Demux  ( input_thread_t * );
-static int Control( input_thread_t *, int i_query, va_list args );
+static int Demux  ( demux_t * );
+static int Control( demux_t *, int i_query, va_list args );
 
-typedef struct asf_stream_s
+typedef struct
 {
     int i_cat;
 
@@ -66,18 +66,19 @@ typedef struct asf_stream_s
 
     block_t         *p_frame; /* use to gather complete frame */
 
-} asf_stream_t;
+} asf_track_t;
 
 struct demux_sys_t
 {
     mtime_t             i_time;     /* s */
     mtime_t             i_length;   /* length of file file */
+    int64_t             i_bitrate;  /* global file bitrate */
 
     asf_object_root_t            *p_root;
     asf_object_file_properties_t *p_fp;
 
-    unsigned int        i_streams;
-    asf_stream_t        *stream[128];
+    unsigned int        i_track;
+    asf_track_t         *track[128];
 
     int64_t             i_data_begin;
     int64_t             i_data_end;
@@ -86,328 +87,91 @@ struct demux_sys_t
 };
 
 static mtime_t  GetMoviePTS( demux_sys_t * );
-static int      DemuxPacket( input_thread_t *, vlc_bool_t b_play_audio );
+static int      DemuxInit( demux_t * );
+static void     DemuxEnd( demux_t * );
+static int      DemuxPacket( demux_t * );
 
 /*****************************************************************************
  * Open: check file and initializes ASF structures
  *****************************************************************************/
 static int Open( vlc_object_t * p_this )
 {
-    input_thread_t  *p_input = (input_thread_t *)p_this;
-    uint8_t         *p_peek;
-
-    guid_t          guid;
-
-    demux_sys_t     *p_sys;
-    unsigned int    i_stream, i;
-    asf_object_content_description_t *p_cd;
-
-    vlc_bool_t      b_seekable;
+    demux_t     *p_demux = (demux_t *)p_this;
+    demux_sys_t *p_sys;
+    guid_t      guid;
+    uint8_t     *p_peek;
 
     /* a little test to see if it could be a asf stream */
-    if( input_Peek( p_input, &p_peek, 16 ) < 16 )
+    if( stream_Peek( p_demux->s, &p_peek, 16 ) < 16 )
     {
-        msg_Warn( p_input, "ASF plugin discarded (cannot peek)" );
+        msg_Warn( p_demux, "ASF plugin discarded (cannot peek)" );
         return VLC_EGENERIC;
     }
     ASF_GetGUID( &guid, p_peek );
     if( !ASF_CmpGUID( &guid, &asf_object_header_guid ) )
     {
-        msg_Warn( p_input, "ASF plugin discarded (not a valid file)" );
+        msg_Warn( p_demux, "ASF plugin discarded (not a valid file)" );
         return VLC_EGENERIC;
     }
 
-    /* Set p_input field */
-    p_input->pf_demux         = Demux;
-    p_input->pf_demux_control = Control;
-    p_input->p_demux_data = p_sys = malloc( sizeof( demux_sys_t ) );
+    /* Set p_demux fields */
+    p_demux->pf_demux = Demux;
+    p_demux->pf_control = Control;
+    p_demux->p_sys = p_sys = malloc( sizeof( demux_sys_t ) );
     memset( p_sys, 0, sizeof( demux_sys_t ) );
-    p_sys->i_time = -1;
-    p_sys->i_length = 0;
 
-    /* Now load all object ( except raw data ) */
-    stream_Control( p_input->s, STREAM_CAN_FASTSEEK, &b_seekable );
-    if( (p_sys->p_root = ASF_ReadObjectRoot( p_input->s, b_seekable )) == NULL )
+    /* Load the headers */
+    if( DemuxInit( p_demux ) )
     {
-        msg_Warn( p_input, "ASF plugin discarded (not a valid file)" );
-        free( p_sys );
         return VLC_EGENERIC;
     }
-    p_sys->p_fp = p_sys->p_root->p_fp;
-
-    if( p_sys->p_fp->i_min_data_packet_size != p_sys->p_fp->i_max_data_packet_size )
-    {
-        msg_Warn( p_input,
-                  "ASF plugin discarded (invalid file_properties object)" );
-        goto error;
-    }
-
-    p_sys->i_streams = ASF_CountObject( p_sys->p_root->p_hdr,
-                                          &asf_object_stream_properties_guid );
-    if( !p_sys->i_streams )
-    {
-        msg_Warn( p_input, "ASF plugin discarded (cannot find any stream!)" );
-        goto error;
-    }
-
-    msg_Dbg( p_input, "found %d streams", p_sys->i_streams );
-
-    vlc_mutex_lock( &p_input->stream.stream_lock );
-    if( input_InitStream( p_input, 0 ) == -1)
-    {
-        vlc_mutex_unlock( &p_input->stream.stream_lock );
-        msg_Err( p_input, "cannot init stream" );
-        goto error;
-    }
-    p_input->stream.i_mux_rate = 0 ; /* updated later */
-    vlc_mutex_unlock( &p_input->stream.stream_lock );
-
-    for( i_stream = 0; i_stream < p_sys->i_streams; i_stream ++ )
-    {
-        asf_stream_t    *p_stream;
-        asf_object_stream_properties_t *p_sp;
-
-        p_sp = ASF_FindObject( p_sys->p_root->p_hdr,
-                               &asf_object_stream_properties_guid,
-                               i_stream );
-
-        p_stream =
-            p_sys->stream[p_sp->i_stream_number] =
-                malloc( sizeof( asf_stream_t ) );
-        memset( p_stream, 0, sizeof( asf_stream_t ) );
-
-        p_stream->i_time = -1;
-        p_stream->p_sp = p_sp;
-        p_stream->p_es = NULL;
-        p_stream->p_frame = NULL;
-
-        if( ASF_CmpGUID( &p_sp->i_stream_type, &asf_object_stream_type_audio ) &&
-            p_sp->i_type_specific_data_length >= sizeof( WAVEFORMATEX ) - 2 )
-        {
-            es_format_t  fmt;
-            uint8_t      *p_data = p_sp->p_type_specific_data;
-
-            es_format_Init( &fmt, AUDIO_ES, 0 );
-            wf_tag_to_fourcc( GetWLE( &p_data[0] ), &fmt.i_codec, NULL );
-            fmt.audio.i_channels        = GetWLE(  &p_data[2] );
-            fmt.audio.i_rate      = GetDWLE( &p_data[4] );
-            fmt.i_bitrate         = GetDWLE( &p_data[8] ) * 8;
-            fmt.audio.i_blockalign      = GetWLE(  &p_data[12] );
-            fmt.audio.i_bitspersample   = GetWLE(  &p_data[14] );
-
-            if( p_sp->i_type_specific_data_length > sizeof( WAVEFORMATEX ) )
-            {
-                fmt.i_extra = __MIN( GetWLE( &p_data[16] ),
-                                     p_sp->i_type_specific_data_length - sizeof( WAVEFORMATEX ) );
-                fmt.p_extra = malloc( fmt.i_extra );
-                memcpy( fmt.p_extra, &p_data[sizeof( WAVEFORMATEX )], fmt.i_extra );
-            }
-
-            p_stream->i_cat = AUDIO_ES;
-            p_stream->p_es = es_out_Add( p_input->p_es_out, &fmt );
-
-            msg_Dbg( p_input,
-                    "added new audio stream(codec:0x%x,ID:%d)",
-                    GetWLE( p_data ), p_sp->i_stream_number );
-        }
-        else if( ASF_CmpGUID( &p_sp->i_stream_type, &asf_object_stream_type_video ) &&
-                 p_sp->i_type_specific_data_length >= 11 + sizeof( BITMAPINFOHEADER ) )
-        {
-            es_format_t  fmt;
-            uint8_t      *p_data = &p_sp->p_type_specific_data[11];
-
-            es_format_Init( &fmt, VIDEO_ES,
-                            VLC_FOURCC( p_data[16], p_data[17], p_data[18], p_data[19] ) );
-            fmt.video.i_width = GetDWLE( p_data + 4 );
-            fmt.video.i_height= GetDWLE( p_data + 8 );
-
-            if( p_sp->i_type_specific_data_length > 11 + sizeof( BITMAPINFOHEADER ) )
-            {
-                fmt.i_extra = __MIN( GetDWLE( p_data ),
-                                     p_sp->i_type_specific_data_length - 11 - sizeof( BITMAPINFOHEADER ) );
-                fmt.p_extra = malloc( fmt.i_extra );
-                memcpy( fmt.p_extra, &p_data[sizeof( BITMAPINFOHEADER )], fmt.i_extra );
-            }
-
-            p_stream->i_cat = VIDEO_ES;
-            p_stream->p_es = es_out_Add( p_input->p_es_out, &fmt );
-
-            msg_Dbg( p_input, "added new video stream(ID:%d)",
-                     p_sp->i_stream_number );
-        }
-        else
-        {
-            p_stream->i_cat = UNKNOWN_ES;
-            msg_Dbg( p_input, "ignoring unknown stream(ID:%d)",
-                     p_sp->i_stream_number );
-        }
-    }
-
-    p_sys->i_data_begin = p_sys->p_root->p_data->i_object_pos + 50;
-    if( p_sys->p_root->p_data->i_object_size != 0 )
-    { /* local file */
-        p_sys->i_data_end = p_sys->p_root->p_data->i_object_pos +
-                                    p_sys->p_root->p_data->i_object_size;
-    }
-    else
-    { /* live/broacast */
-        p_sys->i_data_end = -1;
-    }
-
-
-    /* go to first packet */
-    stream_Seek( p_input->s, p_sys->i_data_begin );
-
-    /* try to calculate movie time */
-    if( p_sys->p_fp->i_data_packets_count > 0 )
-    {
-        int64_t i_count;
-        int64_t i_size = stream_Size( p_input->s );
-
-        if( p_sys->i_data_end > 0 && i_size > p_sys->i_data_end )
-        {
-            i_size = p_sys->i_data_end;
-        }
-
-        /* real number of packets */
-        i_count = ( i_size - p_sys->i_data_begin ) /
-                  p_sys->p_fp->i_min_data_packet_size;
-
-        /* calculate the time duration in micro-s */
-        p_sys->i_length = (mtime_t)p_sys->p_fp->i_play_duration / 10 *
-                   (mtime_t)i_count /
-                   (mtime_t)p_sys->p_fp->i_data_packets_count;
-
-        if( p_sys->i_length > 0 )
-        {
-            p_input->stream.i_mux_rate =
-                i_size / 50 * (int64_t)1000000 / p_sys->i_length;
-        }
-    }
-
-    /* Create meta informations */
-    p_sys->meta = vlc_meta_New();
-
-    if( ( p_cd = ASF_FindObject( p_sys->p_root->p_hdr,
-                                 &asf_object_content_description_guid, 0 ) ) )
-    {
-        if( p_cd->psz_title && *p_cd->psz_title )
-        {
-            vlc_meta_Add( p_sys->meta, VLC_META_TITLE, p_cd->psz_title );
-        }
-        if( p_cd->psz_author && *p_cd->psz_author )
-        {
-             vlc_meta_Add( p_sys->meta, VLC_META_AUTHOR, p_cd->psz_author );
-        }
-        if( p_cd->psz_copyright && *p_cd->psz_copyright )
-        {
-            vlc_meta_Add( p_sys->meta, VLC_META_COPYRIGHT, p_cd->psz_copyright );
-        }
-        if( p_cd->psz_description && *p_cd->psz_description )
-        {
-            vlc_meta_Add( p_sys->meta, VLC_META_DESCRIPTION, p_cd->psz_description );
-        }
-        if( p_cd->psz_rating && *p_cd->psz_rating )
-        {
-            vlc_meta_Add( p_sys->meta, VLC_META_RATING, p_cd->psz_rating );
-        }
-    }
-    for( i_stream = 0, i = 0; i < 128; i++ )
-    {
-        asf_object_codec_list_t *p_cl = ASF_FindObject( p_sys->p_root->p_hdr,
-                                                        &asf_object_codec_list_guid, 0 );
-
-        if( p_sys->stream[i] )
-        {
-            vlc_meta_t *tk = vlc_meta_New();
-            TAB_APPEND( p_sys->meta->i_track, p_sys->meta->track, tk );
-
-            if( p_cl && i_stream < p_cl->i_codec_entries_count )
-            {
-                if( p_cl->codec[i_stream].psz_name &&
-                    *p_cl->codec[i_stream].psz_name )
-                {
-                    vlc_meta_Add( tk, VLC_META_CODEC_NAME,
-                                  p_cl->codec[i_stream].psz_name );
-                }
-                if( p_cl->codec[i_stream].psz_description &&
-                    *p_cl->codec[i_stream].psz_description )
-                {
-                    vlc_meta_Add( tk, VLC_META_CODEC_DESCRIPTION,
-                                  p_cl->codec[i_stream].psz_description );
-                }
-            }
-            i_stream++;
-        }
-    }
     return VLC_SUCCESS;
-
-error:
-    ASF_FreeObjectRoot( p_input->s, p_sys->p_root );
-    free( p_sys );
-    return VLC_EGENERIC;
 }
 
 
 /*****************************************************************************
  * Demux: read packet and send them to decoders
  *****************************************************************************/
-static int Demux( input_thread_t *p_input )
+static int Demux( demux_t *p_demux )
 {
-    demux_sys_t *p_sys = p_input->p_demux_data;
-    vlc_bool_t b_play_audio;
-    int i;
-
-    /* catch seek from user */
-    if( p_input->stream.p_selected_program->i_synchro_state == SYNCHRO_REINIT )
-    {
-        int64_t i_offset;
-
-        msleep( p_input->i_pts_delay );
-
-        i_offset = stream_Tell( p_input->s ) - p_sys->i_data_begin;
-        if( i_offset  < 0 )
-        {
-            i_offset = 0;
-        }
-        if( i_offset % p_sys->p_fp->i_min_data_packet_size > 0 )
-        {
-            i_offset -= i_offset % p_sys->p_fp->i_min_data_packet_size;
-        }
-
-        if( stream_Seek( p_input->s, i_offset + p_sys->i_data_begin ) )
-        {
-            msg_Warn( p_input, "cannot resynch after seek (EOF?)" );
-            return -1;
-        }
-
-        p_sys->i_time = -1;
-        for( i = 0; i < 128 ; i++ )
-        {
-#define p_stream p_sys->stream[i]
-            if( p_stream )
-            {
-                p_stream->i_time = -1;
-            }
-#undef p_stream
-        }
-    }
-
-    /* Check if we need to send the audio data to decoder */
-    b_play_audio = !p_input->stream.control.b_mute;
+    demux_sys_t *p_sys = p_demux->p_sys;
 
     for( ;; )
     {
+        uint8_t *p_peek;
         mtime_t i_length;
         mtime_t i_time_begin = GetMoviePTS( p_sys );
         int i_result;
 
-        if( p_input->b_die )
+        if( p_demux->b_die )
         {
             break;
         }
 
-        if( ( i_result = DemuxPacket( p_input, b_play_audio ) ) <= 0 )
+        /* Check if we have concatenated files */
+        if( stream_Peek( p_demux->s, &p_peek, 16 ) == 16 )
+        {
+            guid_t guid;
+
+            ASF_GetGUID( &guid, p_peek );
+            if( ASF_CmpGUID( &guid, &asf_object_header_guid ) )
+            {
+                msg_Warn( p_demux, "Found a new ASF header" );
+                /* We end this stream */
+                DemuxEnd( p_demux );
+
+                /* And we prepare to read the next one */
+                if( DemuxInit( p_demux ) )
+                {
+                    msg_Err( p_demux, "failed to load the new header" );
+                    return 0;
+                }
+                continue;
+            }
+        }
+
+        /* Read and demux a packet */
+        if( ( i_result = DemuxPacket( p_demux ) ) <= 0 )
         {
             return i_result;
         }
@@ -425,15 +189,14 @@ static int Demux( input_thread_t *p_input )
         }
     }
 
+    /* Set the PCR */
     p_sys->i_time = GetMoviePTS( p_sys );
     if( p_sys->i_time >= 0 )
     {
-        input_ClockManageRef( p_input,
-                              p_input->stream.p_selected_program,
-                              p_sys->i_time * 9 / 100 );
+        es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_sys->i_time );
     }
 
-    return( 1 );
+    return 1;
 }
 
 /*****************************************************************************
@@ -441,42 +204,28 @@ static int Demux( input_thread_t *p_input )
  *****************************************************************************/
 static void Close( vlc_object_t * p_this )
 {
-    input_thread_t *p_input = (input_thread_t *)p_this;
-    demux_sys_t    *p_sys = p_input->p_demux_data;
-    int i_stream;
+    demux_t     *p_demux = (demux_t *)p_this;
 
-    msg_Dbg( p_input, "freeing all memory" );
+    DemuxEnd( p_demux );
 
-    vlc_meta_Delete( p_sys->meta );
-
-    ASF_FreeObjectRoot( p_input->s, p_sys->p_root );
-    for( i_stream = 0; i_stream < 128; i_stream++ )
-    {
-        asf_stream_t *p_stream = p_sys->stream[i_stream];
-
-        if( p_stream )
-        {
-            if( p_stream->p_frame )
-            {
-                block_ChainRelease( p_stream->p_frame );
-            }
-            free( p_stream );
-        }
-    }
-    free( p_sys );
+    free( p_demux->p_sys );
 }
 
 /*****************************************************************************
  * Control:
  *****************************************************************************/
-static int Control( input_thread_t *p_input, int i_query, va_list args )
+static int Control( demux_t *p_demux, int i_query, va_list args )
 {
-    demux_sys_t *p_sys = p_input->p_demux_data;
+    demux_sys_t *p_sys = p_demux->p_sys;
     int64_t     *pi64;
+    int         i;
     vlc_meta_t **pp_meta;
 
     switch( i_query )
     {
+        case DEMUX_SET_TIME:
+            return VLC_EGENERIC;
+
         case DEMUX_GET_LENGTH:
             pi64 = (int64_t*)va_arg( args, int64_t * );
             *pi64 = p_sys->i_length;
@@ -487,8 +236,19 @@ static int Control( input_thread_t *p_input, int i_query, va_list args )
             *pp_meta = vlc_meta_Duplicate( p_sys->meta );
             return VLC_SUCCESS;
 
+        case DEMUX_SET_POSITION:
+            p_sys->i_time = -1;
+            for( i = 0; i < 128 ; i++ )
+            {
+                asf_track_t *tk = p_sys->track[i];
+                if( tk ) tk->i_time = -1;
+            }
+
         default:
-            return demux_vaControlDefault( p_input, i_query, args );
+            return demux2_vaControlHelper( p_demux->s,
+                                           p_sys->i_data_begin, p_sys->i_data_end,
+                                           p_sys->i_bitrate, p_sys->p_fp->i_min_data_packet_size,
+                                           i_query, args );
     }
 }
 
@@ -498,27 +258,27 @@ static int Control( input_thread_t *p_input, int i_query, va_list args )
 static mtime_t GetMoviePTS( demux_sys_t *p_sys )
 {
     mtime_t i_time;
-    int     i_stream;
+    int     i;
 
     i_time = -1;
-    for( i_stream = 0; i_stream < 128 ; i_stream++ )
+    for( i = 0; i < 128 ; i++ )
     {
-        asf_stream_t *p_stream = p_sys->stream[i_stream];
+        asf_track_t *tk = p_sys->track[i];
 
-        if( p_stream && p_stream->p_es && p_stream->i_time > 0)
+        if( tk && tk->p_es && tk->i_time > 0)
         {
             if( i_time < 0 )
             {
-                i_time = p_stream->i_time;
+                i_time = tk->i_time;
             }
             else
             {
-                i_time = __MIN( i_time, p_stream->i_time );
+                i_time = __MIN( i_time, tk->i_time );
             }
         }
     }
 
-    return( i_time );
+    return i_time;
 }
 
 #define GETVALUE2b( bits, var, def ) \
@@ -531,21 +291,21 @@ static mtime_t GetMoviePTS( demux_sys_t *p_sys )
         default: var = def; break;\
     }
 
-static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
+static int DemuxPacket( demux_t *p_demux )
 {
-    demux_sys_t *p_sys = p_input->p_demux_data;
-    int     i_data_packet_min = p_sys->p_fp->i_min_data_packet_size;
-    uint8_t *p_peek;
-    int     i_skip;
+    demux_sys_t *p_sys = p_demux->p_sys;
+    int         i_data_packet_min = p_sys->p_fp->i_min_data_packet_size;
+    uint8_t     *p_peek;
+    int         i_skip;
 
-    int     i_packet_size_left;
-    int     i_packet_flags;
-    int     i_packet_property;
+    int         i_packet_size_left;
+    int         i_packet_flags;
+    int         i_packet_property;
 
-    int     b_packet_multiple_payload;
-    int     i_packet_length;
-    int     i_packet_sequence;
-    int     i_packet_padding_length;
+    int         b_packet_multiple_payload;
+    int         i_packet_length;
+    int         i_packet_sequence;
+    int         i_packet_padding_length;
 
     uint32_t    i_packet_send_time;
     uint16_t    i_packet_duration;
@@ -554,11 +314,10 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
     int         i_payload_length_type;
 
 
-    if( stream_Peek( p_input->s, &p_peek,i_data_packet_min)<i_data_packet_min )
+    if( stream_Peek( p_demux->s, &p_peek,i_data_packet_min)<i_data_packet_min )
     {
-        // EOF ?
-        msg_Warn( p_input, "cannot peek while getting new packet, EOF ?" );
-        return( 0 );
+        msg_Warn( p_demux, "cannot peek while getting new packet, EOF ?" );
+        return 0;
     }
     i_skip = 0;
 
@@ -585,7 +344,7 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
     }
     else
     {
-        msg_Warn( p_input, "p_peek[0]&0x80 != 0x80" );
+        msg_Warn( p_demux, "p_peek[0]&0x80 != 0x80" );
     }
 
     /* sanity check */
@@ -625,7 +384,7 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
 
     for( i_payload = 0; i_payload < i_payload_count ; i_payload++ )
     {
-        asf_stream_t   *p_stream;
+        asf_track_t   *tk;
 
         int i_stream_number;
         int i_media_object_number;
@@ -667,8 +426,7 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
         }
         else if( i_replicated_data_length == 1 )
         {
-
-            msg_Dbg( p_input, "found compressed payload" );
+            /* msg_Dbg( p_demux, "found compressed payload" ); */
 
             i_pts = (mtime_t)i_tmp * 1000;
             i_pts_delta = (mtime_t)p_peek[i_skip] * 1000; i_skip++;
@@ -698,28 +456,22 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
         {
             break;
         }
-
 #if 0
-         msg_Dbg( p_input,
+         msg_Dbg( p_demux,
                   "payload(%d/%d) stream_number:%d media_object_number:%d media_object_offset:%d replicated_data_length:%d payload_data_length %d",
-                  i_payload + 1,
-                  i_payload_count,
-                  i_stream_number,
-                  i_media_object_number,
-                  i_media_object_offset,
-                  i_replicated_data_length,
-                  i_payload_data_length );
+                  i_payload + 1, i_payload_count, i_stream_number, i_media_object_number,
+                  i_media_object_offset, i_replicated_data_length, i_payload_data_length );
 #endif
 
-        if( !( p_stream = p_sys->stream[i_stream_number] ) )
+        if( ( tk = p_sys->track[i_stream_number] ) == NULL )
         {
-            msg_Warn( p_input,
+            msg_Warn( p_demux,
                       "undeclared stream[Id 0x%x]", i_stream_number );
             i_skip += i_payload_data_length;
             continue;   // over payload
         }
 
-        if( !p_stream->p_es )
+        if( !tk->p_es )
         {
             i_skip += i_payload_data_length;
             continue;
@@ -746,38 +498,35 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
             }
 
             /* FIXME I don't use i_media_object_number, sould I ? */
-            if( p_stream->p_frame && i_media_object_offset == 0 )
+            if( tk->p_frame && i_media_object_offset == 0 )
             {
                 /* send complete packet to decoder */
-                block_t *p_gather = block_ChainGather( p_stream->p_frame );
+                block_t *p_gather = block_ChainGather( tk->p_frame );
 
-                es_out_Send( p_input->p_es_out, p_stream->p_es, p_gather );
+                es_out_Send( p_demux->out, tk->p_es, p_gather );
 
-                p_stream->p_frame = NULL;
+                tk->p_frame = NULL;
             }
 
             i_read = i_sub_payload_data_length + i_skip;
-            if( ( p_frag = stream_Block( p_input->s, i_read ) ) == NULL )
+            if( ( p_frag = stream_Block( p_demux->s, i_read ) ) == NULL )
             {
-                msg_Warn( p_input, "cannot read data" );
-                return( 0 );
+                msg_Warn( p_demux, "cannot read data" );
+                return 0;
             }
             i_packet_size_left -= i_read;
 
             p_frag->p_buffer += i_skip;
             p_frag->i_buffer -= i_skip;
 
-            if( p_stream->p_frame == NULL )
+            if( tk->p_frame == NULL )
             {
-                p_stream->i_time =
+                tk->i_time =
                     ( (mtime_t)i_pts + i_payload * (mtime_t)i_pts_delta );
 
-                p_frag->i_pts =
-                    input_ClockGetTS( p_input,
-                                      p_input->stream.p_selected_program,
-                                      p_stream->i_time * 9 /100 );
+                p_frag->i_pts = tk->i_time;
 
-                if( p_stream->i_cat != VIDEO_ES )
+                if( tk->i_cat != VIDEO_ES )
                     p_frag->i_dts = p_frag->i_pts;
                 else
                 {
@@ -786,17 +535,16 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
                 }
             }
 
-            block_ChainAppend( &p_stream->p_frame, p_frag );
+            block_ChainAppend( &tk->p_frame, p_frag );
 
             i_skip = 0;
             if( i_packet_size_left > 0 )
             {
-                if( stream_Peek( p_input->s, &p_peek, i_packet_size_left )
+                if( stream_Peek( p_demux->s, &p_peek, i_packet_size_left )
                                                          < i_packet_size_left )
                 {
-                    // EOF ?
-                    msg_Warn( p_input, "cannot peek, EOF ?" );
-                    return( 0 );
+                    msg_Warn( p_demux, "cannot peek, EOF ?" );
+                    return 0;
                 }
             }
         }
@@ -804,26 +552,344 @@ static int DemuxPacket( input_thread_t *p_input, vlc_bool_t b_play_audio )
 
     if( i_packet_size_left > 0 )
     {
-        if( stream_Read( p_input->s, NULL, i_packet_size_left )
+        if( stream_Read( p_demux->s, NULL, i_packet_size_left )
                                                          < i_packet_size_left )
         {
-            msg_Warn( p_input, "cannot skip data, EOF ?" );
-            return( 0 );
+            msg_Warn( p_demux, "cannot skip data, EOF ?" );
+            return 0;
         }
     }
 
-    return( 1 );
+    return 1;
 
 loop_error_recovery:
-    msg_Warn( p_input, "unsupported packet header" );
+    msg_Warn( p_demux, "unsupported packet header" );
     if( p_sys->p_fp->i_min_data_packet_size != p_sys->p_fp->i_max_data_packet_size )
     {
-        msg_Err( p_input, "unsupported packet header, fatal error" );
-        return( -1 );
+        msg_Err( p_demux, "unsupported packet header, fatal error" );
+        return -1;
     }
-    stream_Read( p_input->s, NULL, i_data_packet_min );
+    stream_Read( p_demux->s, NULL, i_data_packet_min );
 
-    return( 1 );
+    return 1;
 }
 
+/*****************************************************************************
+ *
+ *****************************************************************************/
+static int DemuxInit( demux_t *p_demux )
+{
+    demux_sys_t *p_sys = p_demux->p_sys;
+    vlc_bool_t  b_seekable;
+    int         i;
+
+    unsigned int    i_stream;
+    asf_object_content_description_t *p_cd;
+
+    /* init context */
+    p_sys->i_time   = -1;
+    p_sys->i_length = 0;
+    p_sys->i_bitrate = 0;
+    p_sys->p_root   = NULL;
+    p_sys->p_fp     = NULL;
+    p_sys->i_track  = 0;
+    for( i = 0; i < 128; i++ )
+    {
+        p_sys->track[i] = NULL;
+    }
+    p_sys->i_data_begin = -1;
+    p_sys->i_data_end   = -1;
+    p_sys->meta         = NULL;
+
+    /* Now load all object ( except raw data ) */
+    stream_Control( p_demux->s, STREAM_CAN_FASTSEEK, &b_seekable );
+    if( (p_sys->p_root = ASF_ReadObjectRoot( p_demux->s, b_seekable )) == NULL )
+    {
+        msg_Warn( p_demux, "ASF plugin discarded (not a valid file)" );
+        return VLC_EGENERIC;
+    }
+    p_sys->p_fp = p_sys->p_root->p_fp;
+
+    if( p_sys->p_fp->i_min_data_packet_size != p_sys->p_fp->i_max_data_packet_size )
+    {
+        msg_Warn( p_demux, "ASF plugin discarded (invalid file_properties object)" );
+        goto error;
+    }
+
+    p_sys->i_track = ASF_CountObject( p_sys->p_root->p_hdr,
+                                      &asf_object_stream_properties_guid );
+    if( p_sys->i_track <= 0 )
+    {
+        msg_Warn( p_demux, "ASF plugin discarded (cannot find any stream!)" );
+        goto error;
+    }
+
+    msg_Dbg( p_demux, "found %d streams", p_sys->i_track );
+
+    for( i_stream = 0; i_stream < p_sys->i_track; i_stream ++ )
+    {
+        asf_track_t    *tk;
+        asf_object_stream_properties_t *p_sp;
+
+        p_sp = ASF_FindObject( p_sys->p_root->p_hdr,
+                               &asf_object_stream_properties_guid,
+                               i_stream );
+
+        tk = p_sys->track[p_sp->i_stream_number] = malloc( sizeof( asf_track_t ) );
+        memset( tk, 0, sizeof( asf_track_t ) );
+
+        tk->i_time = -1;
+        tk->p_sp = p_sp;
+        tk->p_es = NULL;
+        tk->p_frame = NULL;
+
+        if( ASF_CmpGUID( &p_sp->i_stream_type, &asf_object_stream_type_audio ) &&
+            p_sp->i_type_specific_data_length >= sizeof( WAVEFORMATEX ) - 2 )
+        {
+            es_format_t fmt;
+            uint8_t *p_data = p_sp->p_type_specific_data;
+            int i_format;
+
+            es_format_Init( &fmt, AUDIO_ES, 0 );
+            i_format = GetWLE( &p_data[0] );
+            wf_tag_to_fourcc( i_format, &fmt.i_codec, NULL );
+            fmt.audio.i_channels        = GetWLE(  &p_data[2] );
+            fmt.audio.i_rate      = GetDWLE( &p_data[4] );
+            fmt.i_bitrate         = GetDWLE( &p_data[8] ) * 8;
+            fmt.audio.i_blockalign      = GetWLE(  &p_data[12] );
+            fmt.audio.i_bitspersample   = GetWLE(  &p_data[14] );
+
+            if( p_sp->i_type_specific_data_length > sizeof( WAVEFORMATEX ) &&
+                i_format != WAVE_FORMAT_MPEGLAYER3 &&
+                i_format != WAVE_FORMAT_MPEG )
+            {
+                fmt.i_extra = __MIN( GetWLE( &p_data[16] ),
+                                     p_sp->i_type_specific_data_length -
+                                     sizeof( WAVEFORMATEX ) );
+                fmt.p_extra = malloc( fmt.i_extra );
+                memcpy( fmt.p_extra, &p_data[sizeof( WAVEFORMATEX )],
+                        fmt.i_extra );
+            }
+
+            tk->i_cat = AUDIO_ES;
+            tk->p_es = es_out_Add( p_demux->out, &fmt );
+            es_format_Clean( &fmt );
+
+            msg_Dbg( p_demux, "added new audio stream(codec:0x%x,ID:%d)",
+                    GetWLE( p_data ), p_sp->i_stream_number );
+        }
+        else if( ASF_CmpGUID( &p_sp->i_stream_type,
+                              &asf_object_stream_type_video ) &&
+                 p_sp->i_type_specific_data_length >= 11 +
+                 sizeof( BITMAPINFOHEADER ) )
+        {
+            es_format_t  fmt;
+            uint8_t      *p_data = &p_sp->p_type_specific_data[11];
+
+            es_format_Init( &fmt, VIDEO_ES,
+                            VLC_FOURCC( p_data[16], p_data[17],
+                                        p_data[18], p_data[19] ) );
+            fmt.video.i_width = GetDWLE( p_data + 4 );
+            fmt.video.i_height= GetDWLE( p_data + 8 );
+
+            if( p_sp->i_type_specific_data_length > 11 +
+                sizeof( BITMAPINFOHEADER ) )
+            {
+                fmt.i_extra = __MIN( GetDWLE( p_data ),
+                                     p_sp->i_type_specific_data_length - 11 -
+                                     sizeof( BITMAPINFOHEADER ) );
+                fmt.p_extra = malloc( fmt.i_extra );
+                memcpy( fmt.p_extra, &p_data[sizeof( BITMAPINFOHEADER )],
+                        fmt.i_extra );
+            }
+
+            /* Look for an aspect ratio */
+            if( p_sys->p_root->p_metadata )
+            {
+                asf_object_metadata_t *p_meta = p_sys->p_root->p_metadata;
+                int i, i_aspect_x = 0, i_aspect_y = 0;
+
+                for( i = 0; i < p_meta->i_record_entries_count; i++ )
+                {
+                    if( !strcmp( p_meta->record[i].psz_name, "AspectRatioX" ) )
+                    {
+                        if( (!i_aspect_x && !p_meta->record[i].i_stream) ||
+                            p_meta->record[i].i_stream ==
+                            p_sp->i_stream_number )
+                            i_aspect_x = p_meta->record[i].i_val;
+                    }
+                    if( !strcmp( p_meta->record[i].psz_name, "AspectRatioY" ) )
+                    {
+                        if( (!i_aspect_y && !p_meta->record[i].i_stream) ||
+                            p_meta->record[i].i_stream ==
+                            p_sp->i_stream_number )
+                            i_aspect_y = p_meta->record[i].i_val;
+                    }
+                }
+
+                if( i_aspect_x && i_aspect_y )
+                {
+                    fmt.video.i_aspect = i_aspect_x *
+                        (int64_t)fmt.video.i_width * VOUT_ASPECT_FACTOR /
+                        fmt.video.i_height / i_aspect_y;
+                }
+	    }
+
+            tk->i_cat = VIDEO_ES;
+            tk->p_es = es_out_Add( p_demux->out, &fmt );
+            es_format_Clean( &fmt );
+
+            msg_Dbg( p_demux, "added new video stream(ID:%d)",
+                     p_sp->i_stream_number );
+        }
+        else
+        {
+            tk->i_cat = UNKNOWN_ES;
+            msg_Dbg( p_demux, "ignoring unknown stream(ID:%d)",
+                     p_sp->i_stream_number );
+        }
+    }
+
+    p_sys->i_data_begin = p_sys->p_root->p_data->i_object_pos + 50;
+    if( p_sys->p_root->p_data->i_object_size != 0 )
+    { /* local file */
+        p_sys->i_data_end = p_sys->p_root->p_data->i_object_pos +
+                                    p_sys->p_root->p_data->i_object_size;
+    }
+    else
+    { /* live/broacast */
+        p_sys->i_data_end = -1;
+    }
+
+
+    /* go to first packet */
+    stream_Seek( p_demux->s, p_sys->i_data_begin );
+
+    /* try to calculate movie time */
+    if( p_sys->p_fp->i_data_packets_count > 0 )
+    {
+        int64_t i_count;
+        int64_t i_size = stream_Size( p_demux->s );
+
+        if( p_sys->i_data_end > 0 && i_size > p_sys->i_data_end )
+        {
+            i_size = p_sys->i_data_end;
+        }
+
+        /* real number of packets */
+        i_count = ( i_size - p_sys->i_data_begin ) /
+                  p_sys->p_fp->i_min_data_packet_size;
+
+        /* calculate the time duration in micro-s */
+        p_sys->i_length = (mtime_t)p_sys->p_fp->i_play_duration / 10 *
+                   (mtime_t)i_count /
+                   (mtime_t)p_sys->p_fp->i_data_packets_count;
+
+        if( p_sys->i_length > 0 )
+        {
+            p_sys->i_bitrate = 8 * i_size * (int64_t)1000000 / p_sys->i_length;
+        }
+    }
+
+    /* Create meta information */
+    p_sys->meta = vlc_meta_New();
+
+    if( ( p_cd = ASF_FindObject( p_sys->p_root->p_hdr,
+                                 &asf_object_content_description_guid, 0 ) ) )
+    {
+        if( p_cd->psz_title && *p_cd->psz_title )
+        {
+            vlc_meta_Add( p_sys->meta, VLC_META_TITLE, p_cd->psz_title );
+        }
+        if( p_cd->psz_author && *p_cd->psz_author )
+        {
+             vlc_meta_Add( p_sys->meta, VLC_META_AUTHOR, p_cd->psz_author );
+        }
+        if( p_cd->psz_copyright && *p_cd->psz_copyright )
+        {
+            vlc_meta_Add( p_sys->meta, VLC_META_COPYRIGHT, p_cd->psz_copyright );
+        }
+        if( p_cd->psz_description && *p_cd->psz_description )
+        {
+            vlc_meta_Add( p_sys->meta, VLC_META_DESCRIPTION, p_cd->psz_description );
+        }
+        if( p_cd->psz_rating && *p_cd->psz_rating )
+        {
+            vlc_meta_Add( p_sys->meta, VLC_META_RATING, p_cd->psz_rating );
+        }
+    }
+    for( i_stream = 0, i = 0; i < 128; i++ )
+    {
+        asf_object_codec_list_t *p_cl = ASF_FindObject( p_sys->p_root->p_hdr,
+                                                        &asf_object_codec_list_guid, 0 );
+
+        if( p_sys->track[i] )
+        {
+            vlc_meta_t *tk = vlc_meta_New();
+            TAB_APPEND( p_sys->meta->i_track, p_sys->meta->track, tk );
+
+            if( p_cl && i_stream < p_cl->i_codec_entries_count )
+            {
+                if( p_cl->codec[i_stream].psz_name &&
+                    *p_cl->codec[i_stream].psz_name )
+                {
+                    vlc_meta_Add( tk, VLC_META_CODEC_NAME,
+                                  p_cl->codec[i_stream].psz_name );
+                }
+                if( p_cl->codec[i_stream].psz_description &&
+                    *p_cl->codec[i_stream].psz_description )
+                {
+                    vlc_meta_Add( tk, VLC_META_CODEC_DESCRIPTION,
+                                  p_cl->codec[i_stream].psz_description );
+                }
+            }
+            i_stream++;
+        }
+    }
+
+    es_out_Control( p_demux->out, ES_OUT_RESET_PCR );
+    return VLC_SUCCESS;
+
+error:
+    ASF_FreeObjectRoot( p_demux->s, p_sys->p_root );
+    return VLC_EGENERIC;
+}
+/*****************************************************************************
+ *
+ *****************************************************************************/
+static void DemuxEnd( demux_t *p_demux )
+{
+    demux_sys_t *p_sys = p_demux->p_sys;
+    int         i;
+
+    if( p_sys->p_root )
+    {
+        ASF_FreeObjectRoot( p_demux->s, p_sys->p_root );
+        p_sys->p_root = NULL;
+    }
+    if( p_sys->meta )
+    {
+        vlc_meta_Delete( p_sys->meta );
+        p_sys->meta = NULL;
+    }
+
+    for( i = 0; i < 128; i++ )
+    {
+        asf_track_t *tk = p_sys->track[i];
+
+        if( tk )
+        {
+            if( tk->p_frame )
+            {
+                block_ChainRelease( tk->p_frame );
+            }
+            if( tk->p_es )
+            {
+                es_out_Del( p_demux->out, tk->p_es );
+            }
+            free( tk );
+        }
+        p_sys->track[i] = 0;
+    }
+}
 
