@@ -2,7 +2,7 @@
  * libavi.h : LibAVI library
  ******************************************************************************
  * Copyright (C) 2001-2003 VideoLAN
- * $Id: libavi.h,v 1.15 2004/02/08 18:30:30 sigmunau Exp $
+ * $Id: libavi.h 9270 2004-11-10 13:05:27Z gbazin $
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -224,10 +224,7 @@ int     AVI_ChunkReadRoot( stream_t *, avi_chunk_t *p_root );
 void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
 
 #define AVI_ChunkRead( s, p_chk, p_father ) \
-    _AVI_ChunkRead( s, \
-                    (avi_chunk_t*)p_chk, \
-                    (avi_chunk_t*)p_father )
-
+    _AVI_ChunkRead( s, p_chk, (avi_chunk_t*)p_father )
 #define AVI_ChunkCount( p_chk, i_fourcc ) \
     _AVI_ChunkCount( (avi_chunk_t*)p_chk, i_fourcc )
 #define AVI_ChunkFind( p_chk, i_fourcc, i_number ) \
@@ -259,6 +256,8 @@ void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
 #define AVIFOURCC_rec          VLC_FOURCC('r','e','c',' ')
 #define AVIFOURCC_auds         VLC_FOURCC('a','u','d','s')
 #define AVIFOURCC_vids         VLC_FOURCC('v','i','d','s')
+#define AVIFOURCC_txts         VLC_FOURCC('t','x','t','s')
+#define AVIFOURCC_mids         VLC_FOURCC('m','i','d','s')
 
 #define AVIFOURCC_IARL         VLC_FOURCC('I','A','R','L')
 #define AVIFOURCC_IART         VLC_FOURCC('I','A','R','T')
@@ -357,4 +356,7 @@ void    AVI_ChunkFreeRoot( stream_t *, avi_chunk_t  *p_chk );
 #define FOURCC_I263         VLC_FOURCC('I','2','6','3')
 #define FOURCC_i263         VLC_FOURCC('i','2','6','3')
 
-
+    /* H264 */
+#define FOURCC_H264         VLC_FOURCC('H','2','6','4')
+#define FOURCC_h264         VLC_FOURCC('h','2','6','4')
+#define FOURCC_VSSH         VLC_FOURCC('V','S','S','H')
