@@ -2,7 +2,7 @@
  * m3u.c : M3U playlist format import
  *****************************************************************************
  * Copyright (C) 2004 VideoLAN
- * $Id: m3u.c 7716 2004-05-18 20:41:19Z zorglub $
+ * $Id: m3u.c 8157 2004-07-09 15:15:07Z gbazin $
  *
  * Authors: Clment Stenac <zorglub@videolan.org>
  * Authors: Sigmund Augdal <sigmunau@idi.ntnu.no>
@@ -28,6 +28,7 @@
 #include <stdlib.h>                                      /* malloc(), free() */
 
 #include <vlc/vlc.h>
+#include <vlc/input.h>
 #include <vlc/intf.h>
 
 #include <errno.h>                                                 /* ENOMEM */
@@ -36,8 +37,6 @@
 struct demux_sys_t
 {
     char *psz_prefix;
-    char **ppsz_options;
-    int i_options;
 };
 
 /*****************************************************************************
@@ -58,7 +57,6 @@ int Import_M3U( vlc_object_t *p_this )
 
     if( stream_Peek( p_demux->s , &p_peek, 7 ) < 7 )
     {
-        msg_Err( p_demux, "cannot peek" );
         return VLC_EGENERIC;
     }
     psz_ext = strrchr ( p_demux->psz_path, '.' );
@@ -68,13 +66,14 @@ int Import_M3U( vlc_object_t *p_this )
         ;
     }
     else if( ( psz_ext && !strcasecmp( psz_ext, ".m3u") ) ||
+             ( psz_ext && !strcasecmp( psz_ext, ".ram") ) ||
+             /* A .ram file can contain a single rtsp link */
              ( p_demux->psz_demux && !strcmp(p_demux->psz_demux, "m3u") ) )
     {
         ;
     }
     else
     {
-        msg_Warn(p_demux, "m3u import module discarded");
         return VLC_EGENERIC;
     }
     msg_Dbg( p_demux, "found valid M3U playlist file");
@@ -89,9 +88,6 @@ int Import_M3U( vlc_object_t *p_this )
     }
     p_demux->p_sys->psz_prefix = FindPrefix( p_demux );
 
-    p_demux->p_sys->ppsz_options = NULL;
-    p_demux->p_sys->i_options = 0 ;
-
     return VLC_SUCCESS;
 }
 
@@ -101,26 +97,23 @@ int Import_M3U( vlc_object_t *p_this )
 void Close_M3U( vlc_object_t *p_this )
 {
     demux_t *p_demux = (demux_t *)p_this;
-    if( p_demux->p_sys->psz_prefix )
-    {
-        free( p_demux->p_sys->psz_prefix );
-    }
+
+    if( p_demux->p_sys->psz_prefix ) free( p_demux->p_sys->psz_prefix );
     free( p_demux->p_sys );
 }
 
-
 static int Demux( demux_t *p_demux )
 {
-    mtime_t        i_duration = -1;
-    char          *psz_name = NULL;
-    char          *psz_line;
-    char          *psz_parse;
-    char          *psz_duration;
-    char          *psz_mrl;
-    playlist_t    *p_playlist;
-    int            i_position;
-    char          *psz_option = NULL;
-    int            i;
+    playlist_t *p_playlist;
+    char       *psz_line;
+    int        i_position;
+
+    char *psz_name = NULL;
+    mtime_t i_duration = -1;
+    char **ppsz_options = NULL;
+    int i_options = 0;
+
+    vlc_bool_t b_cleanup = VLC_FALSE;
 
     p_playlist = (playlist_t *) vlc_object_find( p_demux, VLC_OBJECT_PLAYLIST,
                                                  FIND_PARENT );
@@ -130,89 +123,101 @@ static int Demux( demux_t *p_demux )
         return -1;
     }
 
+    vlc_mutex_lock( &p_playlist->object_lock );
     p_playlist->pp_items[p_playlist->i_index]->b_autodeletion = VLC_TRUE;
     i_position = p_playlist->i_index + 1;
-    while( ( psz_line = stream_ReadLine( p_demux->s ) ) )
+    vlc_mutex_unlock( &p_playlist->object_lock );
+
+    psz_line = stream_ReadLine( p_demux->s );
+    while( psz_line )
     {
+        char *psz_parse = psz_line;
 
         /* Skip leading tabs and spaces */
-        while( *psz_line == ' ' || *psz_line == '\t' ||
-               *psz_line == '\n' || *psz_line == '\r' )
-        {
-            psz_line++;
-        }
+        while( *psz_parse == ' ' || *psz_parse == '\t' ||
+               *psz_parse == '\n' || *psz_parse == '\r' ) psz_parse++;
 
-        if( *psz_line == '#' )
+        if( *psz_parse == '#' )
         {
-            /* parse extra info */
-            psz_parse = psz_line;
-            while( *psz_parse &&
-                  strncasecmp( psz_parse, "EXTINF:", sizeof("EXTINF:") - 1 ) &&
-                  strncasecmp( psz_parse, "EXTVLCOPT:",sizeof("EXTVLCOPT:") -1))
-               psz_parse++;
-            if( *psz_parse )
+            /* Parse extra info */
+
+            /* Skip leading tabs and spaces */
+            while( *psz_parse == ' ' || *psz_parse == '\t' || 
+                   *psz_parse == '\n' || *psz_parse == '\r' ||
+                   *psz_parse == '#' ) psz_parse++;
+
+            if( !*psz_parse ) goto error;
+
+            if( !strncasecmp( psz_parse, "EXTINF:", sizeof("EXTINF:") -1 ) )
             {
-                if( !strncasecmp( psz_parse, "EXTINF:", sizeof("EXTINF:") - 1 ) )
+                /* Extended info */
+                char *psz_duration;
+                psz_parse += sizeof("EXTINF:") - 1;
+                while( *psz_parse == '\t' || *psz_parse == ' ' ) psz_parse++;
+
+                psz_duration = psz_parse;
+                psz_parse = strchr( psz_parse, ',' );
+                if( psz_parse )
                 {
-                    psz_parse += sizeof("EXTINF:") - 1;
-                    while( *psz_parse == '\t' || *psz_parse == ' ' )
-                        psz_parse++;
-                    psz_duration = psz_parse;
-                    psz_parse = strchr( psz_parse, ',' );
-                    if ( psz_parse )
-                    {
-                        i_duration *= 1000000;
-                        *psz_parse = '\0';
-                        psz_parse++;
-                        psz_name = strdup( psz_parse );
-                        i_duration = atoi( psz_duration );
-                        if( i_duration != -1 )
-                        {
-                            i_duration *= 1000000;
-                        }
-                    }
-                }
-                else
-                {
-                    /* Option line */
-                    psz_parse = strchr( psz_parse, ':' );
-                    if( !psz_parse ) return 0;
+                    *psz_parse = '\0';
                     psz_parse++;
-
-                    psz_option = strdup( psz_parse );
-                    INSERT_ELEM( p_demux->p_sys->ppsz_options,
-                                 p_demux->p_sys->i_options,
-                                 p_demux->p_sys->i_options,
-                                 psz_option );
+                    psz_name = strdup( psz_parse );
+                    i_duration = atoi( psz_duration );
+                    if( i_duration != -1 ) i_duration *= 1000000;
                 }
             }
+            else if( !strncasecmp( psz_parse, "EXTVLCOPT:",
+                                   sizeof("EXTVLCOPT:") -1 ) )
+
+            {
+                /* VLC Option */
+                char *psz_option;
+                psz_parse += sizeof("EXTVLCOPT:") -1;
+                if( !*psz_parse ) goto error;
+
+                psz_option = strdup( psz_parse );
+                if( psz_option )
+                    INSERT_ELEM( ppsz_options, i_options, i_options,
+                                 psz_option );
+            }
         }
-        else
+        else if( *psz_parse )
         {
-            psz_mrl = ProcessMRL( psz_line, p_demux->p_sys->psz_prefix );
+            char *psz_mrl =
+                ProcessMRL( psz_parse, p_demux->p_sys->psz_prefix );
+
+            b_cleanup = VLC_TRUE;
+            if( !psz_mrl ) goto error;
+
             playlist_AddExt( p_playlist, psz_mrl, psz_name,
-                          PLAYLIST_INSERT, i_position, i_duration,
-                          p_demux->p_sys->ppsz_options,
-                          p_demux->p_sys->i_options );
-            for( i = 0 ; i < p_demux->p_sys->i_options ; i++)
-            {
-                char *psz_option = p_demux->p_sys->ppsz_options[i];
-                REMOVE_ELEM( p_demux->p_sys->ppsz_options,
-                            p_demux->p_sys->i_options,
-                            i );
-                if( psz_option ) free( psz_option );
-            }
-            free( psz_mrl );
+                             PLAYLIST_INSERT, i_position, i_duration,
+                             (const char **)ppsz_options, i_options );
+
             i_position++;
+            free( psz_mrl );
+        }
+
+ error:
+
+        /* Fetch another line */
+        free( psz_line );
+        psz_line = stream_ReadLine( p_demux->s );
+        if( !psz_line ) b_cleanup = VLC_TRUE;
+
+        if( b_cleanup )
+        {
+            /* Cleanup state */
+            while( i_options-- ) free( ppsz_options[i_options] );
+            if( ppsz_options ) free( ppsz_options );
+            ppsz_options = NULL; i_options = 0;
+            if( psz_name ) free( psz_name );
+            psz_name = NULL;
             i_duration = -1;
-            if( psz_name )
-            {
-                free( psz_name );
-                psz_name = NULL;
-            }
+
+            b_cleanup = VLC_FALSE;
         }
-        free( psz_line);
     }
+
     vlc_object_release( p_playlist );
     return VLC_SUCCESS;
 }
