@@ -1,11 +1,11 @@
 /*****************************************************************************
- * prefs.m: MacOS X plugin for vlc
+ * prefs.m: MacOS X module for vlc
  *****************************************************************************
- * Copyright (C) 2002-2003 VideoLAN
- * $Id: prefs.m,v 1.36 2003/12/22 14:32:56 sam Exp $
+ * Copyright (C) 2002-2004 VideoLAN
+ * $Id: prefs.m 8571 2004-08-29 15:11:50Z hartman $
  *
  * Authors: Jon Lech Johansen <jon-vl@nanocrew.net>
- *          Derk-Jan Hartman <thedj at users.sf.net>
+ *          Derk-Jan Hartman <hartman at videolan dot org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -60,7 +60,7 @@
 
 - (void)awakeFromNib
 {
-    p_intf = [NSApp getIntf];
+    p_intf = VLCIntf;
     b_advanced = config_GetInt( p_intf, "advanced" );
 
     [self initStrings];
@@ -125,12 +125,21 @@
             break;
     
         case CONFIG_ITEM_STRING:
+            {
+                char *psz_value;
+                NSString *o_value = [o_vlc_config stringValue];
+                psz_value = (char *)[o_value UTF8String];
+    
+                config_PutPsz( p_intf, psz_name, psz_value );
+            }
+            break;
+
         case CONFIG_ITEM_FILE:
         case CONFIG_ITEM_DIRECTORY:
             {
                 char *psz_value;
                 NSString *o_value = [o_vlc_config stringValue];
-                psz_value = (char *)[o_value UTF8String];
+                psz_value = (char *)[o_value fileSystemRepresentation];
     
                 config_PutPsz( p_intf, psz_name, psz_value );
             }
@@ -204,7 +213,7 @@
 {
     NSBeginInformationalAlertSheet(_NS("Reset Preferences"), _NS("Cancel"), _NS("Continue"), 
         nil, o_prefs_window, self, @selector(sheetDidEnd: returnCode: contextInfo:), NULL, nil,
-        _NS("Beware this will reset your VLC Media Player config file.\n"
+        _NS("Beware this will reset your VLC media player preferences.\n"
             "Are you sure you want to continue?") );
 }
 
@@ -302,6 +311,7 @@
     if( p_parser->i_object_type != VLC_OBJECT_MODULE )
     {
         /* 0OOoo something went really bad */
+        vlc_list_release( p_list );
         return;
     }
     
@@ -338,7 +348,7 @@
         if ( label ) \
         { \
             [o_text_field setStringValue: \
-                [NSApp localizedString: label]]; \
+                [[VLCMain sharedInstance] localizedString: label]]; \
         } \
         [o_text_field sizeToFit]; \
         [o_view addSubview: [o_text_field autorelease]]; \
@@ -359,7 +369,7 @@
         [o_text_field msg: param]; \
         if ( psz_duptip != NULL ) \
         { \
-            [o_text_field setToolTip: [NSApp wrapString: [NSApp localizedString: \
+            [o_text_field setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: \
                                        psz_duptip] toWidth: PREFS_WRAP ]]; \
             free(psz_duptip);\
         } \
@@ -386,14 +396,14 @@
     [o_view setAutoresizingMask: NSViewWidthSizable];
     s_rc.origin.x = X_ORIGIN;
     s_rc.origin.y = Y_ORIGIN;
-    BOOL b_right_cat = FALSE;
+    BOOL b_right_cat = TRUE;
 
     if( p_item ) do
     {
         if( p_item->i_type == CONFIG_HINT_CATEGORY )
         {
             if( !strcmp( p_parser->psz_object_name, "main" ) &&
-                [o_item_name isEqualToString: [NSApp localizedString: p_item->psz_text]] )
+                [o_item_name isEqualToString: [[VLCMain sharedInstance] localizedString: p_item->psz_text]] )
             {
                 b_right_cat = TRUE;
             } else if( strcmp( p_parser->psz_object_name, "main" ) )
@@ -435,7 +445,7 @@
                 
                 if ( psz_duptip != NULL )
                 {
-                    [o_modules setToolTip: [NSApp wrapString: [NSApp localizedString: psz_duptip] toWidth: PREFS_WRAP]];
+                    [o_modules setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: psz_duptip] toWidth: PREFS_WRAP]];
                     free( psz_duptip );
                 }
                 [o_view addSubview: [o_modules autorelease]];
@@ -453,7 +463,7 @@
                         if( !strcmp( p_a_module->psz_capability,
                                     p_item->psz_type ) )
                         {
-                            NSString *o_description = [NSApp
+                            NSString *o_description = [[VLCMain sharedInstance]
                                 localizedString: p_a_module->psz_longname];
                             [o_modules addItemWithTitle: o_description];
                             [[o_modules lastItem] setTag: p_a_module->i_object_id];
@@ -510,10 +520,10 @@
                 o_text_field = [[VLCTextField alloc] initWithFrame: s_rc];
                 CONTROL_CONFIG( o_text_field, o_module_name, CONFIG_ITEM_STRING , p_item->psz_name );
 
-                [o_text_field setStringValue: [NSApp localizedString: psz_value]];
+                [o_text_field setStringValue: [[VLCMain sharedInstance] localizedString: psz_value]];
                 if ( psz_duptip != NULL )
                 {
-                    [o_text_field setToolTip: [NSApp wrapString: [NSApp localizedString:
+                    [o_text_field setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString:
                                             psz_duptip] toWidth: PREFS_WRAP ]];
                     free(psz_duptip);
                 }
@@ -540,7 +550,7 @@
                                     p_item->psz_value : "";
     
                     INPUT_FIELD_STRING( p_item->psz_name, p_item->psz_text, 200,
-                                        [NSApp localizedString: psz_value],
+                                        [[VLCMain sharedInstance] localizedString: psz_value],
                                         p_item->psz_longtext );
                 }
                 else
@@ -570,7 +580,7 @@
 
                     if ( psz_duptip != NULL )
                     {
-                        [o_combo_box setToolTip: [NSApp wrapString: [NSApp localizedString: psz_duptip] toWidth: PREFS_WRAP]];
+                        [o_combo_box setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: psz_duptip] toWidth: PREFS_WRAP]];
                         free( psz_duptip );
                     }
                     [o_view addSubview: [o_combo_box autorelease]];
@@ -578,9 +588,9 @@
                     for( i=0; p_item->ppsz_list[i]; i++ )
                     {
                         [o_combo_box addItemWithObjectValue:
-                            [NSApp localizedString: p_item->ppsz_list[i]]];
+                            [[VLCMain sharedInstance] localizedString: p_item->ppsz_list[i]]];
                     }
-                    [o_combo_box setStringValue: [NSApp localizedString: 
+                    [o_combo_box setStringValue: [[VLCMain sharedInstance] localizedString: 
                         p_item->psz_value ? p_item->psz_value : ""]];
     
                     CONTROL_LABEL( p_item->psz_text );
@@ -620,7 +630,7 @@
 
                     if ( psz_duptip != NULL )
                     {
-                        [o_slider setToolTip: [NSApp wrapString: [NSApp localizedString: psz_duptip] toWidth: PREFS_WRAP]];
+                        [o_slider setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: psz_duptip] toWidth: PREFS_WRAP]];
                         free( psz_duptip );
                     }
                     [o_slider setTarget: self];
@@ -665,7 +675,7 @@
 
                     if ( psz_duptip != NULL )
                     {
-                        [o_slider setToolTip: [NSApp wrapString: [NSApp localizedString: psz_duptip] toWidth: PREFS_WRAP]];
+                        [o_slider setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: psz_duptip] toWidth: PREFS_WRAP]];
                         free( psz_duptip );
                     }
                     [o_slider setTarget: self];
@@ -699,10 +709,10 @@
                 o_btn_bool = [[VLCButton alloc] initWithFrame: s_rc];
                 [o_btn_bool setButtonType: NSSwitchButton];
                 [o_btn_bool setIntValue: p_item->i_value];
-                [o_btn_bool setTitle: [NSApp localizedString: p_item->psz_text]];
+                [o_btn_bool setTitle: [[VLCMain sharedInstance] localizedString: p_item->psz_text]];
                 if ( psz_duptip != NULL )
                 {
-                    [o_btn_bool setToolTip: [NSApp wrapString: [NSApp localizedString: psz_duptip] toWidth: PREFS_WRAP]];
+                    [o_btn_bool setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: psz_duptip] toWidth: PREFS_WRAP]];
                     free( psz_duptip );
                 }
                 [o_btn_bool setTarget: self];
@@ -742,7 +752,7 @@
                     [o_current_cell setControlSize: NSSmallControlSize];
                     if( psz_duptip != NULL )
                     {
-                        [o_matrix setToolTip: [NSApp wrapString: [NSApp localizedString: psz_duptip] toWidth: PREFS_WRAP] forCell: o_current_cell];
+                        [o_matrix setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: psz_duptip] toWidth: PREFS_WRAP] forCell: o_current_cell];
                     }
                     switch( i )
                     {
@@ -791,7 +801,7 @@
 
                 if ( psz_duptip != NULL )
                 {
-                    [o_combo_box setToolTip: [NSApp wrapString: [NSApp localizedString: psz_duptip] toWidth: PREFS_WRAP]];
+                    [o_combo_box setToolTip: [[VLCMain sharedInstance] wrapString: [[VLCMain sharedInstance] localizedString: psz_duptip] toWidth: PREFS_WRAP]];
                 }
                 [o_view addSubview: [o_combo_box autorelease]];
                 
@@ -799,10 +809,10 @@
                 {
                     
                     if( vlc_keys[i].psz_key_string && *vlc_keys[i].psz_key_string )
-                    [o_combo_box addItemWithObjectValue: [NSApp localizedString:vlc_keys[i].psz_key_string]];
+                    [o_combo_box addItemWithObjectValue: [[VLCMain sharedInstance] localizedString:vlc_keys[i].psz_key_string]];
                 }
                 
-                [o_combo_box setStringValue: [NSApp localizedString:KeyToString(( ((unsigned int)p_item->i_value) & ~KEY_MODIFIER ))]];
+                [o_combo_box setStringValue: [[VLCMain sharedInstance] localizedString:KeyToString(( ((unsigned int)p_item->i_value) & ~KEY_MODIFIER ))]];
                 
                 s_rc.origin.y += s_rc.size.height;
                 s_rc.origin.x = X_ORIGIN;
@@ -822,6 +832,7 @@
     #undef X_ORIGIN
         }
         while( p_item->i_type != CONFIG_HINT_END && p_item++ );
+        vlc_object_release( p_parser );
         vlc_list_release( p_list );
     
     [o_prefs_view setDocumentView: o_view];
@@ -886,13 +897,13 @@ static VLCTreeItem *o_root_item = nil;
  * Loads children incrementally */
 - (NSArray *)children {
     if (o_children == NULL) {
-        intf_thread_t *p_intf = [NSApp getIntf];
+        intf_thread_t *p_intf = VLCIntf;
         vlc_list_t      *p_list;
         module_t        *p_module = NULL;
         module_config_t *p_item;
         int i_index,j;
 
-        /* List the plugins */
+        /* List the modules */
         p_list = vlc_list_find( p_intf, VLC_OBJECT_MODULE, FIND_ANYWHERE );
         if( !p_list ) return nil;
 
@@ -909,7 +920,7 @@ static VLCTreeItem *o_root_item = nil;
             }
             if( p_module == NULL )
             {
-                msg_Err( p_intf, "Could not find the main module in our prefs" );
+                msg_Err( p_intf, "could not find the main module in our preferences" );
                 return nil;
             }
             if( i_index < p_list->i_count )
@@ -928,7 +939,7 @@ static VLCTreeItem *o_root_item = nil;
                     switch( p_item->i_type )
                     {
                     case CONFIG_HINT_CATEGORY:
-                        o_child_name = [NSApp localizedString: p_item->psz_text];
+                        o_child_name = [[VLCMain sharedInstance] localizedString: p_item->psz_text];
                         [o_children addObject:[[VLCTreeItem alloc] initWithName: o_child_name
                             ID: p_module->i_object_id parent:self]];
                         break;
@@ -936,7 +947,7 @@ static VLCTreeItem *o_root_item = nil;
                 }
                 while( p_item->i_type != CONFIG_HINT_END && p_item++ );
                 
-                /* Add the plugins item */
+                /* Add the modules item */
                 [o_children addObject:[[VLCTreeItem alloc] initWithName: _NS("Modules")
                     ID: 0 parent:self]];
             }
@@ -957,7 +968,7 @@ static VLCTreeItem *o_root_item = nil;
                 if( !strcmp( p_module->psz_object_name, "main" ) )
                     continue;
         
-                /* Exclude empty plugins */
+                /* Exclude empty modules */
                 p_item = p_module->p_config;
                 if( !p_item ) continue;
                 do
@@ -970,7 +981,7 @@ static VLCTreeItem *o_root_item = nil;
         
                 /* Create the capability tree if it doesn't already exist */
                 NSString *o_capability;
-                o_capability = [NSApp localizedString: p_module->psz_capability];
+                o_capability = [[VLCMain sharedInstance] localizedString: p_module->psz_capability];
                 if( !p_module->psz_capability || !*p_module->psz_capability )
                 {
                     /* Empty capability ? Let's look at the submodules */
@@ -980,7 +991,7 @@ static VLCTreeItem *o_root_item = nil;
                         p_submodule = (module_t*)p_module->pp_children[ j ];
                         if( p_submodule->psz_capability && *p_submodule->psz_capability )
                         {
-                            o_capability = [NSApp localizedString: p_submodule->psz_capability];
+                            o_capability = [[VLCMain sharedInstance] localizedString: p_submodule->psz_capability];
                             BOOL b_found = FALSE;
                             for( j = 0; j < (int)[o_children count]; j++ )
                             {
@@ -1027,7 +1038,7 @@ static VLCTreeItem *o_root_item = nil;
                 if( !strcmp( p_module->psz_object_name, "main" ) )
                     continue;
         
-                /* Exclude empty plugins */
+                /* Exclude empty modules */
                 p_item = p_module->p_config;
                 if( !p_item ) continue;
                 do
@@ -1040,7 +1051,7 @@ static VLCTreeItem *o_root_item = nil;
         
                 /* Check the capability */
                 NSString *o_capability;
-                o_capability = [NSApp localizedString: p_module->psz_capability];
+                o_capability = [[VLCMain sharedInstance] localizedString: p_module->psz_capability];
                 if( !p_module->psz_capability || !*p_module->psz_capability )
                 {
                     /* Empty capability ? Let's look at the submodules */
@@ -1050,11 +1061,11 @@ static VLCTreeItem *o_root_item = nil;
                         p_submodule = (module_t*)p_module->pp_children[ j ];
                         if( p_submodule->psz_capability && *p_submodule->psz_capability )
                         {
-                            o_capability = [NSApp localizedString: p_submodule->psz_capability];
+                            o_capability = [[VLCMain sharedInstance] localizedString: p_submodule->psz_capability];
                             if( [o_capability isEqualToString: [self getName]] )
                             {
                             [o_children addObject:[[VLCTreeItem alloc] initWithName:
-                                [NSApp localizedString: p_module->psz_object_name ]
+                                [[VLCMain sharedInstance] localizedString: p_module->psz_object_name ]
                                 ID: p_module->i_object_id parent:self]];
                             }
                         }
@@ -1063,7 +1074,7 @@ static VLCTreeItem *o_root_item = nil;
                 else if( [o_capability isEqualToString: [self getName]] )
                 {
                     [o_children addObject:[[VLCTreeItem alloc] initWithName:
-                        [NSApp localizedString: p_module->psz_object_name ]
+                        [[VLCMain sharedInstance] localizedString: p_module->psz_object_name ]
                         ID: p_module->i_object_id parent:self]];
                 }
             }
@@ -1073,6 +1084,7 @@ static VLCTreeItem *o_root_item = nil;
             /* all the other stuff are leafs */
             o_children = IsALeafNode;
         }
+        vlc_list_release( p_list );
     }
     return o_children;
 }
@@ -1098,7 +1110,7 @@ static VLCTreeItem *o_root_item = nil;
 
 - (BOOL)hasPrefs:(NSString *)o_module_name
 {
-    intf_thread_t *p_intf = [NSApp getIntf];
+    intf_thread_t *p_intf = VLCIntf;
     module_t *p_parser;
     vlc_list_t *p_list;
     char *psz_module_name;
