@@ -2,7 +2,7 @@
  * cmd_input.cpp
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: cmd_input.cpp 7707 2004-05-17 20:48:39Z ipkiss $
+ * $Id: cmd_input.cpp 7769 2004-05-24 21:48:56Z ipkiss $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -24,6 +24,7 @@
 
 #include <vlc/aout.h>
 #include "cmd_input.hpp"
+#include "cmd_dialogs.hpp"
 
 
 void CmdPlay::execute()
@@ -34,7 +35,16 @@ void CmdPlay::execute()
         return;
     }
 
-    playlist_Play( pPlaylist );
+    if( pPlaylist->i_size )
+    {
+        playlist_Play( pPlaylist );
+    }
+    else
+    {
+        // If the playlist is empty, open a file requester instead
+        CmdDlgFile cmd( getIntf() );
+        cmd.execute();
+    }
 }
 
 
