@@ -2,7 +2,7 @@
  * cmd_input.hpp
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: cmd_input.hpp,v 1.3 2004/01/25 17:20:18 kuehne Exp $
+ * $Id: cmd_input.hpp 7707 2004-05-17 20:48:39Z ipkiss $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -28,11 +28,12 @@
 #include "cmd_generic.hpp"
 
 /// Commands to control the input
-DEFINE_COMMAND( Play, "Play" ) 
+DEFINE_COMMAND( Play, "play" )
 DEFINE_COMMAND( Pause, "pause" )
-DEFINE_COMMAND( Stop, "Stop" ) 
+DEFINE_COMMAND( Stop, "stop" )
 DEFINE_COMMAND( Slower, "slower" )
 DEFINE_COMMAND( Faster, "faster" )
+DEFINE_COMMAND( Mute, "mute" )
 
 
 #endif
