@@ -2,7 +2,7 @@
  * ctrl_list.cpp
  *****************************************************************************
  * Copyright (C) 2003 VideoLAN
- * $Id: ctrl_list.cpp,v 1.3 2004/02/29 16:49:55 asmax Exp $
+ * $Id: ctrl_list.cpp 8493 2004-08-22 10:38:26Z asmax $
  *
  * Authors: Cyril Deguet     <asmax@via.ecp.fr>
  *          Olivier Teulire <ipkiss@via.ecp.fr>
@@ -34,8 +34,12 @@
 #include "../events/evt_mouse.hpp"
 #include "../events/evt_scroll.hpp"
 #include "vlc_keys.h"
+#ifdef sun
+#   include "solaris_specific.h" // for lrint
+#endif
 
 #define SCROLL_STEP 0.05
+#define LINE_INTERVAL 1  // Number of pixels inserted between 2 lines
 
 
 CtrlList::CtrlList( intf_thread_t *pIntf, VarList &rList, GenericFont &rFont,
@@ -68,8 +72,7 @@ CtrlList::~CtrlList()
 
 void CtrlList::onUpdate( Subject<VarList> &rList )
 {
-    makeImage();
-    notifyLayout();
+    autoScroll();
     m_pLastSelected = NULL;
 }
 
@@ -85,7 +88,7 @@ void CtrlList::onUpdate( Subject<VarPercent> &rPercent )
     int height = pPos->getHeight();
 
     // How many lines can be displayed ?
-    int itemHeight = m_rFont.getSize();
+    int itemHeight = m_rFont.getSize() + LINE_INTERVAL;
     int maxItems = height / itemHeight;
 
     // Determine what is the first item to display
@@ -121,7 +124,7 @@ void CtrlList::onResize()
     int height = pPos->getHeight();
 
     // How many lines can be displayed ?
-    int itemHeight = m_rFont.getSize();
+    int itemHeight = m_rFont.getSize() + LINE_INTERVAL;
     int maxItems = height / itemHeight;
 
     // Update the position variable
@@ -158,15 +161,57 @@ void CtrlList::handleEvent( EvtGeneric &rEvent )
 {
     if( rEvent.getAsString().find( "key:down" ) != string::npos )
     {
-        char key = ((EvtKey&)rEvent).getKey();
+        int key = ((EvtKey&)rEvent).getKey();
+        VarList::Iterator it = m_rList.begin();
+        bool previousWasSelected = false;
+        while( it != m_rList.end() )
+        {
+            VarList::Iterator next = it;
+            ++next;
+            if( key == KEY_UP )
+            {
+                // Scroll up one item
+                if( it != m_rList.begin() || &*it != m_pLastSelected )
+                {
+                    bool nextWasSelected = ( &*next == m_pLastSelected );
+                    (*it).m_selected = nextWasSelected;
+                    if( nextWasSelected )
+                    {
+                        m_pLastSelected = &*it;
+                    }
+                }
+            }
+            else if( key == KEY_DOWN )
+            {
+                // Scroll down one item
+                if( next != m_rList.end() || &*it != m_pLastSelected )
+                {
+                    (*it).m_selected = previousWasSelected;
+                }
+                if( previousWasSelected )
+                {
+                    m_pLastSelected = &*it;
+                    previousWasSelected = false;
+                }
+                else
+                {
+                    previousWasSelected = ( &*it == m_pLastSelected );
+                }
+            }
+            it = next;
+        }
+
+        // Redraw the control
+        makeImage();
+        notifyLayout();
     }
 
     else if( rEvent.getAsString().find( "mouse:left" ) != string::npos )
     {
         EvtMouse &rEvtMouse = (EvtMouse&)rEvent;
         const Position *pos = getPosition();
-        int yPos = m_lastPos +
-                ( rEvtMouse.getYPos() - pos->getTop() ) / m_rFont.getSize();
+        int yPos = m_lastPos + ( rEvtMouse.getYPos() - pos->getTop() ) /
+            (m_rFont.getSize() + LINE_INTERVAL);
         VarList::Iterator it;
         int index = 0;
 
@@ -320,6 +365,47 @@ void CtrlList::draw( OSGraphics &rImage, int xDest, int yDest )
 }
 
 
+void CtrlList::autoScroll()
+{
+    // Get the size of the control
+    const Position *pPos = getPosition();
+    if( !pPos )
+    {
+        return;
+    }
+    int height = pPos->getHeight();
+
+    // How many lines can be displayed ?
+    int itemHeight = m_rFont.getSize() + LINE_INTERVAL;
+    int maxItems = height / itemHeight;
+
+    // Find the current playing stream
+    int playIndex = 0;
+    VarList::ConstIterator it;
+    for( it = m_rList.begin(); it != m_rList.end(); it++ )
+    {
+        if( (*it).m_playing )
+        {
+            break;
+        }
+        playIndex++;
+    }
+    if( it != m_rList.end() &&
+        ( playIndex < m_lastPos || playIndex >= m_lastPos + maxItems ) )
+    {
+        // Scroll the list to have the playing stream visible
+        VarPercent &rVarPos = m_rList.getPositionVar();
+        rVarPos.set( 1.0 - (float)playIndex / (float)m_rList.size() );
+        // The image will be changed by onUpdate(VarPercent&)
+    }
+    else
+    {
+        makeImage();
+        notifyLayout();
+    }
+}
+
+
 void CtrlList::makeImage()
 {
     if( m_pImage )
@@ -335,7 +421,7 @@ void CtrlList::makeImage()
     }
     int width = pPos->getWidth();
     int height = pPos->getHeight();
-    int itemHeight = m_rFont.getSize();
+    int itemHeight = m_rFont.getSize() + LINE_INTERVAL;
 
     // Create an image
     OSFactory *pOsFactory = OSFactory::instance( getIntf() );
@@ -372,6 +458,10 @@ void CtrlList::makeImage()
 
         // Draw the text
         GenericBitmap *pText = m_rFont.drawString( *pStr, color, width );
+        if( !pText )
+        {
+            return;
+        }
         yPos += itemHeight - pText->getHeight();
         int ySrc = 0;
         if( yPos < 0 )
