@@ -2,7 +2,7 @@
  * wizard.cpp : wxWindows plugin for vlc
  *****************************************************************************
  * Copyright (C) 2000-2004 VideoLAN
- * $Id: wizard.cpp 9045 2004-10-23 16:08:57Z zorglub $
+ * $Id: wizard.cpp 9136 2004-11-04 16:41:56Z hartman $
  *
  * Authors: Clment Stenac <zorglub@videolan.org>
  *
@@ -115,7 +115,7 @@ END_EVENT_TABLE()
 #define PARTIAL _("Use this to read only a part of the stream. " \
                   "You must be able to control the incoming stream " \
                   "(for example, a file or a disc, but not an UDP " \
-                  "network stream.\n" \
+                  "network stream.)\n" \
                   "Enter the starting and ending times (in seconds)")
 
 #define INPUT_BUTTON _("Choose")
