@@ -2,7 +2,7 @@
  * ipv6.c: IPv6 network abstraction layer
  *****************************************************************************
  * Copyright (C) 2002 VideoLAN
- * $Id: ipv6.c,v 1.7 2002/12/23 16:21:54 massiot Exp $
+ * $Id: ipv6.c 7909 2004-06-06 12:45:32Z zorglub $
  *
  * Authors: Alexis Guillard <alexis.guillard@bt.com>
  *          Christophe Massiot <massiot@via.ecp.fr>
@@ -63,14 +63,21 @@ static const struct in6_addr in6addr_any = {{IN6ADDR_ANY_INIT}};
 #   define IPPROTO_IPV6 41 
 #endif
 #ifndef IPV6_JOIN_GROUP
-#   define IPV6_JOIN_GROUP 20
+#   define IPV6_JOIN_GROUP 12
 #endif
+#ifndef IPV6_MULTICAST_HOPS
+#   define IPV6_MULTICAST_HOPS 10
+#endif
+#ifndef IPV6_UNICAST_HOPS
+#   define IPV6_UNICAST_HOPS 4
+#endif
+#   define close closesocket
 #endif
 
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
-static int Open( vlc_object_t * );
+static int NetOpen( vlc_object_t * );
 
 /*****************************************************************************
  * Module descriptor
@@ -78,7 +85,7 @@ static int Open( vlc_object_t * );
 vlc_module_begin();
     set_description( _("IPv6 network abstraction layer") );
     set_capability( "network", 40 );
-    set_callbacks( Open, NULL );
+    set_callbacks( NetOpen, NULL );
 vlc_module_end();
 
 /*****************************************************************************
@@ -113,7 +120,7 @@ static int BuildAddr( vlc_object_t * p_this, struct sockaddr_in6 * p_socket,
     }
     if( !_getaddrinfo || !_freeaddrinfo )
     {
-        msg_Err( p_this, "no IPv6 stack installed" );
+        msg_Warn( p_this, "no IPv6 stack installed" );
         if( wship6_dll ) FreeLibrary( wship6_dll );
         free( psz_backup );
         return( -1 );
@@ -131,7 +138,9 @@ static int BuildAddr( vlc_object_t * p_this, struct sockaddr_in6 * p_socket,
     else if( psz_address[0] == '['
               && psz_address[strlen(psz_address) - 1] == ']' )
     {
+        psz_address[strlen(psz_address) - 1] = '\0';
         psz_address++;
+
         /* see if there is an interface name in there... */
         if( (psz_multicast_interface = strchr(psz_address, '%')) != NULL )
         {
@@ -141,15 +150,14 @@ static int BuildAddr( vlc_object_t * p_this, struct sockaddr_in6 * p_socket,
                              psz_multicast_interface );
 
             /* now convert that interface name to an index */
-#if !defined( WIN32 )
+#if defined( WIN32 )
+            /* FIXME ?? */
+            p_socket->sin6_scope_id = atol(psz_multicast_interface);
+#elif defined( HAVE_IF_NAMETOINDEX )
             p_socket->sin6_scope_id = if_nametoindex(psz_multicast_interface);
-#else
-            /* FIXME: for now we always use the default interface */
-            p_socket->sin6_scope_id = 0;
 #endif
-            msg_Warn( p_this, " = #%i", p_socket->sin6_scope_id );
+            msg_Dbg( p_this, " = #%i", p_socket->sin6_scope_id );
         }
-        psz_address[strlen(psz_address) - 1] = '\0' ;
 
 #if !defined( WIN32 )
         inet_pton(AF_INET6, psz_address, &p_socket->sin6_addr.s6_addr); 
@@ -157,9 +165,8 @@ static int BuildAddr( vlc_object_t * p_this, struct sockaddr_in6 * p_socket,
 #else
         memset(&hints, 0, sizeof(hints));
         hints.ai_family = AF_INET6;
-        hints.ai_flags = AI_NUMERICHOST;
 
-        if( _getaddrinfo( psz_address, NULL, &hints, &res ) )
+        if( _getaddrinfo( psz_address, NULL, &hints, &res ) != 0 )
         {
             FreeLibrary( wship6_dll );
             free( psz_backup );
@@ -180,7 +187,7 @@ static int BuildAddr( vlc_object_t * p_this, struct sockaddr_in6 * p_socket,
         /* We have a fqdn, try to find its address */
         if ( (p_hostent = gethostbyname2( psz_address, AF_INET6 )) == NULL )
         {
-            msg_Err( p_this, "ipv6 error: unknown host %s", psz_address );
+            msg_Warn( p_this, "IPv6 error: unknown host %s", psz_address );
             free( psz_backup );
             return( -1 );
         }
@@ -190,7 +197,10 @@ static int BuildAddr( vlc_object_t * p_this, struct sockaddr_in6 * p_socket,
                  p_hostent->h_length );
 
 #elif defined(WIN32)
-        if( _getaddrinfo( psz_address, NULL, &hints, &res ) )
+        memset(&hints, 0, sizeof(hints));
+        hints.ai_family = AF_INET6;
+
+        if( _getaddrinfo( psz_address, NULL, &hints, &res ) != 0 )
         {
             FreeLibrary( wship6_dll );
             free( psz_backup );
@@ -202,7 +212,7 @@ static int BuildAddr( vlc_object_t * p_this, struct sockaddr_in6 * p_socket,
         _freeaddrinfo( res );
 
 #else
-        msg_Err( p_this, "ipv6 error: IPv6 address %s is invalid",
+        msg_Warn( p_this, "IPv6 error: IPv6 address %s is invalid",
                  psz_address );
         free( psz_backup );
         return( -1 );
@@ -240,12 +250,13 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     int i_handle, i_opt;
     socklen_t i_opt_size;
     struct sockaddr_in6 sock;
+    vlc_value_t val;
 
     /* Open a SOCK_DGRAM (UDP) socket, in the AF_INET6 domain, automatic (0)
      * protocol */
     if( (i_handle = socket( AF_INET6, SOCK_DGRAM, 0 )) == -1 )
     {
-        msg_Err( p_this, "cannot create socket (%s)", strerror(errno) );
+        msg_Warn( p_this, "cannot create socket (%s)", strerror(errno) );
         return( -1 );
     }
 
@@ -254,7 +265,7 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     if( setsockopt( i_handle, SOL_SOCKET, SO_REUSEADDR,
                     (void *) &i_opt, sizeof( i_opt ) ) == -1 )
     {
-        msg_Err( p_this, "cannot configure socket (SO_REUSEADDR: %s)",
+        msg_Warn( p_this, "cannot configure socket (SO_REUSEADDR: %s)",
                          strerror(errno) );
         close( i_handle );
         return( -1 );
@@ -269,7 +280,7 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
         msg_Warn( p_this, "cannot configure socket (SO_RCVBUF: %s)",
                           strerror(errno) );
     }
- 
+
     /* Check if we really got what we have asked for, because Linux, etc.
      * will silently limit the max buffer size to net.core.rmem_max which
      * is typically only 65535 bytes */
@@ -283,21 +294,38 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
     }
     else if( i_opt < 0x80000 )
     {
-        msg_Warn( p_this, "socket buffer size is 0x%x instead of 0x%x",
+        msg_Warn( p_this, "Socket buffer size is 0x%x instead of 0x%x",
                           i_opt, 0x80000 );
     }
-    
+
     /* Build the local socket */
     if ( BuildAddr( p_this, &sock, psz_bind_addr, i_bind_port ) == -1 )        
     {
         close( i_handle );
         return( -1 );
     }
- 
+
+#if defined(WIN32)
+    /* Under Win32 and for multicasting, we bind to IN6ADDR_ANY */
+    if( IN6_IS_ADDR_MULTICAST(&sock.sin6_addr) )
+    {
+        struct sockaddr_in6 sockany = sock;
+        sockany.sin6_addr = in6addr_any;
+        sockany.sin6_scope_id = 0;
+
+        /* Bind it */
+        if( bind( i_handle, (struct sockaddr *)&sockany, sizeof( sock ) ) < 0 )
+        {
+            msg_Warn( p_this, "cannot bind socket (%s)", strerror(errno) );
+            close( i_handle );
+            return( -1 );
+        }
+    } else
+#endif
     /* Bind it */
     if( bind( i_handle, (struct sockaddr *)&sock, sizeof( sock ) ) < 0 )
     {
-        msg_Err( p_this, "cannot bind socket (%s)", strerror(errno) );
+        msg_Warn( p_this, "cannot bind socket (%s)", strerror(errno) );
         close( i_handle );
         return( -1 );
     }
@@ -309,12 +337,13 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
         if( setsockopt( i_handle, SOL_SOCKET, SO_BROADCAST,
                         (void*) &i_opt, sizeof( i_opt ) ) == -1 )
         {
-            msg_Warn( p_this, "ipv6 warning: cannot configure socket "
+            msg_Warn( p_this, "IPv6 warning: cannot configure socket "
                               "(SO_BROADCAST: %s)", strerror(errno) );
         }
     }
- 
+
     /* Join the multicast group if the socket is a multicast address */
+#if defined( WIN32 ) || defined( HAVE_IF_NAMETOINDEX )
     if( IN6_IS_ADDR_MULTICAST(&sock.sin6_addr) )
     {
         struct ipv6_mreq     imr;
@@ -323,41 +352,135 @@ static int OpenUDP( vlc_object_t * p_this, network_socket_t * p_socket )
         imr.ipv6mr_interface = sock.sin6_scope_id;
         imr.ipv6mr_multiaddr = sock.sin6_addr;
         res = setsockopt(i_handle, IPPROTO_IPV6, IPV6_JOIN_GROUP, (void*) &imr,
+#if defined(WIN32)
+                         sizeof(imr) + 4); /* Doesn't work without this */
+#else
                          sizeof(imr));
+#endif
 
         if( res == -1 )
         {
-            msg_Err( p_this, "setsockopt JOIN_GROUP failed" );
-        }
+            msg_Err( p_this, "cannot join multicast group" );
+        } 
     }
+#else
+    msg_Warn( p_this, "Multicast IPv6 is not supported on your OS" );
+#endif
 
 
     if( *psz_server_addr )
     {
+        int ttl = p_socket->i_ttl;
+        if( ttl < 1 )
+        {
+            ttl = config_GetInt( p_this, "ttl" );
+        }
+        if( ttl < 1 ) ttl = 1;
+
         /* Build socket for remote connection */
         if ( BuildAddr( p_this, &sock, psz_server_addr, i_server_port ) == -1 )
         {
-            msg_Err( p_this, "cannot build remote address" );
+            msg_Warn( p_this, "cannot build remote address" );
             close( i_handle );
             return( -1 );
         }
- 
+
         /* Connect the socket */
         if( connect( i_handle, (struct sockaddr *) &sock,
                      sizeof( sock ) ) == (-1) )
         {
-            msg_Err( p_this, "cannot connect socket (%s)", strerror(errno) );
+            msg_Warn( p_this, "cannot connect socket (%s)", strerror(errno) );
             close( i_handle );
             return( -1 );
         }
+
+        /* Set the time-to-live */
+        if( ttl > 1 )
+        {
+#if defined( WIN32 ) || defined( HAVE_IF_NAMETOINDEX )
+            if( IN6_IS_ADDR_MULTICAST(&sock.sin6_addr) )
+            {
+                if( setsockopt( i_handle, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
+                                (void *)&ttl, sizeof( ttl ) ) < 0 )
+                {
+#ifdef HAVE_ERRNO_H
+                    msg_Err( p_this, "failed to set multicast ttl (%s)",
+                             strerror(errno) );
+#else
+                    msg_Err( p_this, "failed to set multicast ttl" );
+#endif
+                }
+            }
+            else
+#endif
+            {
+                if( setsockopt( i_handle, IPPROTO_IPV6, IPV6_UNICAST_HOPS,
+                                (void *)&ttl, sizeof( ttl ) ) < 0 )
+                {
+#ifdef HAVE_ERRNO_H
+                    msg_Err( p_this, "failed to set unicast ttl (%s)",
+                              strerror(errno) );
+#else
+                    msg_Err( p_this, "failed to set unicast ttl" );
+#endif
+                }
+            }
+        }
     }
 
     p_socket->i_handle = i_handle;
-    p_socket->i_mtu = config_GetInt( p_this, "mtu" );
+
+    var_Create( p_this, "mtu", VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+    var_Get( p_this, "mtu", &val );
+    p_socket->i_mtu = val.i_int;
 
     return( 0 );
 }
 
+/*****************************************************************************
+ * SocketTCP: create a TCP socket
+ *****************************************************************************
+ * This function returns -1 in case of error.
+ *****************************************************************************/
+static int SocketTCP( vlc_object_t * p_this )
+{
+    int i_handle;
+    
+    /* Open a SOCK_STREAM (TCP) socket, in the PF_INET6 domain, automatic (0)
+     * protocol */
+    if( (i_handle = socket( PF_INET6, SOCK_STREAM, 0 )) == -1 )
+    {
+#ifdef HAVE_ERRNO_H
+        msg_Warn( p_this, "cannot create socket (%s)", strerror(errno) );
+#else
+        msg_Warn( p_this, "cannot create socket" );
+#endif
+        return -1;
+    }
+
+    /* Set to non-blocking */
+#if defined( WIN32 ) || defined( UNDER_CE )
+    {
+        unsigned long i_dummy = 1;
+        if( ioctlsocket( i_handle, FIONBIO, &i_dummy ) != 0 )
+        {
+            msg_Err( p_this, "cannot set socket to non-blocking mode" );
+        }
+    }
+#elif defined( HAVE_ERRNO_H )
+    {
+        int i_flags;
+        if( ( i_flags = fcntl( i_handle, F_GETFL, 0 ) ) < 0 ||
+            fcntl( i_handle, F_SETFL, i_flags | O_NONBLOCK ) < 0 )
+        {
+            msg_Err( p_this, "cannot set socket to non-blocking mode" );
+        }
+    }
+#endif
+
+    return i_handle;
+}
+
 /*****************************************************************************
  * OpenTCP: open a TCP socket
  *****************************************************************************
@@ -379,40 +502,171 @@ static int OpenTCP( vlc_object_t * p_this, network_socket_t * p_socket )
         i_server_port = 80;
     }
 
-    /* Open a SOCK_STREAM (TCP) socket, in the AF_INET6 domain, automatic (0)
-     * protocol */
-    if( (i_handle = socket( AF_INET6, SOCK_STREAM, 0 )) == -1 )
+    if( (i_handle = SocketTCP( p_this )) == -1 )
+        return VLC_EGENERIC;
+
+    /* Build remote address */
+    if ( BuildAddr( p_this, &sock, psz_server_addr, i_server_port ) == -1 )
+        goto error;
+
+    /* Connect the socket */
+    if( connect( i_handle, (struct sockaddr *) &sock, sizeof( sock ) ) == -1 )
     {
-        msg_Err( p_this, "cannot create socket (%s)", strerror(errno) );
-        return( -1 );
+#if defined( WIN32 ) || defined( UNDER_CE )
+        if( WSAGetLastError() == WSAEWOULDBLOCK )
+#elif defined( HAVE_ERRNO_H )
+        if( errno == EINPROGRESS )
+#else
+        if( 0 )
+#endif
+        {
+            int i_ret, i_opt, i_opt_size = sizeof( i_opt ), i_max_count;
+            struct timeval timeout;
+            vlc_value_t val;
+            fd_set fds;
+
+            /* FIXME: There is no ipv6-timeout option, so we use ipv4-timeout
+             * instead */
+            if( !var_Type( p_this, "ipv4-timeout" ) )
+            {
+                var_Create( p_this, "ipv4-timeout",
+                            VLC_VAR_INTEGER | VLC_VAR_DOINHERIT );
+            }
+            var_Get( p_this, "ipv4-timeout", &val );
+            i_max_count = val.i_int * 1000 / 100000 /* timeout.tv_usec */;
+
+            msg_Dbg( p_this, "connection in progress" );
+            do
+            {
+                if( p_this->b_die || i_max_count <= 0 )
+                {
+                    msg_Dbg( p_this, "connection aborted" );
+                    goto error;
+                }
+
+                i_max_count--;
+
+                /* Initialize file descriptor set */
+                FD_ZERO( &fds );
+                FD_SET( i_handle, &fds );
+
+                /* We'll wait 0.1 second if nothing happens */
+                timeout.tv_sec = 0;
+                timeout.tv_usec = 100000;
+
+            } while( ( i_ret = select( i_handle + 1, NULL, &fds, NULL,
+                                       &timeout ) ) == 0 ||
+#if defined( WIN32 ) || defined( UNDER_CE )
+                     ( i_ret < 0 && WSAGetLastError() == WSAEWOULDBLOCK ) );
+#elif defined( HAVE_ERRNO_H )
+                     ( i_ret < 0 && errno == EINTR ) );
+#else
+                     ( i_ret < 0 ) );
+#endif
+
+            if( i_ret < 0 )
+            {
+                msg_Warn( p_this, "cannot connect socket (select failed)" );
+                goto error;
+            }
+
+#if !defined( SYS_BEOS )
+            if( getsockopt( i_handle, SOL_SOCKET, SO_ERROR, (void*)&i_opt,
+                            &i_opt_size ) == -1 || i_opt != 0 )
+            {
+                msg_Warn( p_this, "cannot connect socket (SO_ERROR)" );
+                goto error;
+            }
+#endif
+        }
+        else
+        {
+#if defined( HAVE_ERRNO_H )
+            msg_Warn( p_this, "cannot connect socket (%s)", strerror(errno) );
+#else
+            msg_Warn( p_this, "cannot connect socket" );
+#endif
+            goto error;
+        }
+    }
+
+    p_socket->i_handle = i_handle;
+    p_socket->i_mtu = 0; /* There is no MTU notion in TCP */
+    return VLC_SUCCESS;
+
+error:
+    close( i_handle );
+    return VLC_EGENERIC;
+}
+
+/*****************************************************************************
+ * ListenTCP: open a TCP passive socket (server-side)
+ *****************************************************************************
+ * psz_server_addr, i_server_port : address and port used for the bind()
+ *   system call. If i_server_port == 0, 80 is used.
+ * Other parameters are ignored.
+ * This function returns -1 in case of error.
+ *****************************************************************************/
+static int ListenTCP( vlc_object_t * p_this, network_socket_t * p_socket )
+{
+    char * psz_server_addr = p_socket->psz_server_addr;
+    int i_server_port = p_socket->i_server_port;
+
+    int i_handle, i_dummy = 1;
+    struct sockaddr_in6 sock;
+
+    if( (i_handle = SocketTCP( p_this )) == -1 )
+        return VLC_EGENERIC;
+
+    if ( setsockopt( i_handle, SOL_SOCKET, SO_REUSEADDR,
+                (void *)&i_dummy, sizeof( i_dummy ) ) == -1 )
+    {
+        msg_Warn( p_this, "cannot configure socket (SO_REUSEADDR)" );
     }
 
     /* Build remote address */
     if ( BuildAddr( p_this, &sock, psz_server_addr, i_server_port ) == -1 )
     {
-        close( i_handle );
-        return( -1 );
+        msg_Dbg( p_this, "could not build local address" );
+        return VLC_EGENERIC;
     }
-
-    /* Connect the socket */
-    if( connect( i_handle, (struct sockaddr *) &sock,
-                 sizeof( sock ) ) == (-1) )
+    
+    /* Bind the socket */
+    if( bind( i_handle, (struct sockaddr *) &sock, sizeof( sock )) == -1 )
     {
-        msg_Err( p_this, "cannot connect socket (%s)", strerror(errno) );
-        close( i_handle );
-        return( -1 );
+#ifdef HAVE_ERRNO_H
+        msg_Err( p_this, "cannot bind socket (%s)", strerror(errno) );
+#else
+        msg_Err( p_this, "cannot bind socket" );
+#endif
+        goto error;
+    }
+ 
+    /* Listen */
+    if( listen( i_handle, 100 ) == -1 )
+    {
+#ifdef HAVE_ERRNO_H
+         msg_Err( p_this, "cannot bring the socket in listening mode (%s)",
+                  strerror(errno) );
+#else
+         msg_Err( p_this, "cannot bring the socket in listening mode" );
+#endif
+         goto error;
     }
 
     p_socket->i_handle = i_handle;
     p_socket->i_mtu = 0; /* There is no MTU notion in TCP */
+    return VLC_SUCCESS;
 
-    return( 0 );
+error:
+    close( i_handle );
+    return VLC_EGENERIC;
 }
 
 /*****************************************************************************
- * Open: wrapper around OpenUDP and OpenTCP
+ * NetOpen: wrapper around OpenUDP, ListenTCP and OpenTCP
  *****************************************************************************/
-static int Open( vlc_object_t * p_this )
+static int NetOpen( vlc_object_t * p_this )
 {
     network_socket_t * p_socket = p_this->p_private;
 
@@ -420,6 +674,10 @@ static int Open( vlc_object_t * p_this )
     {
         return OpenUDP( p_this, p_socket );
     }
+    else if( p_socket->i_type == NETWORK_TCP_PASSIVE )
+    {
+        return ListenTCP( p_this, p_socket );
+    }
     else
     {
         return OpenTCP( p_this, p_socket );
