@@ -1,10 +1,11 @@
 /*****************************************************************************
- * asf.c
+ * asf.c: asf muxer module for vlc
  *****************************************************************************
- * Copyright (C) 2003 VideoLAN
- * $Id: asf.c 7477 2004-04-25 08:53:52Z zorglub $
+ * Copyright (C) 2003-2004 VideoLAN
+ * $Id: asf.c 8888 2004-10-02 17:57:33Z zorglub $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
+ *          Gildas Bazin <gbazin@videolan.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +32,9 @@
 #include <vlc/sout.h>
 
 #include "codecs.h"
+typedef GUID guid_t;
+
+#define MAX_ASF_TRACKS 128
 
 /*****************************************************************************
  * Module descriptor
@@ -44,11 +48,11 @@ static void Close  ( vlc_object_t * );
 #define TITLE_LONGTEXT N_("Allows you to define the title that will be put " \
                           "in ASF comments.")
 #define AUTHOR_TEXT N_("Author")
-#define AUTHOR_LONGTEXT N_("Allows you to define the author that will be put " \
-                          "in ASF comments.")
+#define AUTHOR_LONGTEXT N_("Allows you to define the author that will be put "\
+                           "in ASF comments.")
 #define COPYRIGHT_TEXT N_("Copyright")
-#define COPYRIGHT_LONGTEXT N_("Allows you to define the copyright string that "\
-                          "will be put in ASF comments.")
+#define COPYRIGHT_LONGTEXT N_("Allows you to define the copyright string " \
+                              "that will be put in ASF comments.")
 #define COMMENT_TEXT N_("Comment")
 #define COMMENT_LONGTEXT N_("Allows you to define the comment that will be " \
                             "put in ASF comments.")
@@ -80,27 +84,19 @@ vlc_module_end();
  *****************************************************************************/
 static const char *ppsz_sout_options[] = {
     "title", "author", "copyright", "comment", "rating", NULL
-};;
+};
 
-static int  Capability(sout_mux_t *, int, void *, void * );
-static int  AddStream( sout_mux_t *, sout_input_t * );
-static int  DelStream( sout_mux_t *, sout_input_t * );
-static int  Mux      ( sout_mux_t * );
-
-typedef struct
-{
-    uint32_t v1; /* le */
-    uint16_t v2; /* le */
-    uint16_t v3; /* le */
-    uint8_t  v4[8];
-} guid_t;
+static int Control  ( sout_mux_t *, int, va_list );
+static int AddStream( sout_mux_t *, sout_input_t * );
+static int DelStream( sout_mux_t *, sout_input_t * );
+static int Mux      ( sout_mux_t * );
 
 typedef struct
 {
     int          i_id;
     int          i_cat;
 
-    /* codec informations */
+    /* codec information */
     uint16_t     i_tag;     /* for audio */
     vlc_fourcc_t i_fourcc;  /* for video */
     char         *psz_name; /* codec name */
@@ -108,7 +104,10 @@ typedef struct
     int          i_sequence;
 
     int          i_extra;
-    uint8_t     *p_extra;
+    uint8_t      *p_extra;
+
+    es_format_t  fmt;
+
 } asf_track_t;
 
 struct sout_mux_sys_t
@@ -118,14 +117,15 @@ struct sout_mux_sys_t
     int64_t         i_packet_count;
     mtime_t         i_dts_first;
     mtime_t         i_dts_last;
+    mtime_t         i_preroll_time;
     int64_t         i_bitrate;
 
     int             i_track;
-    asf_track_t     track[128];
+    asf_track_t     track[MAX_ASF_TRACKS];
 
     vlc_bool_t      b_write_header;
 
-    block_t   *pk;
+    block_t         *pk;
     int             i_pk_used;
     int             i_pk_frame;
     mtime_t         i_pk_dts;
@@ -143,16 +143,17 @@ struct sout_mux_sys_t
 
 static int MuxGetStream( sout_mux_t *, int *pi_stream, mtime_t *pi_dts );
 
-static block_t *asf_header_create( sout_mux_t *, vlc_bool_t b_broadcast );
-static block_t *asf_packet_create( sout_mux_t *,
-                                         asf_track_t *, block_t * );
+static block_t *asf_header_create( sout_mux_t *, vlc_bool_t );
+static block_t *asf_packet_create( sout_mux_t *, asf_track_t *, block_t * );
 static block_t *asf_stream_end_create( sout_mux_t *);
+static block_t *asf_packet_flush( sout_mux_t * );
 
 typedef struct
 {
     int      i_buffer_size;
     int      i_buffer;
     uint8_t  *p_buffer;
+
 } bo_t;
 
 static void bo_init     ( bo_t *, uint8_t *, int  );
@@ -175,9 +176,9 @@ static int Open( vlc_object_t *p_this )
     int i;
 
     msg_Dbg( p_mux, "Asf muxer opened" );
-    sout_ParseCfg( p_mux, SOUT_CFG_PREFIX, ppsz_sout_options, p_mux->p_cfg );
+    sout_CfgParse( p_mux, SOUT_CFG_PREFIX, ppsz_sout_options, p_mux->p_cfg );
 
-    p_mux->pf_capacity  = Capability;
+    p_mux->pf_control   = Control;
     p_mux->pf_addstream = AddStream;
     p_mux->pf_delstream = DelStream;
     p_mux->pf_mux       = Mux;
@@ -193,23 +194,26 @@ static int Open( vlc_object_t *p_this )
     p_sys->i_pk_frame   = 0;
     p_sys->i_dts_first  = -1;
     p_sys->i_dts_last   = 0;
+    p_sys->i_preroll_time = 2000;
     p_sys->i_bitrate    = 0;
     p_sys->i_seq        = 0;
 
     p_sys->b_write_header = VLC_TRUE;
-    p_sys->i_track = 1;
+    p_sys->i_track = 0;
     p_sys->i_packet_size = 4096;
     p_sys->i_packet_count= 0;
-    /* generate a random fid */
+
+    /* Generate a random fid */
     srand( mdate() & 0xffffffff );
-    p_sys->fid.v1 = 0xbabac001;
-    p_sys->fid.v2 = ( (uint64_t)rand() << 16 ) / RAND_MAX;
-    p_sys->fid.v3 = ( (uint64_t)rand() << 16 ) / RAND_MAX;
+    p_sys->fid.Data1 = 0xbabac001;
+    p_sys->fid.Data2 = ( (uint64_t)rand() << 16 ) / RAND_MAX;
+    p_sys->fid.Data3 = ( (uint64_t)rand() << 16 ) / RAND_MAX;
     for( i = 0; i < 8; i++ )
     {
-        p_sys->fid.v4[i] = ( (uint64_t)rand() << 8 ) / RAND_MAX;
+        p_sys->fid.Data4[i] = ( (uint64_t)rand() << 8 ) / RAND_MAX;
     }
-    /* meta data */
+
+    /* Meta data */
     var_Get( p_mux, SOUT_CFG_PREFIX "title", &val );
     p_sys->psz_title = val.psz_string;
 
@@ -225,8 +229,8 @@ static int Open( vlc_object_t *p_this )
     var_Get( p_mux, SOUT_CFG_PREFIX "rating", &val );
     p_sys->psz_rating = val.psz_string;
 
-    msg_Dbg( p_mux,
-             "meta data: title='%s' author='%s' copyright='%s' comment='%s' rating='%s'",
+    msg_Dbg( p_mux, "meta data: title='%s' author='%s' copyright='%s' "
+             "comment='%s' rating='%s'",
              p_sys->psz_title, p_sys->psz_author, p_sys->psz_copyright,
              p_sys->psz_comment, p_sys->psz_rating );
 
@@ -245,6 +249,12 @@ static void Close( vlc_object_t * p_this )
 
     msg_Dbg( p_mux, "Asf muxer closed" );
 
+    /* Flush last packet if any */
+    if( (out = asf_packet_flush( p_mux ) ) )
+    {
+        sout_AccessOutWrite( p_mux->p_access, out );
+    }
+
     if( ( out = asf_stream_end_create( p_mux ) ) )
     {
         sout_AccessOutWrite( p_mux->p_access, out );
@@ -257,9 +267,10 @@ static void Close( vlc_object_t * p_this )
         sout_AccessOutWrite( p_mux->p_access, out );
     }
 
-    for( i = 1; i < p_sys->i_track; i++ )
+    for( i = 0; i < p_sys->i_track; i++ )
     {
         free( p_sys->track[i].p_extra );
+        es_format_Clean( &p_sys->track[i].fmt );
     }
     free( p_sys );
 }
@@ -267,17 +278,36 @@ static void Close( vlc_object_t * p_this )
 /*****************************************************************************
  * Capability:
  *****************************************************************************/
-static int Capability( sout_mux_t *p_mux, int i_query,
-                       void *p_args, void *p_answer )
+static int Control( sout_mux_t *p_mux, int i_query, va_list args )
 {
-   switch( i_query )
-   {
-        case SOUT_MUX_CAP_GET_ADD_STREAM_ANY_TIME:
-            *(vlc_bool_t*)p_answer = VLC_FALSE;
-            return( SOUT_MUX_CAP_ERR_OK );
+    sout_mux_sys_t *p_sys = p_mux->p_sys;
+    vlc_bool_t *pb_bool;
+    char **ppsz;
+
+    switch( i_query )
+    {
+       case MUX_CAN_ADD_STREAM_WHILE_MUXING:
+           pb_bool = (vlc_bool_t*)va_arg( args, vlc_bool_t * );
+           if( p_sys->b_asf_http ) *pb_bool = VLC_TRUE;
+           else *pb_bool = VLC_FALSE;
+           return VLC_SUCCESS;
+
+       case MUX_GET_ADD_STREAM_WAIT:
+           pb_bool = (vlc_bool_t*)va_arg( args, vlc_bool_t * );
+           *pb_bool = VLC_TRUE;
+           return VLC_SUCCESS;
+
+       case MUX_GET_MIME:
+           ppsz = (char**)va_arg( args, char ** );
+           if( p_sys->b_asf_http )
+               *ppsz = strdup( "video/x-ms-asf-stream" );
+           else
+               *ppsz = strdup( "video/x-ms-asf" );
+           return VLC_SUCCESS;
+
         default:
-            return( SOUT_MUX_CAP_ERR_UNIMPLEMENTED );
-   }
+            return VLC_EGENERIC;
+    }
 }
 
 /*****************************************************************************
@@ -290,14 +320,14 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
     bo_t             bo;
 
     msg_Dbg( p_mux, "adding input" );
-    if( p_sys->i_track > 127 )
+    if( p_sys->i_track >= MAX_ASF_TRACKS )
     {
         msg_Dbg( p_mux, "cannot add this track (too much track)" );
         return VLC_EGENERIC;
     }
 
     tk = p_input->p_sys = &p_sys->track[p_sys->i_track];
-    tk->i_id  = p_sys->i_track;
+    tk->i_id  = p_sys->i_track + 1;
     tk->i_cat = p_input->p_fmt->i_cat;
     tk->i_sequence = 0;
 
@@ -305,9 +335,9 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
     {
         case AUDIO_ES:
         {
-            int      i_blockalign = p_input->p_fmt->audio.i_blockalign;
-            int      i_bitspersample = 0;
-            int      i_extra = 0;
+            int i_blockalign = p_input->p_fmt->audio.i_blockalign;
+            int i_bitspersample = 0;
+            int i_extra = 0;
 
             switch( p_input->p_fmt->i_codec )
             {
@@ -370,7 +400,6 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
                     return VLC_EGENERIC;
             }
 
-
             tk->i_extra = sizeof( WAVEFORMATEX ) +
                           p_input->p_fmt->i_extra + i_extra;
             tk->p_extra = malloc( tk->i_extra );
@@ -470,6 +499,11 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
                 tk->psz_name = "Windows Media Video 2";
                 tk->i_fourcc = VLC_FOURCC( 'W', 'M', 'V', '2' );
             }
+            else if( p_input->p_fmt->i_codec == VLC_FOURCC('W','M','V','3') )
+            {
+                tk->psz_name = "Windows Media Video 3";
+                tk->i_fourcc = VLC_FOURCC( 'W', 'M', 'V', '3' );
+            }
             else
             {
                 tk->psz_name = _("Unknown Video");
@@ -502,6 +536,8 @@ static int AddStream( sout_mux_t *p_mux, sout_input_t *p_input )
             return VLC_EGENERIC;
     }
 
+    es_format_Copy( &tk->fmt, p_input->p_fmt );
+
     p_sys->i_track++;
     return VLC_SUCCESS;
 }
@@ -518,7 +554,7 @@ static int DelStream( sout_mux_t *p_mux, sout_input_t *p_input )
 /*****************************************************************************
  * Mux:
  *****************************************************************************/
-static int Mux      ( sout_mux_t *p_mux )
+static int Mux( sout_mux_t *p_mux )
 {
     sout_mux_sys_t *p_sys = p_mux->p_sys;
 
@@ -570,10 +606,7 @@ static int Mux      ( sout_mux_t *p_mux )
     return VLC_SUCCESS;
 }
 
-
-static int MuxGetStream( sout_mux_t *p_mux,
-                         int        *pi_stream,
-                         mtime_t    *pi_dts )
+static int MuxGetStream( sout_mux_t *p_mux, int *pi_stream, mtime_t *pi_dts )
 {
     mtime_t i_dts;
     int     i_stream;
@@ -597,8 +630,7 @@ static int MuxGetStream( sout_mux_t *p_mux,
         }
 
         p_data = block_FifoShow( p_input->p_fifo );
-        if( i_stream == -1 ||
-            p_data->i_dts < i_dts )
+        if( i_stream == -1 || p_data->i_dts < i_dts )
         {
             i_stream = i;
             i_dts    = p_data->i_dts;
@@ -654,9 +686,7 @@ static void bo_add_mem( bo_t *p_bo, uint8_t *p_mem, int i_size )
 
     if( i_copy > 0 )
     {
-        memcpy( &p_bo->p_buffer[p_bo->i_buffer],
-                p_mem,
-                i_copy );
+        memcpy( &p_bo->p_buffer[p_bo->i_buffer], p_mem, i_copy );
     }
     p_bo->i_buffer += i_size;
 }
@@ -666,14 +696,9 @@ static void bo_addle_str16( bo_t *bo, char *str )
     bo_addle_u16( bo, strlen( str ) + 1 );
     for( ;; )
     {
-        uint16_t c;
-
-        c = (uint8_t)*str++;
+        uint16_t c = (uint8_t)*str++;
         bo_addle_u16( bo, c );
-        if( c == '\0' )
-        {
-            break;
-        }
+        if( c == '\0' ) break;
     }
 }
 
@@ -681,130 +706,61 @@ static void bo_addle_str16_nosize( bo_t *bo, char *str )
 {
     for( ;; )
     {
-        uint16_t c;
-
-        c = (uint8_t)*str++;
+        uint16_t c = (uint8_t)*str++;
         bo_addle_u16( bo, c );
-        if( c == '\0' )
-        {
-            break;
-        }
+        if( c == '\0' ) break;
     }
 }
 
 /****************************************************************************
- * guid
+ * GUID definitions
  ****************************************************************************/
 static void bo_add_guid( bo_t *p_bo, const guid_t *id )
 {
     int i;
-    bo_addle_u32( p_bo, id->v1 );
-    bo_addle_u16( p_bo, id->v2 );
-    bo_addle_u16( p_bo, id->v3 );
+    bo_addle_u32( p_bo, id->Data1 );
+    bo_addle_u16( p_bo, id->Data2 );
+    bo_addle_u16( p_bo, id->Data3 );
     for( i = 0; i < 8; i++ )
     {
-        bo_add_u8( p_bo, id->v4[i] );
+        bo_add_u8( p_bo, id->Data4[i] );
     }
 }
 
 static const guid_t asf_object_header_guid =
-{
-    0x75B22630,
-    0x668E,
-    0x11CF,
-    { 0xA6,0xD9, 0x00,0xAA,0x00,0x62,0xCE,0x6C }
-};
+{0x75B22630, 0x668E, 0x11CF, {0xA6, 0xD9, 0x00, 0xAA, 0x00, 0x62, 0xCE, 0x6C}};
 static const guid_t asf_object_data_guid =
-{
-    0x75B22636,
-    0x668E,
-    0x11CF,
-    { 0xA6,0xD9, 0x00,0xAA,0x00,0x62,0xCE,0x6C }
-};
-
+{0x75B22636, 0x668E, 0x11CF, {0xA6, 0xD9, 0x00, 0xAA, 0x00, 0x62, 0xCE, 0x6C}};
 static const guid_t asf_object_file_properties_guid =
-{
-    0x8cabdca1,
-    0xa947,
-    0x11cf,
-    { 0x8e,0xe4, 0x00,0xC0,0x0C,0x20,0x53,0x65 }
-
-};
+{0x8cabdca1, 0xa947, 0x11cf, {0x8e, 0xe4, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65}};
 static const guid_t asf_object_stream_properties_guid =
-{
-    0xB7DC0791,
-    0xA9B7,
-    0x11CF,
-    { 0x8E,0xE6, 0x00,0xC0,0x0C,0x20,0x53,0x65 }
-
-};
+{0xB7DC0791, 0xA9B7, 0x11CF, {0x8E, 0xE6, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65}};
 static const guid_t asf_object_header_extention_guid =
-{
-   0x5FBF03B5,
-   0xA92E,
-   0x11CF,
-   { 0x8E,0xE3, 0x00,0xC0,0x0C,0x20,0x53,0x65 }
-};
-
+{0x5FBF03B5, 0xA92E, 0x11CF, {0x8E, 0xE3, 0x00, 0xC0, 0x0C, 0x20, 0x53, 0x65}};
 static const guid_t asf_object_stream_type_audio =
-{
-    0xF8699E40,
-    0x5B4D,
-    0x11CF,
-    { 0xA8,0xFD, 0x00,0x80,0x5F,0x5C,0x44,0x2B }
-};
-
+{0xF8699E40, 0x5B4D, 0x11CF, {0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B}};
 static const guid_t asf_object_stream_type_video =
-{
-    0xbc19efc0,
-    0x5B4D,
-    0x11CF,
-    { 0xA8,0xFD, 0x00,0x80,0x5F,0x5C,0x44,0x2B }
-};
-
+{0xbc19efc0, 0x5B4D, 0x11CF, {0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B}};
 static const guid_t asf_guid_audio_conceal_none =
-{
-    0x20FB5700,
-    0x5B55,
-    0x11CF,
-    { 0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B }
-};
+{0x20FB5700, 0x5B55, 0x11CF, {0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B}};
 static const guid_t asf_guid_video_conceal_none =
-{
-    0x20FB5700,
-    0x5B55,
-    0x11CF,
-    { 0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B }
-};
+{0x20FB5700, 0x5B55, 0x11CF, {0xA8, 0xFD, 0x00, 0x80, 0x5F, 0x5C, 0x44, 0x2B}};
 static const guid_t asf_guid_reserved_1 =
-{
-    0xABD3D211,
-    0xA9BA,
-    0x11cf,
-    { 0x8E, 0xE6,0x00, 0xC0, 0x0C ,0x20, 0x53, 0x65 }
-};
+{0xABD3D211, 0xA9BA, 0x11cf, {0x8E, 0xE6, 0x00, 0xC0, 0x0C ,0x20, 0x53, 0x65}};
 static const guid_t asf_object_codec_comment_guid =
-{
-    0x86D15240,
-    0x311D,
-    0x11D0,
-    { 0xA3, 0xA4, 0x00, 0xA0, 0xC9, 0x03, 0x48, 0xF6 }
-};
+{0x86D15240, 0x311D, 0x11D0, {0xA3, 0xA4, 0x00, 0xA0, 0xC9, 0x03, 0x48, 0xF6}};
 static const guid_t asf_object_codec_comment_reserved_guid =
-{
-    0x86D15241,
-    0x311D,
-    0x11D0,
-    { 0xA3, 0xA4, 0x00, 0xA0, 0xC9, 0x03, 0x48, 0xF6 }
-};
+{0x86D15241, 0x311D, 0x11D0, {0xA3, 0xA4, 0x00, 0xA0, 0xC9, 0x03, 0x48, 0xF6}};
 static const guid_t asf_object_content_description_guid =
-{
-    0x75B22633,
-    0x668E,
-    0x11CF,
-    { 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c }
-};
+{0x75B22633, 0x668E, 0x11CF, {0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c}};
+static const guid_t asf_object_index_guid =
+{0x33000890, 0xE5B1, 0x11CF, {0x89, 0xF4, 0x00, 0xA0, 0xC9, 0x03, 0x49, 0xCB}};
+static const guid_t asf_object_metadata_guid =
+{0xC5F8CBEA, 0x5BAF, 0x4877, {0x84, 0x67, 0xAA, 0x8C, 0x44, 0xFA, 0x4C, 0xCA}};
 
+/****************************************************************************
+ * Misc
+ ****************************************************************************/
 static void asf_chunk_add( bo_t *bo,
                            int i_type, int i_len, int i_flags, int i_seq )
 {
@@ -815,33 +771,28 @@ static void asf_chunk_add( bo_t *bo,
     bo_addle_u16( bo, i_len + 8 );
 }
 
-static block_t *asf_header_create( sout_mux_t *p_mux,
-                                         vlc_bool_t b_broadcast )
+static block_t *asf_header_create( sout_mux_t *p_mux, vlc_bool_t b_broadcast )
 {
     sout_mux_sys_t *p_sys = p_mux->p_sys;
     asf_track_t    *tk;
-
-    mtime_t        i_duration = 0;
-    int i_size;
-    int i_ci_size;
-    int i_cd_size = 0;
+    mtime_t i_duration = 0;
+    int i_size, i;
+    int i_ci_size, i_cm_size = 0, i_cd_size = 0;
     block_t *out;
-    bo_t          bo;
-    int           i;
+    bo_t bo;
+
+    msg_Dbg( p_mux, "Asf muxer creating header" );
 
     if( p_sys->i_dts_first > 0 )
     {
         i_duration = p_sys->i_dts_last - p_sys->i_dts_first;
-        if( i_duration < 0 )
-        {
-            i_duration = 0;
-        }
+        if( i_duration < 0 ) i_duration = 0;
     }
 
     /* calculate header size */
     i_size = 30 + 104 + 46;
     i_ci_size = 44;
-    for( i = 1; i < p_sys->i_track; i++ )
+    for( i = 0; i < p_sys->i_track; i++ )
     {
         i_size += 78 + p_sys->track[i].i_extra;
         i_ci_size += 8 + 2 * strlen( p_sys->track[i].psz_name );
@@ -854,6 +805,8 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
             i_ci_size += 6;
         }
     }
+
+    /* size of the content description object */
     if( *p_sys->psz_title || *p_sys->psz_author || *p_sys->psz_copyright ||
         *p_sys->psz_comment || *p_sys->psz_rating )
     {
@@ -864,7 +817,17 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
                              strlen( p_sys->psz_rating ) + 1 );
     }
 
-    i_size += i_ci_size + i_cd_size;
+    /* size of the metadata object */
+    for( i = 0; i < p_sys->i_track; i++ )
+    {
+        if( p_sys->track[i].i_cat == VIDEO_ES )
+        {
+            i_cm_size = 26 + 2 * (16 + 2 * sizeof("AspectRatio?"));
+            break;
+        }
+    }
+
+    i_size += i_ci_size + i_cd_size + i_cm_size;
 
     if( p_sys->b_asf_http )
     {
@@ -877,10 +840,11 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
         out = block_New( p_mux, i_size + 50 );
         bo_init( &bo, out->p_buffer, i_size + 50 );
     }
+
     /* header object */
     bo_add_guid ( &bo, &asf_object_header_guid );
     bo_addle_u64( &bo, i_size );
-    bo_addle_u32( &bo, 2 + p_sys->i_track - 1 );
+    bo_addle_u32( &bo, 2 + p_sys->i_track );
     bo_add_u8   ( &bo, 1 );
     bo_add_u8   ( &bo, 2 );
 
@@ -896,7 +860,7 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
     bo_addle_u64( &bo, b_broadcast ? 0xffffffffLL : p_sys->i_packet_count );
     bo_addle_u64( &bo, i_duration * 10 );   /* play duration (100ns) */
     bo_addle_u64( &bo, i_duration * 10 );   /* send duration (100ns) */
-    bo_addle_u64( &bo, 3000 );              /* preroll duration (ms) */
+    bo_addle_u64( &bo, p_sys->i_preroll_time ); /* preroll duration (ms) */
     bo_addle_u32( &bo, b_broadcast ? 0x01 : 0x00);      /* flags */
     bo_addle_u32( &bo, p_sys->i_packet_size );  /* packet size min */
     bo_addle_u32( &bo, p_sys->i_packet_size );  /* packet size max */
@@ -904,10 +868,47 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
 
     /* header extention */
     bo_add_guid ( &bo, &asf_object_header_extention_guid );
-    bo_addle_u64( &bo, 46 );
+    bo_addle_u64( &bo, 46 + i_cm_size );
     bo_add_guid ( &bo, &asf_guid_reserved_1 );
     bo_addle_u16( &bo, 6 );
-    bo_addle_u32( &bo, 0 );
+    bo_addle_u32( &bo, 0 + i_cm_size );
+
+    /* metadata object (part of header extension) */
+    if( i_cm_size )
+    {
+        int64_t i_num, i_den;
+        int i_dst_num, i_dst_den;
+
+        for( i = 0; i < p_sys->i_track; i++ )
+            if( p_sys->track[i].i_cat == VIDEO_ES ) break;
+
+        i_num = p_sys->track[i].fmt.video.i_aspect *
+            (int64_t)p_sys->track[i].fmt.video.i_height;
+        i_den = VOUT_ASPECT_FACTOR * p_sys->track[i].fmt.video.i_width;
+        vlc_reduce( &i_dst_num, &i_dst_den, i_num, i_den, 0 );
+
+        msg_Dbg( p_mux, "pixel aspect-ratio: %i/%i", i_dst_num, i_dst_den );
+
+        bo_add_guid ( &bo, &asf_object_metadata_guid );
+        bo_addle_u64( &bo, i_cm_size );
+        bo_addle_u16( &bo, 2 ); /* description records count */
+        /* 1st description record */
+        bo_addle_u16( &bo, 0 ); /* reserved */
+        bo_addle_u16( &bo, i + 1 ); /* stream number (0 for the whole file) */
+        bo_addle_u16( &bo, 2 * sizeof("AspectRatioX") ); /* name length */
+        bo_addle_u16( &bo, 0x3 /* DWORD */ ); /* data type */
+        bo_addle_u32( &bo, 4 ); /* data length */
+        bo_addle_str16_nosize( &bo, "AspectRatioX" );
+        bo_addle_u32( &bo, i_dst_num ); /* data */
+        /* 2nd description record */
+        bo_addle_u16( &bo, 0 ); /* reserved */
+        bo_addle_u16( &bo, i + 1 ); /* stream number (0 for the whole file) */
+        bo_addle_u16( &bo, 2 * sizeof("AspectRatioY") ); /* name length */
+        bo_addle_u16( &bo, 0x3 /* DWORD */ ); /* data type */
+        bo_addle_u32( &bo, 4 ); /* data length */
+        bo_addle_str16_nosize( &bo, "AspectRatioY" );
+        bo_addle_u32( &bo, i_dst_den ); /* data */
+    }
 
     /* content description header */
     if( i_cd_size > 0 )
@@ -928,7 +929,7 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
     }
 
     /* stream properties */
-    for( i = 1; i < p_sys->i_track; i++ )
+    for( i = 0; i < p_sys->i_track; i++ )
     {
         tk = &p_sys->track[i];
 
@@ -956,8 +957,8 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
     bo_add_guid ( &bo, &asf_object_codec_comment_guid );
     bo_addle_u64( &bo, i_ci_size );
     bo_add_guid ( &bo, &asf_object_codec_comment_reserved_guid );
-    bo_addle_u32( &bo, p_sys->i_track - 1 );
-    for( i = 1; i < p_sys->i_track; i++ )
+    bo_addle_u32( &bo, p_sys->i_track );
+    for( i = 0; i < p_sys->i_track; i++ )
     {
         tk = &p_sys->track[i];
 
@@ -973,7 +974,6 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
         {
             bo_addle_u16( &bo, 4 );
             bo_add_mem  ( &bo, (uint8_t*)&tk->i_fourcc, 4 );
-
         }
     }
 
@@ -990,8 +990,43 @@ static block_t *asf_header_create( sout_mux_t *p_mux,
 /****************************************************************************
  *
  ****************************************************************************/
+static block_t *asf_packet_flush( sout_mux_t *p_mux )
+{
+    sout_mux_sys_t *p_sys = p_mux->p_sys;
+    int i_pad, i_preheader = p_sys->b_asf_http ? 12 : 0;
+    block_t *pk;
+    bo_t bo;
+
+    if( !p_sys->pk ) return 0;
+
+    i_pad = p_sys->i_packet_size - p_sys->i_pk_used;
+    memset( p_sys->pk->p_buffer + p_sys->i_pk_used, 0, i_pad );
+
+    bo_init( &bo, p_sys->pk->p_buffer, 14 + i_preheader );
+
+    if( p_sys->b_asf_http )
+        asf_chunk_add( &bo, 0x4424, p_sys->i_packet_size, 0x0, p_sys->i_seq++);
+
+    bo_add_u8   ( &bo, 0x82 );
+    bo_addle_u16( &bo, 0 );
+    bo_add_u8( &bo, 0x11 );
+    bo_add_u8( &bo, 0x5d );
+    bo_addle_u16( &bo, i_pad );
+    bo_addle_u32( &bo, (p_sys->i_pk_dts - p_sys->i_dts_first) / 1000 +
+                  p_sys->i_preroll_time );
+    bo_addle_u16( &bo, 0 /* data->i_length */ );
+    bo_add_u8( &bo, 0x80 | p_sys->i_pk_frame );
+
+    pk = p_sys->pk;
+    p_sys->pk = NULL;
+
+    p_sys->i_packet_count++;
+
+    return pk;
+}
+
 static block_t *asf_packet_create( sout_mux_t *p_mux,
-                                         asf_track_t *tk, block_t *data )
+                                   asf_track_t *tk, block_t *data )
 {
     sout_mux_sys_t *p_sys = p_mux->p_sys;
 
@@ -1003,32 +1038,33 @@ static block_t *asf_packet_create( sout_mux_t *p_mux,
 
     while( i_pos < i_data )
     {
-        bo_t          bo;
-        int           i_payload;
+        bo_t bo;
+        int i_payload;
 
         if( p_sys->pk == NULL )
         {
-            p_sys->pk = block_New( p_mux,
-                                   p_sys->i_packet_size + i_preheader);
+            p_sys->pk = block_New( p_mux, p_sys->i_packet_size + i_preheader );
             /* reserve 14 bytes for the packet header */
             p_sys->i_pk_used = 14 + i_preheader;
             p_sys->i_pk_frame = 0;
             p_sys->i_pk_dts = data->i_dts;
         }
 
-
         bo_init( &bo, &p_sys->pk->p_buffer[p_sys->i_pk_used],
-                      p_sys->i_packet_size - p_sys->i_pk_used );
+                 p_sys->i_packet_size - p_sys->i_pk_used );
 
         /* add payload (header size = 17) */
         i_payload = __MIN( i_data - i_pos,
                            p_sys->i_packet_size - p_sys->i_pk_used - 17 );
-        bo_add_u8   ( &bo, 0x80 | tk->i_id );
+        bo_add_u8   ( &bo, !(data->i_flags & BLOCK_FLAG_TYPE_P ||
+                      data->i_flags & BLOCK_FLAG_TYPE_B) ?
+                      0x80 | tk->i_id : tk->i_id );
         bo_add_u8   ( &bo, tk->i_sequence );
         bo_addle_u32( &bo, i_pos );
         bo_add_u8   ( &bo, 0x08 );  /* flags */
         bo_addle_u32( &bo, i_data );
-        bo_addle_u32( &bo, ( data->i_dts - p_sys->i_dts_first )/ 1000 );
+        bo_addle_u32( &bo, (data->i_dts - p_sys->i_dts_first) / 1000 +
+                      p_sys->i_preroll_time );
         bo_addle_u16( &bo, i_payload );
         bo_add_mem  ( &bo, &p_data[i_pos], i_payload );
         i_pos += i_payload;
@@ -1038,32 +1074,9 @@ static block_t *asf_packet_create( sout_mux_t *p_mux,
 
         if( p_sys->i_pk_used + 17 >= p_sys->i_packet_size )
         {
-            /* not enough data for another payload, flush the packet */
-            int i_pad = p_sys->i_packet_size - p_sys->i_pk_used;
-
-            bo_init( &bo, p_sys->pk->p_buffer, 14 + i_preheader );
-
-            if( p_sys->b_asf_http )
-            {
-                asf_chunk_add( &bo, 0x4424,
-                               p_sys->i_packet_size, 0x00, p_sys->i_seq++);
-            }
-            bo_add_u8   ( &bo, 0x82 );
-            bo_addle_u16( &bo, 0 );
-            bo_add_u8( &bo, 0x11 );
-            bo_add_u8( &bo, 0x5d );
-            bo_addle_u16( &bo, i_pad );
-            bo_addle_u32( &bo, ( p_sys->i_pk_dts - p_sys->i_dts_first )/ 1000 );
-            bo_addle_u16( &bo, 0 * data->i_length / 1000 );
-            bo_add_u8( &bo, 0x80 | p_sys->i_pk_frame );
-
-            /* append the packet */
-            *last = p_sys->pk;
-            last  = &p_sys->pk->p_next;
-
-            p_sys->pk = NULL;
-
-            p_sys->i_packet_count++;
+            /* Not enough data for another payload, flush the packet */
+            *last = asf_packet_flush( p_mux );
+            last  = &(*last)->p_next;
         }
     }
 
@@ -1078,7 +1091,7 @@ static block_t *asf_stream_end_create( sout_mux_t *p_mux )
     sout_mux_sys_t *p_sys = p_mux->p_sys;
 
     block_t *out = NULL;
-    bo_t          bo;
+    bo_t bo;
 
     if( p_sys->b_asf_http )
     {
@@ -1086,5 +1099,18 @@ static block_t *asf_stream_end_create( sout_mux_t *p_mux )
         bo_init( &bo, out->p_buffer, 12 );
         asf_chunk_add( &bo, 0x4524, 0, 0x00, p_sys->i_seq++ );
     }
+    else
+    {
+        /* Create index */
+        out = block_New( p_mux, 56 );
+        bo_init( &bo, out->p_buffer, 56 );
+        bo_add_guid ( &bo, &asf_object_index_guid );
+        bo_addle_u64( &bo, 56 );
+        bo_add_guid ( &bo, &p_sys->fid );
+        bo_addle_u64( &bo, 10000000 );
+        bo_addle_u32( &bo, 5 );
+        bo_addle_u32( &bo, 0 );
+    }
+
     return out;
 }
