@@ -2,7 +2,7 @@
  * pes.c: PES packetizer used by the MPEG multiplexers
  *****************************************************************************
  * Copyright (C) 2001, 2002 VideoLAN
- * $Id: pes.c 9067 2004-10-27 14:59:24Z gbazin $
+ * $Id: pes.c 9130 2004-11-03 18:32:30Z gbazin $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *          Eric Petit <titer@videolan.org>
@@ -95,7 +95,8 @@ static inline int PESHeader( uint8_t *p_hdr, mtime_t i_pts, mtime_t i_dts,
                 int i_pts_dts;
 
                 if( i_pts > 0 && i_dts > 0 &&
-                    ( i_pts != i_dts || p_fmt->i_cat == VIDEO_ES ) )
+                    ( i_pts != i_dts || ( p_fmt->i_cat == VIDEO_ES &&
+                      p_fmt->i_codec != VLC_FOURCC('m','p','g','v') ) ) )
                 {
                     i_pts_dts = 0x03;
                     if ( !i_header_size ) i_header_size = 0xa;
