@@ -2,7 +2,7 @@
  * standard.c: standard stream output module
  *****************************************************************************
  * Copyright (C) 2003-2004 VideoLAN
- * $Id: standard.c 8680 2004-09-10 09:49:01Z gbazin $
+ * $Id: standard.c 9201 2004-11-06 16:51:46Z yoann $
  *
  * Authors: Laurent Aimar <fenrir@via.ecp.fr>
  *
@@ -56,6 +56,10 @@
 #define NAME_LONGTEXT N_( \
     "Name of the session that will be announced with SAP or SLP" )
 
+#define GROUP_TEXT N_("Session groupname")
+#define GROUP_LONGTEXT N_( \
+    "Name of the group that will be announced for the session" )
+
 #define SAP_TEXT N_("SAP announcing")
 #define SAP_LONGTEXT N_("Announce this session with SAP")
 
@@ -84,8 +88,10 @@ vlc_module_begin();
                 URL_LONGTEXT, VLC_FALSE );
 
     add_bool( SOUT_CFG_PREFIX "sap", 0, NULL, SAP_TEXT, SAP_LONGTEXT, VLC_TRUE );
-    add_string( SOUT_CFG_PREFIX "name", "", NULL, NAME_TEXT,NAME_LONGTEXT
-                                     , VLC_TRUE );
+    add_string( SOUT_CFG_PREFIX "name", "", NULL, NAME_TEXT, NAME_LONGTEXT,
+                                        VLC_TRUE );
+    add_string( SOUT_CFG_PREFIX "group", "", NULL, GROUP_TEXT, GROUP_LONGTEXT,
+                                        VLC_TRUE );
     add_bool( SOUT_CFG_PREFIX "sap-ipv6", 0, NULL, SAPv6_TEXT, SAPv6_LONGTEXT,
                                         VLC_TRUE );
 
@@ -100,8 +106,8 @@ vlc_module_end();
  *****************************************************************************/
 static const char *ppsz_sout_options[] = {
     "access", "mux", "url",
-    "sap", "name", "sap-ipv6",
-    "slp",NULL
+    "sap", "name", "sap-ipv6", "group",
+    "slp", NULL
 };
 
 #define DEFAULT_IPV6_SCOPE '8'
@@ -325,13 +331,21 @@ static int Open( vlc_object_t *p_this )
         var_Get( p_stream, SOUT_CFG_PREFIX "name", &val );
         if( *val.psz_string )
         {
-            p_session->psz_name = val.psz_string;
+            p_session->psz_name = strdup( val.psz_string );
         }
         else
         {
-            free( val.psz_string );
             p_session->psz_name = strdup( psz_url );
         }
+        free( val.psz_string );
+
+        var_Get( p_stream, SOUT_CFG_PREFIX "group", &val );
+        if( *val.psz_string )
+        {
+            p_session->psz_group = strdup( val.psz_string );
+        }
+        free( val.psz_string );
+
         var_Get( p_stream, SOUT_CFG_PREFIX "sap-ipv6", &val );
         p_method->i_ip_version = val.b_bool ? 6 : 4;
 
