@@ -2,7 +2,7 @@
  * clone.c : Clone video plugin for vlc
  *****************************************************************************
  * Copyright (C) 2002, 2003 VideoLAN
- * $Id: clone.c,v 1.12 2004/01/25 20:05:28 hartman Exp $
+ * $Id: clone.c 8551 2004-08-28 17:36:02Z gbazin $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -54,10 +54,10 @@ static int  SendEvents( vlc_object_t *, char const *,
  *****************************************************************************/
 #define COUNT_TEXT N_("Number of clones")
 #define COUNT_LONGTEXT N_("Select the number of video windows in which to "\
-    "clone the video")
+    "clone the video.")
 
-#define VOUTLIST_TEXT N_("List of vout modules")
-#define VOUTLIST_LONGTEXT N_("Select the specific vout modules that you want to activate")
+#define VOUTLIST_TEXT N_("List of video output modules")
+#define VOUTLIST_LONGTEXT N_("Select the specific video output modules that you want to activate.")
 
 vlc_module_begin();
     set_description( _("Clone video filter") );
@@ -88,6 +88,19 @@ struct vout_sys_t
     vout_thread_t **pp_vout;
 };
 
+/*****************************************************************************
+ * Control: control facility for the vout (forwards to child vout)
+ *****************************************************************************/
+static int Control( vout_thread_t *p_vout, int i_query, va_list args )
+{
+    int i_vout;
+    for( i_vout = 0; i_vout < p_vout->p_sys->i_clones; i_vout++ )
+    {
+        vout_vaControl( p_vout->p_sys->pp_vout[ i_vout ], i_query, args );
+    }
+    return VLC_SUCCESS;
+}
+
 /*****************************************************************************
  * Create: allocates Clone video thread output method
  *****************************************************************************
@@ -111,6 +124,7 @@ static int Create( vlc_object_t *p_this )
     p_vout->pf_manage = NULL;
     p_vout->pf_render = Render;
     p_vout->pf_display = NULL;
+    p_vout->pf_control = Control;
 
     psz_clonelist = config_GetPsz( p_vout, "clone-vout-list" );
     if( psz_clonelist )
@@ -327,11 +341,12 @@ static void Render( vout_thread_t *p_vout, picture_t *p_pic )
                  && i_out_pitch == i_copy_pitch )
             {
                 p_vout->p_vlc->pf_memcpy( p_out, p_in, i_in_pitch
-                                           * p_outpic->p[i_plane].i_lines );
+                                     * p_outpic->p[i_plane].i_visible_lines );
             }
             else
             {
-                p_in_end = p_in + i_in_pitch * p_outpic->p[i_plane].i_lines;
+                p_in_end = p_in + i_in_pitch *
+                    p_outpic->p[i_plane].i_visible_lines;
 
                 while( p_in < p_in_end )
                 {
