@@ -2,7 +2,7 @@
  * mixer.c : audio output mixing operations
  *****************************************************************************
  * Copyright (C) 2002-2004 VideoLAN
- * $Id: mixer.c 7632 2004-05-10 12:21:29Z gbazin $
+ * $Id: mixer.c 9010 2004-10-18 13:57:03Z gbazin $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *
@@ -274,8 +274,15 @@ static int MixBuffer( aout_instance_t * p_aout )
                 /* Round to the nearest multiple */
                 i_nb_bytes /= p_aout->mixer.mixer.i_bytes_per_frame;
                 i_nb_bytes *= p_aout->mixer.mixer.i_bytes_per_frame;
-                p_input->p_first_byte_to_mix = p_buffer->p_buffer
-                                                + i_nb_bytes;
+                if( i_nb_bytes < 0 )
+                {
+                    /* Is it really the best way to do it ? */
+                    aout_FifoSet( p_aout, &p_aout->output.fifo, 0 );
+                    aout_DateSet( &exact_start_date, 0 );
+                    break;
+                }
+
+                p_input->p_first_byte_to_mix = p_buffer->p_buffer + i_nb_bytes;
             }
         }
     }
