@@ -4,7 +4,7 @@
  * interface, such as command line.
  *****************************************************************************
  * Copyright (C) 1998-2004 VideoLAN
- * $Id: interface.c,v 1.113 2004/02/08 11:23:17 gbazin Exp $
+ * $Id: interface.c 8481 2004-08-21 11:47:04Z ipkiss $
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *
@@ -37,17 +37,18 @@
 #include <string.h>                                            /* strerror() */
 
 #include <vlc/vlc.h>
-
-#include "stream_control.h"
-#include "input_ext-intf.h"
+#include <vlc/input.h>
 
 #include "audio_output.h"
 
 #include "vlc_interface.h"
-
 #include "vlc_video.h"
 #include "video_output.h"
 
+#ifdef SYS_DARWIN
+#    include "Cocoa/Cocoa.h"
+#endif /* SYS_DARWIN */
+
 /*****************************************************************************
  * Local prototypes
  *****************************************************************************/
@@ -59,6 +60,17 @@ static int SwitchIntfCallback( vlc_object_t *, char const *,
 static int AddIntfCallback( vlc_object_t *, char const *,
                             vlc_value_t , vlc_value_t , void * );
 
+#ifdef SYS_DARWIN
+/*****************************************************************************
+ * VLCApplication interface
+ *****************************************************************************/
+@interface VLCApplication : NSApplication
+{
+}
+
+@end
+#endif
+
 /*****************************************************************************
  * intf_Create: prepare interface before main loop
  *****************************************************************************
@@ -75,7 +87,6 @@ static int AddIntfCallback( vlc_object_t *, char const *,
 intf_thread_t* __intf_Create( vlc_object_t *p_this, const char *psz_module )
 {
     intf_thread_t * p_intf;
-    char *psz_intf;
 
     /* Allocate structure */
     p_intf = vlc_object_create( p_this, VLC_OBJECT_INTF );
@@ -84,21 +95,13 @@ intf_thread_t* __intf_Create( vlc_object_t *p_this, const char *psz_module )
         msg_Err( p_this, "out of memory" );
         return NULL;
     }
-
-    /* XXX: workaround for a bug in VLC 0.5.0 where the dvdplay plugin was
-     * registering itself in $interface, which we do not want to happen. */
-    psz_intf = config_GetPsz( p_intf, "intf" );
-    if( psz_intf )
-    {
-        if( !strcasecmp( psz_intf, "dvdplay" ) )
-        {
-            config_PutPsz( p_intf, "intf", "" );
-        }
-        free( psz_intf );
-    }
+    p_intf->pf_request_window = NULL;
+    p_intf->pf_release_window = NULL;
+    p_intf->pf_control_window = NULL;
+    p_intf->b_play = VLC_FALSE;
 
     /* Choose the best module */
-    p_intf->p_module = module_Need( p_intf, "interface", psz_module );
+    p_intf->p_module = module_Need( p_intf, "interface", psz_module, 0 );
 
     if( p_intf->p_module == NULL )
     {
@@ -138,6 +141,52 @@ intf_thread_t* __intf_Create( vlc_object_t *p_this, const char *psz_module )
  */
 int intf_RunThread( intf_thread_t *p_intf )
 {
+#ifdef SYS_DARWIN
+    NSAutoreleasePool * o_pool;
+
+    if( p_intf->b_block )
+    {
+        /* This is the primary intf */
+        /* Run a manager thread, launch the interface, kill the manager */
+        if( vlc_thread_create( p_intf, "manager", Manager,
+                               VLC_THREAD_PRIORITY_LOW, VLC_FALSE ) )
+        {
+            msg_Err( p_intf, "cannot spawn manager thread" );
+            return VLC_EGENERIC;
+        }
+    }
+
+    if( p_intf->b_block && strncmp( p_intf->p_module->psz_shortname, "clivlc", 6) )
+    {
+        o_pool = [[NSAutoreleasePool alloc] init];
+	[VLCApplication sharedApplication];
+    }
+
+    if( p_intf->b_block && ( !strncmp( p_intf->p_module->psz_shortname, "macosx" , 6 ) ||
+                             !strncmp( p_intf->p_vlc->psz_object_name, "clivlc", 6 ) ) )
+    {
+        /* VLC in normal primary interface mode */
+        RunInterface( p_intf );
+        p_intf->b_die = VLC_TRUE;
+    }
+    else
+    {
+        /* Run the interface in a separate thread */
+        if( vlc_thread_create( p_intf, "interface", RunInterface,
+                               VLC_THREAD_PRIORITY_LOW, VLC_FALSE ) )
+        {
+            msg_Err( p_intf, "cannot spawn interface thread" );
+            return VLC_EGENERIC;
+        }
+
+        if( p_intf->b_block )
+        {
+            /* VLC in primary interface mode with a working macosx vout */
+            [NSApp run];
+            p_intf->b_die = VLC_TRUE;
+        }
+    }
+#else
     if( p_intf->b_block )
     {
         /* Run a manager thread, launch the interface, kill the manager */
@@ -151,7 +200,6 @@ int intf_RunThread( intf_thread_t *p_intf )
         RunInterface( p_intf );
 
         p_intf->b_die = VLC_TRUE;
-
         /* Do not join the thread... intf_StopThread will do it for us */
     }
     else
@@ -164,6 +212,7 @@ int intf_RunThread( intf_thread_t *p_intf )
             return VLC_EGENERIC;
         }
     }
+#endif
 
     return VLC_SUCCESS;
 }
@@ -196,8 +245,11 @@ void intf_StopThread( intf_thread_t *p_intf )
  */
 void intf_Destroy( intf_thread_t *p_intf )
 {
-    /* Unlock module */
-    module_Unneed( p_intf, p_intf->p_module );
+    /* Unlock module if present (a switch may have failed) */
+    if( p_intf->p_module )
+    {
+        module_Unneed( p_intf, p_intf->p_module );
+    }
 
     vlc_mutex_destroy( &p_intf->change_lock );
 
@@ -236,6 +288,12 @@ static void Manager( intf_thread_t *p_intf )
         if( p_intf->p_vlc->b_die )
         {
             p_intf->b_die = VLC_TRUE;
+#ifdef SYS_DARWIN
+    if( strncmp( p_intf->p_vlc->psz_object_name, "clivlc", 6 ) )
+    {
+        [NSApp stop: NULL];
+    }
+#endif
             return;
         }
     }
@@ -246,7 +304,18 @@ static void Manager( intf_thread_t *p_intf )
  *****************************************************************************/
 static void RunInterface( intf_thread_t *p_intf )
 {
+    static char *ppsz_interfaces[] =
+    {
+        "skins2", "Skins 2",
+        "wxwindows", "wxWindows",
+        NULL, NULL
+    };
+    char **ppsz_parser;
+
+    vlc_list_t *p_list;
+    int i;
     vlc_value_t val, text;
+    char *psz_intf;
 
     /* Variable used for interface switching */
     p_intf->psz_switch_intf = NULL;
@@ -255,56 +324,77 @@ static void RunInterface( intf_thread_t *p_intf )
     text.psz_string = _("Switch interface");
     var_Change( p_intf, "intf-switch", VLC_VAR_SETTEXT, &text, NULL );
 
-    val.psz_string = "skins"; text.psz_string = "Skins";
-    var_Change( p_intf, "intf-switch", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "skins2"; text.psz_string = "Skins 2";
-    var_Change( p_intf, "intf-switch", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "wxwin"; text.psz_string = "wxWindows";
-    var_Change( p_intf, "intf-switch", VLC_VAR_ADDCHOICE, &val, &text );
+    /* Only fill the list with available modules */
+    p_list = vlc_list_find( p_intf, VLC_OBJECT_MODULE, FIND_ANYWHERE );
+    for( ppsz_parser = ppsz_interfaces; *ppsz_parser; ppsz_parser += 2 )
+    {
+        for( i = 0; i < p_list->i_count; i++ )
+        {
+            module_t *p_module = (module_t *)p_list->p_values[i].p_object;
+            if( !strcmp( p_module->psz_object_name, ppsz_parser[0] ) )
+            {
+                val.psz_string = ppsz_parser[0];
+                text.psz_string = ppsz_parser[1];
+                var_Change( p_intf, "intf-switch", VLC_VAR_ADDCHOICE,
+                            &val, &text );
+                break;
+            }
+        }
+    }
+    vlc_list_release( p_list );
 
     var_AddCallback( p_intf, "intf-switch", SwitchIntfCallback, NULL );
 
     /* Variable used for interface spawning */
     var_Create( p_intf, "intf-add", VLC_VAR_STRING |
                 VLC_VAR_HASCHOICE | VLC_VAR_ISCOMMAND );
-    text.psz_string = _("Add interface");
+    text.psz_string = _("Add Interface");
     var_Change( p_intf, "intf-add", VLC_VAR_SETTEXT, &text, NULL );
 
     val.psz_string = "rc"; text.psz_string = "Console";
     var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "telnet"; text.psz_string = "Telnet Interface";
+    var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
+    val.psz_string = "http"; text.psz_string = "Web Interface";
+    var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
     val.psz_string = "logger"; text.psz_string = "Debug logging";
     var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "http"; text.psz_string = "HTTP remote control";
+    val.psz_string = "sap"; text.psz_string = "SAP Playlist";
     var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "sap"; text.psz_string = "SAP interface";
-    var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
-    val.psz_string = "gestures"; text.psz_string = "Mouse gestures control";
+    val.psz_string = "gestures"; text.psz_string = "Mouse Gestures";
     var_Change( p_intf, "intf-add", VLC_VAR_ADDCHOICE, &val, &text );
 
     var_AddCallback( p_intf, "intf-add", AddIntfCallback, NULL );
 
-    /* Give control to the interface */
-    p_intf->pf_run( p_intf );
-
-    /* Provide ability to switch the main interface on the fly */
-    while( p_intf->psz_switch_intf )
+    do
     {
-        char *psz_intf = p_intf->psz_switch_intf;
-        p_intf->psz_switch_intf = NULL;
-        p_intf->b_die = VLC_FALSE;
+        /* Give control to the interface */
+        p_intf->pf_run( p_intf );
 
-        /* Make sure the old interface is completely uninitialised */
+        /* Reset play on start status */
+        p_intf->b_play = VLC_FALSE;
+
+        if( !p_intf->psz_switch_intf )
+        {
+            break;
+        }
+
+        /* Make sure the old interface is completely uninitialized */
         module_Unneed( p_intf, p_intf->p_module );
 
-        p_intf->p_module = module_Need( p_intf, "interface", psz_intf );
-        free( psz_intf );
+        /* Provide ability to switch the main interface on the fly */
+        psz_intf = p_intf->psz_switch_intf;
+        p_intf->psz_switch_intf = NULL;
 
-        if( p_intf->p_module )
-        {
-            p_intf->pf_run( p_intf );
-        }
-        else break;
+        vlc_mutex_lock( &p_intf->object_lock );
+        p_intf->b_die = VLC_FALSE;
+        p_intf->b_dead = VLC_FALSE;
+        vlc_mutex_unlock( &p_intf->object_lock );
+
+        p_intf->p_module = module_Need( p_intf, "interface", psz_intf, 0 );
+        free( psz_intf );
     }
+    while( p_intf->p_module );
 }
 
 static int SwitchIntfCallback( vlc_object_t *p_this, char const *psz_cmd,
@@ -348,3 +438,37 @@ static int AddIntfCallback( vlc_object_t *p_this, char const *psz_cmd,
 
     return VLC_SUCCESS;
 }
+
+#ifdef SYS_DARWIN
+/*****************************************************************************
+ * VLCApplication implementation 
+ *****************************************************************************/
+@implementation VLCApplication 
+
+- (void)stop: (id)sender
+{
+    NSEvent *o_event;
+    NSAutoreleasePool *o_pool;
+    [super stop:sender];
+
+    o_pool = [[NSAutoreleasePool alloc] init];
+    /* send a dummy event to break out of the event loop */
+    o_event = [NSEvent mouseEventWithType: NSLeftMouseDown
+                location: NSMakePoint( 1, 1 ) modifierFlags: 0
+                timestamp: 1 windowNumber: [[NSApp mainWindow] windowNumber]
+                context: [NSGraphicsContext currentContext] eventNumber: 1
+                clickCount: 1 pressure: 0.0];
+    [NSApp postEvent: o_event atStart: YES];
+    [o_pool release];
+}
+
+- (void)terminate: (id)sender
+{
+    if( [NSApp isRunning] )
+        [NSApp stop:sender];
+    [super terminate: sender];
+}
+
+@end
+#endif
+
