@@ -1,8 +1,8 @@
 /*****************************************************************************
  * variables.c: routines for object variables handling
  *****************************************************************************
- * Copyright (C) 2002 VideoLAN
- * $Id: variables.c,v 1.34 2003/11/05 17:57:29 gbazin Exp $
+ * Copyright (C) 2002-2004 VideoLAN
+ * $Id: variables.c 9119 2004-11-02 23:40:20Z gbazin $
  *
  * Authors: Samuel Hocevar <sam@zoy.org>
  *
@@ -63,20 +63,26 @@ static void DupList( vlc_value_t *p_val )
     int i;
     vlc_list_t *p_list = malloc( sizeof(vlc_list_t) );
 
+    p_list->i_count = p_val->p_list->i_count;
     if( p_val->p_list->i_count )
     {
-        p_list->i_count = p_val->p_list->i_count;
         p_list->p_values = malloc( p_list->i_count * sizeof(vlc_value_t) );
         p_list->pi_types = malloc( p_list->i_count * sizeof(int) );
     }
+    else
+    {
+        p_list->p_values = NULL;
+        p_list->pi_types = NULL;
+    }
 
     for( i = 0; i < p_list->i_count; i++ )
     {
         p_list->p_values[i] = p_val->p_list->p_values[i];
+        p_list->pi_types[i] = p_val->p_list->pi_types[i];
         switch( p_val->p_list->pi_types[i] & VLC_VAR_TYPE )
         {
         case VLC_VAR_STRING:
-            
+
             DupString( &p_list->p_values[i] );
             break;
         default:
@@ -501,11 +507,18 @@ int __var_Change( vlc_object_t *p_this, const char *psz_name,
             {
                 p_var->pf_free( &p_var->choices.p_values[i] );
             }
-            if( p_var->choices.i_count )
-                free( p_var->choices.p_values );
+            for( i = 0 ; i < p_var->choices_text.i_count ; i++ )
+            {
+                if( p_var->choices_text.p_values[i].psz_string )
+                    free( p_var->choices_text.p_values[i].psz_string );
+            }
+            if( p_var->choices.i_count ) free( p_var->choices.p_values );
+            if( p_var->choices_text.i_count ) free( p_var->choices_text.p_values );
 
             p_var->choices.i_count = 0;
             p_var->choices.p_values = NULL;
+            p_var->choices_text.i_count = 0;
+            p_var->choices_text.p_values = NULL;
             p_var->i_default = -1;
             break;
         case VLC_VAR_SETDEFAULT:
@@ -575,6 +588,18 @@ int __var_Change( vlc_object_t *p_this, const char *psz_name,
             break;
         case VLC_VAR_FREELIST:
             FreeList( p_val );
+            if( p_val2 && p_val2->p_list )
+            {
+                for( i = 0; i < p_val2->p_list->i_count; i++ )
+                    if( p_val2->p_list->p_values[i].psz_string )
+                        free( p_val2->p_list->p_values[i].psz_string );
+                if( p_val2->p_list->i_count )
+                {
+                    free( p_val2->p_list->p_values );
+                    free( p_val2->p_list->pi_types );
+                }
+                free( p_val2->p_list );
+            }
             break;
         case VLC_VAR_SETTEXT:
             if( p_var->psz_text ) free( p_var->psz_text );
@@ -614,6 +639,41 @@ int __var_Change( vlc_object_t *p_this, const char *psz_name,
                 }
             }
             break;
+        case VLC_VAR_TRIGGER_CALLBACKS:
+            {
+                /* Deal with callbacks. Tell we're in a callback, release the lock,
+                 * call stored functions, retake the lock. */
+                if( p_var->i_entries )
+                {
+                    int i_var;
+                    int i_entries = p_var->i_entries;
+                    callback_entry_t *p_entries = p_var->p_entries;
+
+                    p_var->b_incallback = VLC_TRUE;
+                    vlc_mutex_unlock( &p_this->var_lock );
+
+                    /* The real calls */
+                    for( ; i_entries-- ; )
+                    {
+                        p_entries[i_entries].pf_callback( p_this, psz_name, p_var->val, p_var->val,
+                                                          p_entries[i_entries].p_data );
+                    }
+
+                    vlc_mutex_lock( &p_this->var_lock );
+
+                    i_var = Lookup( p_this->p_vars, p_this->i_vars, psz_name );
+                    if( i_var < 0 )
+                    {
+                        msg_Err( p_this, "variable %s has disappeared", psz_name );
+                        vlc_mutex_unlock( &p_this->var_lock );
+                        return VLC_ENOVAR;
+                    }
+
+                    p_var = &p_this->p_vars[i_var];
+                    p_var->b_incallback = VLC_FALSE;
+                }
+            }
+            break;
 
         default:
             break;
@@ -1170,6 +1230,35 @@ static int InheritValue( vlc_object_t *p_this, const char *psz_name,
         case VLC_VAR_BOOL:
             p_val->b_bool = config_GetInt( p_this, psz_name );
             break;
+        case VLC_VAR_LIST:
+        {
+            char *psz_orig, *psz_var;
+            vlc_list_t *p_list = malloc(sizeof(vlc_list_t));
+            p_val->p_list = p_list;
+            p_list->i_count = 0;
+
+            psz_var = psz_orig = config_GetPsz( p_this, psz_name );
+            while( psz_var && *psz_var )
+            {
+                char *psz_item = psz_var;
+                vlc_value_t val;
+                while( *psz_var && *psz_var != ',' ) psz_var++;
+                if( *psz_var == ',' )
+                {
+                    *psz_var = '\0';
+                    psz_var++;
+                }
+                val.i_int = strtol( psz_item, NULL, 0 );
+                INSERT_ELEM( p_list->p_values, p_list->i_count,
+                             p_list->i_count, val );
+                /* p_list->i_count is incremented twice by INSERT_ELEM */
+                p_list->i_count--;
+                INSERT_ELEM( p_list->pi_types, p_list->i_count,
+                             p_list->i_count, VLC_VAR_INTEGER );
+            }
+            if( psz_orig ) free( psz_orig );
+            break;
+        }
         default:
             return VLC_ENOOBJ;
             break;
