@@ -1,8 +1,8 @@
 /*****************************************************************************
  * vout_synchro.c : frame dropping routines
  *****************************************************************************
- * Copyright (C) 1999-2001 VideoLAN
- * $Id: vout_synchro.c,v 1.3 2003/06/09 00:33:34 massiot Exp $
+ * Copyright (C) 1999-2004 VideoLAN
+ * $Id: vout_synchro.c 8013 2004-06-22 19:31:54Z fenrir $
  *
  * Authors: Christophe Massiot <massiot@via.ecp.fr>
  *          Samuel Hocevar <sam@via.ecp.fr>
@@ -12,7 +12,7 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -64,7 +64,7 @@
  *    ========================
  * On fast machines, we decode all I's.
  * Otherwise :
- * We can decode an I picture if we simply have enough time to decode it 
+ * We can decode an I picture if we simply have enough time to decode it
  * before displaying :
  *      t0 - t > tauI + DELTA
  *
@@ -99,9 +99,9 @@
 
 #include <vlc/vlc.h>
 #include <vlc/vout.h>
+#include <vlc/input.h>
 
 #include "vout_synchro.h"
-#include "stream_control.h"
 
 /*
  * Local prototypes
@@ -109,6 +109,7 @@
 
 /* Error margins */
 #define DELTA                   (int)(0.075*CLOCK_FREQ)
+#define MAX_VALID_TAU           (int)(0.3*CLOCK_FREQ)
 
 #define DEFAULT_NB_P            5
 #define DEFAULT_NB_B            1
@@ -117,7 +118,7 @@
  * vout_SynchroInit : You know what ?
  *****************************************************************************/
 vout_synchro_t * __vout_SynchroInit( vlc_object_t * p_object,
-                                   vout_thread_t * p_vout, int i_frame_rate )
+                                     int i_frame_rate )
 {
     vout_synchro_t * p_synchro = vlc_object_create( p_object,
                                                   sizeof(vout_synchro_t) );
@@ -138,11 +139,10 @@ vout_synchro_t * __vout_SynchroInit( vlc_object_t * p_object,
     p_synchro->current_pts = mdate() + DEFAULT_PTS_DELAY;
     p_synchro->backward_pts = 0;
     p_synchro->i_current_period = p_synchro->i_backward_period = 0;
-    p_synchro->i_trashed_pic = p_synchro->i_not_chosen_pic = 
+    p_synchro->i_trashed_pic = p_synchro->i_not_chosen_pic =
         p_synchro->i_pic = 0;
 
     p_synchro->i_frame_rate = i_frame_rate;
-    p_synchro->p_vout = p_vout;
 
     return p_synchro;
 }
@@ -168,24 +168,23 @@ void vout_SynchroReset( vout_synchro_t * p_synchro )
 /*****************************************************************************
  * vout_SynchroChoose : Decide whether we will decode a picture or not
  *****************************************************************************/
-vlc_bool_t vout_SynchroChoose( vout_synchro_t * p_synchro, int i_coding_type )
+vlc_bool_t vout_SynchroChoose( vout_synchro_t * p_synchro, int i_coding_type,
+                               int i_render_time )
 {
 #define TAU_PRIME( coding_type )    (p_synchro->p_tau[(coding_type)] \
                                     + (p_synchro->p_tau[(coding_type)] >> 1) \
-                                    + tau_yuv)
+                                    + p_synchro->i_render_time)
 #define S (*p_synchro)
     /* VPAR_SYNCHRO_DEFAULT */
-    mtime_t         now, period, tau_yuv;
+    mtime_t         now, period;
     mtime_t         pts = 0;
     vlc_bool_t      b_decode = 0;
 
     now = mdate();
     period = 1000000 * 1001 / p_synchro->i_frame_rate
-                     * p_synchro->i_current_rate / DEFAULT_RATE;
+                     * p_synchro->i_current_rate / INPUT_RATE_DEFAULT;
 
-    vlc_mutex_lock( &p_synchro->p_vout->change_lock );
-    tau_yuv = p_synchro->p_vout->render_time;
-    vlc_mutex_unlock( &p_synchro->p_vout->change_lock );
+    p_synchro->i_render_time = i_render_time;
 
     switch( i_coding_type )
     {
@@ -318,7 +317,8 @@ void vout_SynchroEnd( vout_synchro_t * p_synchro, int i_coding_type,
         /* If duration too high, something happened (pause ?), so don't
          * take it into account. */
         if( tau < 3 * p_synchro->p_tau[i_coding_type]
-             || !p_synchro->pi_meaningful[i_coding_type] )
+             || ( !p_synchro->pi_meaningful[i_coding_type]
+                   && tau < MAX_VALID_TAU ) )
         {
             /* Mean with average tau, to ensure stability. */
             p_synchro->p_tau[i_coding_type] =
@@ -350,9 +350,9 @@ void vout_SynchroNewPicture( vout_synchro_t * p_synchro, int i_coding_type,
                              mtime_t next_dts, int i_current_rate )
 {
     mtime_t         period = 1000000 * 1001 / p_synchro->i_frame_rate
-                              * i_current_rate / DEFAULT_RATE;
+                              * i_current_rate / INPUT_RATE_DEFAULT;
 #if 0
-    mtime_t         now = mdate(); 
+    mtime_t         now = mdate();
 #endif
     p_synchro->i_current_rate = i_current_rate;
 
@@ -382,7 +382,7 @@ void vout_SynchroNewPicture( vout_synchro_t * p_synchro, int i_coding_type,
               p_synchro->i_n_p,
               p_synchro->p_tau[B_CODING_TYPE],
               p_synchro->i_n_b,
-              p_synchro->p_vout->render_time,
+              p_synchro->i_render_time,
               p_synchro->i_not_chosen_pic,
               p_synchro->i_trashed_pic -
               p_synchro->i_not_chosen_pic,
@@ -426,7 +426,7 @@ void vout_SynchroNewPicture( vout_synchro_t * p_synchro, int i_coding_type,
 
     p_synchro->current_pts += p_synchro->i_current_period
                                         * (period >> 1);
- 
+
 #define PTS_THRESHOLD   (period >> 2)
     if( i_coding_type == B_CODING_TYPE )
     {
@@ -457,7 +457,7 @@ void vout_SynchroNewPicture( vout_synchro_t * p_synchro, int i_coding_type,
 
         if( p_synchro->backward_pts )
         {
-            if( next_dts && 
+            if( next_dts &&
                 (next_dts - p_synchro->backward_pts
                     > PTS_THRESHOLD
               || p_synchro->backward_pts - next_dts
